@@ -8,7 +8,7 @@
 #
 #
 
-sh /usr/share/heartbeat/heartbeat/cim/do_register.sh -u -t  \
--r /usr/share/heartbeat/heartbeat/cim/LinuxHA.reg \
--m /usr/share/heartbeat/heartbeat/cim/LinuxHA.mof
+sh /usr/local/share/heartbeat/heartbeat/cim/do_register.sh -u -t  \
+-r /usr/local/share/heartbeat/heartbeat/cim/LinuxHA.reg \
+-m /usr/local/share/heartbeat/heartbeat/cim/LinuxHA.mof
 
