@@ -31,6 +31,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <libgen.h>
+#include <time.h>
 
 #include <heartbeat.h>
 #include <hb_api.h>
@@ -62,7 +63,6 @@ const char *type       = NULL;
 const char *rsc_id     = NULL;
 const char *dest_uname = NULL;
 const char *attr_value = NULL;
-const char *crm_system_name = "crm_master";
 
 #define OPTARGS	"V?GDQU:u:s:n:v:l:t:i:!r:"
 
@@ -101,8 +101,7 @@ main(int argc, char **argv)
 	};
 #endif
 
-	crm_system_name = basename(argv[0]);
-	crm_log_init(crm_system_name, LOG_ERR, FALSE, FALSE, argc, argv);
+	crm_log_init(basename(argv[0]), LOG_ERR, FALSE, FALSE, argc, argv);
 	
 	if(argc < 2) {
 		usage(crm_system_name, LSB_EXIT_EINVAL);
@@ -134,6 +133,7 @@ main(int argc, char **argv)
 				break;
 			case 'D':
 				DO_DELETE = TRUE;
+				DO_WRITE = FALSE;
 				break;
 			case 'U':
 				crm_debug_2("Option %c => %s", flag, optarg);
@@ -280,6 +280,13 @@ main(int argc, char **argv)
 			fprintf(stderr,"Please specify a node with -U or -u\n");
 			return 1;	
 		}
+		
+		if(DO_WRITE && char2score(attr_value) <= 0) {
+		    if(safe_str_neq(attr_value, "0")) {
+			fprintf(stderr,"%s is an inappropriate value for a failcount.\n", attr_value);
+			return 1;
+		    }
+                }
 	
 		set_name = NULL;
 		attr_name = crm_concat("fail-count", rsc_id, '-');
