@@ -54,9 +54,7 @@
 
 
 /* GMainLoop *mainloop = NULL; */
-const char *crm_system_name = "crm_mon";
-
-#define OPTARGS	"V?i:nrh:cdp:s1wX:"
+#define OPTARGS	"V?i:nrh:cdp:s1wX:oft"
 
 
 void usage(const char *cmd, int exit_status);
@@ -70,6 +68,7 @@ int print_html_status(crm_data_t *cib, const char *filename, gboolean web_cgi);
 void make_daemon(gboolean daemonize, const char *pidfile);
 gboolean mon_timer_popped(gpointer data);
 void mon_update(const HA_Message*, int, int, crm_data_t*,void*);
+void clean_up(int rc);
 
 char *xml_file = NULL;
 char *as_html_file = NULL;
@@ -87,10 +86,40 @@ cib_t *cib_conn = NULL;
 int failed_connections = 0;
 gboolean one_shot = FALSE;
 gboolean has_warnings = FALSE;
+gboolean print_failcount = FALSE;
+gboolean print_operations = FALSE;
+gboolean print_timing = FALSE;
+
+/*
+ * Non-mainloop signal handler.
+ */
+static void
+mon_shutdown_wrapper(int nsig)
+{
+    clean_up(LSB_EXIT_OK);
+}
+
+/*
+ * Mainloop signal handler.
+ */
+static gboolean
+mon_shutdown(int nsig, gpointer unused)
+{
+    clean_up(-1);
+    if (mainloop && g_main_is_running(mainloop)) {
+	g_main_quit(mainloop);
+	
+    } else {
+	exit(LSB_EXIT_OK);
+    }
+    return FALSE;
+}
 
 #if CURSES_ENABLED
 #  define print_as(fmt...) if(as_console) {	\
 		printw(fmt);			\
+		clrtoeol();			\
+		refresh();			\
 	} else {				\
 		fprintf(stdout, fmt);		\
 	}
@@ -113,6 +142,9 @@ main(int argc, char **argv)
 		{"interval", 1, 0, 'i'},
 		{"group-by-node", 0, 0, 'n'},
 		{"inactive", 0, 0, 'r'},
+		{"failcounts", 0, 0, 'f'},		
+		{"operations", 0, 0, 'o'},		
+		{"timing-details", 0, 0, 't'},		
 		{"as-html", 1, 0, 'h'},		
 		{"web-cgi", 0, 0, 'w'},
 		{"simple-status", 0, 0, 's'},
@@ -126,8 +158,7 @@ main(int argc, char **argv)
 	};
 #endif
 	pid_file = crm_strdup("/tmp/ClusterMon.pid");
-	crm_system_name = basename(argv[0]);
-	crm_log_init(crm_system_name, LOG_ERR-1, FALSE, FALSE, 0, NULL);
+	crm_log_init(basename(argv[0]), LOG_ERR-1, FALSE, FALSE, 0, NULL);
 
 	if (strcmp(crm_system_name, "crm_mon.cgi")==0) {
 		web_cgi = TRUE;
@@ -161,7 +192,18 @@ main(int argc, char **argv)
 			case 'd':
 				daemonize = TRUE;
 				break;
+			case 't':
+				print_timing = TRUE;
+				print_operations = TRUE;
+				break;
+			case 'o':
+				print_operations = TRUE;
+				break;
+			case 'f':
+				print_failcount = TRUE;
+				break;
 			case 'p':
+				crm_free(pid_file);
 				pid_file = crm_strdup(optarg);
 				break;
 			case 'X':
@@ -190,6 +232,9 @@ main(int argc, char **argv)
 			case '1':
 				one_shot = TRUE;
 				break;
+			case '?':
+				usage(crm_system_name, LSB_EXIT_OK);
+				break;
 			default:
 				printf("Argument code 0%o (%c) is not (?yet?) supported\n", flag, flag);
 				++argerr;
@@ -207,6 +252,10 @@ main(int argc, char **argv)
 		usage(crm_system_name, LSB_EXIT_GENERIC);
 	}
 
+	/* Set signal callback function. */
+	signal(SIGTERM, mon_shutdown_wrapper);
+	signal(SIGINT, mon_shutdown_wrapper);
+
 	if(as_html_file == NULL && !web_cgi && !simple_status) {
 #if CURSES_ENABLED
 		as_console = TRUE;
@@ -263,7 +312,13 @@ main(int argc, char **argv)
 	}
 
 	mon_timer_popped(NULL);
+
+	G_main_add_SignalHandler(
+		G_PRIORITY_HIGH, SIGTERM, mon_shutdown, NULL, NULL);
+	G_main_add_SignalHandler(
+		G_PRIORITY_HIGH, SIGINT, mon_shutdown, NULL, NULL);
 	g_main_run(mainloop);
+	g_main_destroy(mainloop);
 	return_to_orig_privs();
 	
 	crm_info("Exiting %s", crm_system_name);	
@@ -324,7 +379,8 @@ mon_timer_popped(gpointer data)
 
 		} else if (simple_status || one_shot) {
 			fprintf(stdout, "Critical: Unable to connect to the CIB\n");
-			exit(2);
+			clean_up(LSB_EXIT_GENERIC);
+
 		} else {
 			failed_connections++;
 			CRM_DEV_ASSERT(cib_conn->cmds->signoff(cib_conn) == cib_ok);
@@ -363,30 +419,34 @@ mon_update(const HA_Message *msg, int call_id, int rc,
 		CRM_DEV_ASSERT(safe_str_eq(crm_element_name(cib), XML_TAG_CIB));
 #endif		
 		if(as_html_file || web_cgi) {
-			print_html_status(cib, as_html_file, web_cgi);
+			if (print_html_status(cib, as_html_file, web_cgi) != 0) {
+				fprintf(stderr, "Critical: Unable to output html file\n");
+				clean_up(LSB_EXIT_GENERIC);
+			}
 		} else if (simple_status) {
 			print_simple_status(cib);
 			if (has_warnings) {
-				exit(1); 
+				clean_up(LSB_EXIT_GENERIC);
 			}
 		} else {
 			print_status(cib);
 		}
 		if(one_shot) {
-			exit(LSB_EXIT_OK);
+			clean_up(LSB_EXIT_OK);
 		}
 		
 			
 	} else if(simple_status) {
-		fprintf(stderr, "Critical: query failed: %s", cib_error2string(rc));
-		exit(2);
+		fprintf(stderr, "Critical: query failed: %s\n", cib_error2string(rc));
+		clean_up(LSB_EXIT_GENERIC);
+		
 	} else if(one_shot) {
-		fprintf(stderr, "Query failed: %s", cib_error2string(rc));
-		exit(LSB_EXIT_OK);
+		fprintf(stderr, "Query failed: %s\n", cib_error2string(rc));
+		clean_up(LSB_EXIT_OK);
 
 	} else {
 		CRM_DEV_ASSERT(cib_conn->cmds->signoff(cib_conn) == cib_ok);
-		crm_err("Query failed: %s", cib_error2string(rc));
+		print_as("Query failed: %s", cib_error2string(rc));
 		prefix = "Query failed! ";
 		
 	}
@@ -397,6 +457,7 @@ void
 wait_for_refresh(int offset, const char *prefix, int msec) 
 {
 	int lpc = msec / 1000;
+	struct timespec sleept = {1 , 0};
 
 	if(as_console == FALSE) {
 		timer_id = Gmain_timeout_add(msec, mon_timer_popped, NULL);
@@ -417,7 +478,9 @@ wait_for_refresh(int offset, const char *prefix, int msec)
 			timer_id = Gmain_timeout_add(
 				1000, mon_timer_popped, NULL);
 		} else {
-			sleep(1);
+			if (nanosleep(&sleept, NULL) != 0) {
+				return;
+			}
 		}
 	}
 }
@@ -432,6 +495,198 @@ wait_for_refresh(int offset, const char *prefix, int msec)
 		has_warnings = TRUE;			\
 	} while(0)
 
+static int get_failcount(node_t *node, resource_t *rsc, int *last_failure, pe_working_set_t *data_set) 
+{
+    int fail_count = 0;
+    resource_t *failed = rsc;
+    char *fail_attr = crm_concat("fail-count", rsc->id, '-');
+    const char *value = g_hash_table_lookup(node->details->attrs, fail_attr);
+
+    *last_failure = 0;
+    if(is_not_set(rsc->flags, pe_rsc_unique)) {
+	failed = uber_parent(rsc);
+    }
+    
+    if(value != NULL) {
+	fail_count = char2score(value);
+	crm_info("%s has failed %d times on %s",
+		 rsc->id, fail_count, node->details->uname);
+    }
+    crm_free(fail_attr);
+    return fail_count;
+}
+
+static void get_ping_score(node_t *node, pe_working_set_t *data_set) 
+{
+    const char *attr = "pingd";
+    const char *value = NULL;
+    value = g_hash_table_lookup(node->details->attrs, attr);
+    
+    if(value != NULL) {
+	print_as(" %s=%s", attr, value);
+    }
+}
+
+static void print_date(time_t time) 
+{
+    int lpc = 0;
+    char date_str[26];
+    asctime_r(localtime(&time), date_str);
+    for(; lpc < 26; lpc++) {
+	if(date_str[lpc] == '\n') {
+	    date_str[lpc] = 0;
+	}
+    }
+    print_as("'%s'", date_str);
+}
+
+static void print_rsc_summary(pe_working_set_t *data_set, node_t *node, resource_t *rsc, gboolean all)
+{
+    gboolean printed = FALSE;
+    time_t last_failure = 0;
+    int failcount = get_failcount(node, rsc, (int*)&last_failure, data_set);	
+
+    if(all || failcount || last_failure > 0) {
+	printed = TRUE;
+	print_as("   %s: ", rsc->id);
+    }
+
+    if(failcount > 0) {
+	printed = TRUE;
+	print_as(" fail-count=%d", failcount);
+    }
+    
+    if(last_failure > 0) {
+	printed = TRUE;
+	print_as(" last-failure=");
+	print_date(last_failure);
+    }
+
+    if(printed) {
+	print_as("\n");
+    }
+}
+
+
+static void print_rsc_history(pe_working_set_t *data_set, node_t *node, crm_data_t *rsc_entry)
+{
+    GListPtr op_list = NULL;
+    gboolean print_name = TRUE;
+    GListPtr sorted_op_list = NULL;
+    const char *rsc_id = crm_element_value(rsc_entry, XML_ATTR_ID);
+    resource_t *rsc = pe_find_resource(data_set->resources, rsc_id);
+
+    xml_child_iter_filter(
+	rsc_entry, rsc_op, XML_LRM_TAG_RSC_OP,
+	op_list = g_list_append(op_list, rsc_op);
+	);
+    
+    sorted_op_list = g_list_sort(op_list, sort_op_by_callid);
+    
+    slist_iter(xml_op, crm_data_t, sorted_op_list, lpc,
+	       const char *value = NULL;
+	       const char *task = crm_element_value(xml_op, XML_LRM_ATTR_TASK);
+	       const char *op_rc = crm_element_value(xml_op, XML_LRM_ATTR_RC);
+	       const char *interval = crm_element_value(xml_op, XML_LRM_ATTR_INTERVAL);
+	       int rc = crm_parse_int(op_rc, "0");
+	       
+	       if(safe_str_eq(task, CRMD_ACTION_STATUS)
+		  && safe_str_eq(interval, "0")) {
+		   task = "probe";
+	       }
+	       
+	       if(rc == 7 && safe_str_eq(task, "probe")) {
+		   continue;
+		   
+	       } else if(safe_str_eq(task, CRMD_ACTION_NOTIFY)) {
+		   continue;
+	       }
+	       
+	       if(print_name) {
+		   print_name = FALSE;
+		   print_rsc_summary(data_set, node, rsc, TRUE);
+	       }
+	       
+	       print_as("    + %s:", task);
+	       if(safe_str_neq(interval, "0")) {
+		   print_as(" interval=%sms", interval);
+	       }
+
+	       if(print_timing) {
+		   int int_value;
+		   value = crm_element_value(xml_op, "last_rc_change");
+		   if(value) {
+		       int_value = crm_parse_int(value, NULL);
+		       print_as(" last-rc-change=");
+		       print_date(int_value);
+		   }
+		   
+		   value = crm_element_value(xml_op, "last_run");
+		   if(value) {
+		       int_value = crm_parse_int(value, NULL);
+		       print_as(" last-run=");
+		       print_date(int_value);
+		   }
+		   
+		   value = crm_element_value(xml_op, "exec_time");
+		   if(value) {
+		       int_value = crm_parse_int(value, NULL);
+		       print_as(" exec-time=");
+		       print_date(int_value);
+		   }
+		   
+		   value = crm_element_value(xml_op, "queue_time");
+		   if(value) {
+		       int_value = crm_parse_int(value, NULL);
+		       print_as(" queue-time=");
+		       print_date(int_value);
+		   }
+	       }
+	       
+	       print_as(" rc=%s (%s)\n", op_rc, execra_code2string(rc));
+	       
+	);
+    
+    /* no need to free the contents */
+    g_list_free(sorted_op_list);
+}
+
+static void print_node_summary(pe_working_set_t *data_set, gboolean operations)
+{
+    crm_data_t *lrm_rsc = NULL;
+    crm_data_t *cib_status = get_object_root(XML_CIB_TAG_STATUS, data_set->input);
+
+    if(operations) {
+	print_as("\nOperations:\n");
+    } else {
+	print_as("\nFailcount summary:\n");
+    }
+    
+    xml_child_iter_filter(
+	cib_status, node_state, XML_CIB_TAG_STATE,
+	node_t *node = pe_find_node_id(data_set->nodes, ID(node_state));
+	print_as("* Node %s: ", crm_element_value(node_state, XML_ATTR_UNAME));
+	get_ping_score(node, data_set); 
+	print_as("\n");
+	
+	lrm_rsc = find_xml_node(node_state, XML_CIB_TAG_LRM, FALSE);
+	lrm_rsc = find_xml_node(lrm_rsc, XML_LRM_TAG_RESOURCES, FALSE);
+	
+	xml_child_iter_filter(
+	    lrm_rsc, rsc_entry, XML_LRM_TAG_RESOURCE,
+
+	    if(operations) {
+		print_rsc_history(data_set, node, rsc_entry);
+
+	    } else {
+		const char *rsc_id = crm_element_value(rsc_entry, XML_ATTR_ID);
+		resource_t *rsc = pe_find_resource(data_set->resources, rsc_id);
+		print_rsc_summary(data_set, node, rsc, FALSE);
+	    }
+	    );
+	);
+}
+
 int
 print_simple_status(crm_data_t *cib) 
 {
@@ -451,7 +706,7 @@ print_simple_status(crm_data_t *cib)
 	}
 
 	slist_iter(node, node_t, data_set.nodes, lpc2,
-		   if(node->details->standby) {
+		   if(node->details->standby && node->details->online) {
 			   nodes_standby++;
 		   } else if(node->details->online) {
 			   nodes_online++;
@@ -529,10 +784,14 @@ print_status(crm_data_t *cib)
 
 	slist_iter(node, node_t, data_set.nodes, lpc2,
 		   const char *node_mode = "OFFLINE";
-		   if(node->details->standby) {
-			   node_mode = "standby";
+		   if(node->details->standby && node->details->online) {
+		       node_mode = "standby";
+
+		   } else if(node->details->standby) {
+		       node_mode = "OFFLINE (standby)";
+
 		   } else if(node->details->online) {
-			   node_mode = "online";
+		       node_mode = "online";
 		   }
 		   
 		   print_as("Node: %s (%s): %s\n",
@@ -573,6 +832,10 @@ print_status(crm_data_t *cib)
 			);
 	}
 
+	if(print_operations || print_failcount) {
+	    print_node_summary(&data_set, print_operations);
+	}
+	
 	if(xml_has_children(data_set.failed)) {
 		print_as("\nFailed actions:\n");
 		xml_child_iter(data_set.failed, xml_op, 
@@ -614,8 +877,8 @@ print_html_status(crm_data_t *cib, const char *filename, gboolean web_cgi)
 	} else {
 		filename_tmp = crm_concat(filename, "tmp", '.');
 		stream = fopen(filename_tmp, "w");
-		cl_perror("Cannot open %s for writing", filename_tmp);
 		if(stream == NULL) {
+			cl_perror("Cannot open %s for writing", filename_tmp);
 			crm_free(filename_tmp);
 			return -1;
 		}	
@@ -692,16 +955,22 @@ print_html_status(crm_data_t *cib, const char *filename, gboolean web_cgi)
 	fprintf(stream, "<ul>\n");
 	slist_iter(node, node_t, data_set.nodes, lpc2,
 		   fprintf(stream, "<li>");
-		   fprintf(stream, "Node: %s (%s): %s",
-			     node->details->uname, node->details->id,
-			     node->details->online?"<font color=\"green\">online</font>\n":"<font color=\"orange\"><b>OFFLINE</b></font>\n");
+		   if(node->details->standby && node->details->online) {
+			fprintf(stream, "Node: %s (%s): %s",node->details->uname, node->details->id,"<font color=\"orange\">standby</font>\n");
+		   } else if(node->details->standby) {
+			fprintf(stream, "Node: %s (%s): %s",node->details->uname, node->details->id,"<font color=\"red\">OFFLINE (standby)</font>\n");
+		   } else if(node->details->online) {
+			fprintf(stream, "Node: %s (%s): %s",node->details->uname, node->details->id,"<font color=\"green\">online</font>\n");
+		   } else {
+			 fprintf(stream, "Node: %s (%s): %s",node->details->uname, node->details->id,"<font color=\"red\">OFFLINE</font>\n");
+		   }
 		   if(group_by_node) {
 			   fprintf(stream, "<ul>\n");
 			   slist_iter(rsc, resource_t,
 				      node->details->running_rsc, lpc2,
 				      fprintf(stream, "<li>");
 				      rsc->fns->print(rsc, NULL,
-						      pe_print_html, stream);
+						      pe_print_html|pe_print_rsconly, stream);
 				      fprintf(stream, "</li>\n");
 				   );
 			   fprintf(stream, "</ul>\n");
@@ -752,6 +1021,7 @@ blank_screen(void)
 		clrtoeol();
 	}
 	move(0, 0);
+	refresh();
 #endif
 }
 
@@ -767,8 +1037,10 @@ usage(const char *cmd, int exit_status)
 	fprintf(stream, "\t--%s (-%c) \t: This text\n", "help", '?');
 	fprintf(stream, "\t--%s (-%c) \t: Increase the debug output\n", "verbose", 'V');
 	fprintf(stream, "\t--%s (-%c) <seconds>\t: Update frequency\n", "interval", 'i');
-	fprintf(stream, "\t--%s (-%c) \t:Group resources by node\n", "group-by-node", 'n');
-	fprintf(stream, "\t--%s (-%c) \t:Display inactive resources\n", "inactive", 'r');
+	fprintf(stream, "\t--%s (-%c) \t: Group resources by node\n", "group-by-node", 'n');
+	fprintf(stream, "\t--%s (-%c) \t: Display inactive resources\n", "inactive", 'r');
+	fprintf(stream, "\t--%s (-%c) \t: Display resource fail counts\n", "failcount", 'f');
+	fprintf(stream, "\t--%s (-%c) \t: Display resource operation history\n", "operations", 'o');
 	fprintf(stream, "\t--%s (-%c) \t: Display cluster status on the console\n", "as-console", 'c');
 	fprintf(stream, "\t--%s (-%c) \t: Display the cluster status once as "
 		"a simple one line output (suitable for nagios)\n", "simple-status", 's');
@@ -781,7 +1053,7 @@ usage(const char *cmd, int exit_status)
 
 	fflush(stream);
 
-	exit(exit_status);
+	clean_up(exit_status);
 }
 
 void
@@ -799,16 +1071,16 @@ make_daemon(gboolean daemonize, const char *pidfile)
 		fprintf(stderr, "%s: could not start daemon\n",
 			crm_system_name);
 		perror("fork");
-		exit(LSB_EXIT_GENERIC);
+		clean_up(LSB_EXIT_GENERIC);
 	} else if (pid > 0) {
-		exit(LSB_EXIT_OK);
+		clean_up(LSB_EXIT_OK);
 	}
 	
 	if (cl_lock_pidfile(pidfile) < 0 ){
 		pid = cl_read_pidfile(pidfile);
 		fprintf(stderr, "%s: already running [pid %ld].\n",
 			crm_system_name, pid);
-		exit(LSB_EXIT_OK);
+		clean_up(LSB_EXIT_OK);
 	}
 	
 	umask(022);
@@ -819,3 +1091,33 @@ make_daemon(gboolean daemonize, const char *pidfile)
 	close(FD_STDERR);
 	(void)open(devnull, O_WRONLY);		/* Stderr: fd 2 */
 }
+
+/*
+ * De-init ncurses, signoff from the CIB and deallocate memory.
+ */
+void clean_up(int rc)
+{
+#if CURSES_ENABLED
+    if(as_console) {
+	as_console = FALSE;
+	echo();
+	nocbreak();
+	endwin();
+    }
+#endif
+    
+    if (cib_conn != NULL) {
+	cib_conn->cmds->signoff(cib_conn);
+	cib_delete(cib_conn);
+	cib_conn = NULL;
+    }
+
+    crm_free(as_html_file);
+    crm_free(xml_file);
+    crm_free(pid_file);
+
+    if(rc >= 0) {
+	exit(rc);
+    }
+    return;
+}
