@@ -59,7 +59,7 @@ char *admin_uuid = NULL;
 void usage(const char *cmd, int exit_status);
 ll_cluster_t *do_init(void);
 int do_work(ll_cluster_t * hb_cluster);
-void crmd_ipc_connection_destroy(gpointer user_data);
+void crmadmin_ipc_connection_destroy(gpointer user_data);
 
 gboolean admin_msg_callback(IPC_Channel * source_data, void *private_data);
 char *pluralSection(const char *a_section);
@@ -102,7 +102,6 @@ char *crm_option = NULL;
 
 int operation_status = 0;
 const char *sys_to = NULL;
-const char *crm_system_name = NULL;
 
 #define OPTARGS	"V?K:S:HE:Dd:i:RNqt:Bv"
 
@@ -143,13 +142,10 @@ main(int argc, char **argv)
 	};
 #endif
 
-	crm_system_name = basename(argv[0]);
-
+	crm_log_init(basename(argv[0]), LOG_ERR, FALSE, TRUE, argc, argv);
 	if(argc < 2) {
 		usage(crm_system_name, LSB_EXIT_EINVAL);
 	}
-
-	crm_log_init(crm_system_name, LOG_ERR, FALSE, TRUE, argc, argv);
 	
 	while (1) {
 #ifdef HAVE_GETOPT_H
@@ -164,11 +160,6 @@ main(int argc, char **argv)
 		switch(flag) {
 #ifdef HAVE_GETOPT_H
 			case 0:
-				printf("option %s", long_options[option_index].name);
-				if (optarg)
-					printf(" with arg %s", optarg);
-				printf("\n");
-			
 				if (strcasecmp("reference",
 					   long_options[option_index].name) == 0) {
 					this_msg_reference =
@@ -454,9 +445,14 @@ do_work(ll_cluster_t * hb_cluster)
 }
 
 void
-crmd_ipc_connection_destroy(gpointer user_data)
+crmadmin_ipc_connection_destroy(gpointer user_data)
 {
-	crm_debug("Connection to CRMd was terminated");
+    crm_err("Connection to CRMd was terminated");
+    if(mainloop) {
+	g_main_quit(mainloop);
+    } else {
+	exit(1);
+    }
 }
 
 
@@ -485,7 +481,7 @@ do_init(void)
 		send_hello_message(
 			crmd_channel, admin_uuid, crm_system_name,"0", "1");
 
-		set_IPC_Channel_dnotify(src, crmd_ipc_connection_destroy);
+		set_IPC_Channel_dnotify(src, crmadmin_ipc_connection_destroy);
 		
 		return hb_cluster;
 	} 
@@ -608,8 +604,7 @@ admin_msg_callback(IPC_Channel * server, void *private_data)
 		crm_debug_2(
 		       "Recieved expected number (%d) of messages from Heartbeat."
 		       "  Exiting normally.", expected_responses);
-		g_main_quit(mainloop);
-		return !hack_return_good;
+		exit(0);
 	}
 
 	message_timer_id = Gmain_timeout_add(
