@@ -271,7 +271,7 @@ retrieveCib(const char *filename, const char *sigfile, gboolean archive_invalid)
 	
     } else if(validate_cib_digest(root, sigfile) == FALSE) {
 	crm_err("Checksum of %s failed!  Configuration contents ignored!", filename);
-	crm_err("Usually this is caused by manually changes, "
+	crm_err("Usually this is caused by manual changes, "
 		"please refer to http://linux-ha.org/v2/faq/cib_changes_detected");
 	crm_warn("Continuing but %s will NOT used.", filename);
 	free_xml(root);
@@ -524,6 +524,21 @@ initializeCib(crm_data_t *new_cib)
 	return TRUE;
 }
 
+static void
+sync_file(const char *file) 
+{
+    FILE *syncme = fopen(file, "a");
+    if(syncme == NULL) {
+	cl_perror("Cannot open file %s for syncing", file);
+	return;
+    }
+    
+    if(fsync(fileno(syncme)) < 0) {
+	cl_perror("fsync for %s failed:", file);
+    }
+    fclose(syncme);
+}
+
 int
 archive_file(const char *oldname, const char *newname, const char *ext, gboolean preserve)
 {
@@ -592,6 +607,7 @@ archive_file(const char *oldname, const char *newname, const char *ext, gboolean
 		} else {
 			crm_debug("%s archived as %s", oldname, backup_file);
 		}
+		sync_file(backup_file);
 	}
 	crm_free(backup_file);
 	return rc;
@@ -723,6 +739,7 @@ write_cib_contents(gpointer p)
 
 	    CRM_ASSERT(retrieveCib(CIB_FILENAME, CIB_FILENAME".sig", FALSE) != NULL);
 	    CRM_ASSERT(retrieveCib(CIB_FILENAME".last", CIB_FILENAME".sig.last", FALSE) != NULL);
+	    
 	    crm_debug("Verified CIB archive");	    
 	}
 	
