@@ -297,15 +297,19 @@ crmd_ha_status_callback(
 	crm_data_t *update = NULL;
 	crm_notice("Status update: Node %s now has status [%s]",node,status);
 
+	if(safe_str_eq(status, PINGSTATUS)) {
+	    return;
+	}
+	
 	if(safe_str_eq(status, DEADSTATUS)) {
 		/* this node is taost */
 		update = create_node_state(
-			node, status, XML_BOOLEAN_NO, OFFLINESTATUS,
+			node, DEADSTATUS, XML_BOOLEAN_NO, OFFLINESTATUS,
 			CRMD_STATE_INACTIVE, NULL, TRUE, __FUNCTION__);
 		
-	} else if(safe_str_eq(status, ACTIVESTATUS)) {
+	} else {
 		update = create_node_state(
-			node, status, NULL, NULL, NULL, NULL,
+			node, ACTIVESTATUS, NULL, NULL, NULL, NULL,
 			FALSE, __FUNCTION__);
 	}
 		
