@@ -331,14 +331,6 @@ do_startup(long long action,
 	fsa_cib_conn = cib_new();
 	fsa_lrm_conn = ll_lrm_new(XML_CIB_TAG_LRM);	
 	
-	crm_debug("Init server comms");
-	if(ipc_server == NULL) {
-		ipc_server = crm_strdup(CRM_SYSTEM_CRMD);
-	}
-
-	was_error = init_server_ipc_comms(ipc_server, crmd_client_connect,
-					  default_ipc_connection_destroy);
-
 	/* set up the timers */
 	crm_malloc0(integration_timer, sizeof(fsa_timer_t));
 	crm_malloc0(finalization_timer, sizeof(fsa_timer_t));
@@ -567,6 +559,17 @@ do_started(long long action,
 		return;
 	}
 
+	crm_debug("Init server comms");
+	if(ipc_server == NULL) {
+		ipc_server = crm_strdup(CRM_SYSTEM_CRMD);
+	}
+
+	if(init_server_ipc_comms(ipc_server, crmd_client_connect,
+				 default_ipc_connection_destroy)) {
+	    crm_err("Couldn't start IPC server");
+	    register_fsa_error(C_FSA_INTERNAL, I_ERROR, NULL);
+	}
+
 	crm_info("The local CRM is operational");
 	clear_bit_inplace(fsa_input_register, R_STARTING);
 	register_fsa_input(msg_data->fsa_cause, I_PENDING, NULL);
@@ -624,6 +627,7 @@ config_query_callback(const HA_Message *msg, int call_id, int rc,
 {
 	const char *value = NULL;
 	GHashTable *config_hash = NULL;
+	ha_time_t *now = new_ha_date(TRUE);
 
 	if(rc != cib_ok) {
 		fsa_data_t *msg_data = NULL;
@@ -646,7 +650,7 @@ config_query_callback(const HA_Message *msg, int call_id, int rc,
 
 	unpack_instance_attributes(
 		output, XML_CIB_TAG_PROPSET, NULL, config_hash,
-		CIB_OPTIONS_FIRST, NULL);
+		CIB_OPTIONS_FIRST, now);
 	
 	value = g_hash_table_lookup(config_hash, XML_CONFIG_ATTR_DC_DEADTIME);
 	if(value == NULL) {
