@@ -284,7 +284,8 @@ do_election_count_vote(long long action,
 	} else if(your_node->node_born_on > our_node->node_born_on) {
 		crm_debug("Election pass: born_on");
 		
-	} else if(strcasecmp(fsa_our_uname, vote_from) > 0) {
+	} else if(fsa_our_uname == NULL
+		  || strcasecmp(fsa_our_uname, vote_from) > 0) {
 		crm_debug("Election fail: uname");
 		we_loose = TRUE;
 
