@@ -49,6 +49,8 @@ struct recurring_op_s
 		char *op_key;
 		int   call_id;
 		int   interval;
+		gboolean remove;
+		gboolean cancelled;
 };
 
 char *make_stop_id(const char *rsc, int call_id);
@@ -189,14 +191,42 @@ do_lrm_control(long long action,
 static void
 ghash_print_pending(gpointer key, gpointer value, gpointer user_data) 
 {
-	const char *action = key;
+	const char *stop_id = key;
 	int *log_level = user_data;
-	do_crm_log(*log_level, "Pending action: %s", action);
+	struct recurring_op_s *pending = value;
+	do_crm_log(*log_level, "Pending action: %s (%s)", stop_id, pending->op_key);
+}
+
+static void
+ghash_print_pending_for_rsc(gpointer key, gpointer value, gpointer user_data) 
+{
+	const char *stop_id = key;
+	char *rsc = user_data;
+	struct recurring_op_s *pending = value;
+	if(safe_str_eq(rsc, pending->rsc_id)) {
+	    do_crm_log(LOG_NOTICE, "%sction %s (%s) incomplete at shutdown",
+		       pending->interval==0?"A":"Recurring a", stop_id, pending->op_key);
+	}
+}
+
+static void
+ghash_count_pending(gpointer key, gpointer value, gpointer user_data) 
+{
+	int *counter = user_data;
+	struct recurring_op_s *pending = value;
+
+	if(pending->interval > 0) {
+	    /* Ignore recurring actions in the shutdown calculations */
+	    return;
+	}
+
+	(*counter)++;
 }
 
 gboolean
 verify_stopped(enum crmd_fsa_state cur_state, int log_level)
 {
+	int counter = 0;
 	gboolean rc = TRUE;
 	GListPtr lrm_list = NULL;
 
@@ -206,7 +236,9 @@ verify_stopped(enum crmd_fsa_state cur_state, int log_level)
 		log_level = LOG_ERR;
 	}	
 
-	if(g_hash_table_size(pending_ops) > 0) {
+	g_hash_table_foreach(pending_ops, ghash_count_pending, &counter);
+
+	if(counter > 0) {
 	    rc = FALSE;
 	    do_crm_log(log_level,
 		       "%d pending LRM operations at shutdown%s",
@@ -230,11 +262,13 @@ verify_stopped(enum crmd_fsa_state cur_state, int log_level)
 			continue;
 		}
 		
-		rc = FALSE;
 		crm_err("Resource %s was active at shutdown."
 			"  You may ignore this error if it is unmanaged.",
 			rsc_id);
-		);
+
+		g_hash_table_foreach(
+		    pending_ops, ghash_print_pending_for_rsc, rsc_id);
+	    );
 	
   bail:
 	set_bit_inplace(fsa_input_register, R_SENT_RSC_STOP);
@@ -392,7 +426,7 @@ append_restart_list(crm_data_t *update, lrm_op_t *op, const char *version)
 		return;
 	}
 
-	restart = create_xml_node(NULL, "parameters");
+	restart = create_xml_node(NULL, XML_TAG_PARAMS);
 	slist_iter(param, const char, restart_list, lpc,
 		   int start = len;
 		   value = g_hash_table_lookup(op->params, param);
@@ -535,7 +569,7 @@ build_operation_update(
 			  " %s_%s_%d %d from %s",
 			  op->rsc_id, op->op_type, op->interval, op->call_id,
 			  op->app_name);
-		local_user_data = generate_transition_key(-1, 0, id);
+		local_user_data = generate_transition_key(-1, 0, 0, id);
 		op->user_data = local_user_data;
 		crm_free(id);
 	}
@@ -847,6 +881,7 @@ delete_op_entry(lrm_op_t *op, const char *rsc_id, const char *key, int call_id)
 	 *
 	 * Avoids refreshing the entire LRM section of this host
 	 */
+
 	if(op != NULL) {
 		xml_top = create_xml_node(NULL, XML_LRM_TAG_RSC_OP);
 		crm_xml_add_int(xml_top, XML_LRM_ATTR_CALLID, op->call_id);
@@ -888,7 +923,6 @@ delete_op_entry(lrm_op_t *op, const char *rsc_id, const char *key, int call_id)
 	}
 
  	crm_log_xml_debug_2(xml_top, "op:cancel");
-
  	free_xml(xml_top);
 }
 
@@ -896,62 +930,90 @@ static gboolean
 cancel_op(lrm_rsc_t *rsc, const char *key, int op, gboolean remove)
 {
 	int rc = HA_OK;
+	struct recurring_op_s *pending = NULL;
 
 	CRM_CHECK(op != 0, return FALSE);
 	CRM_CHECK(rsc != NULL, return FALSE);
 	if(key == NULL) {
-		key = "unknown";
+	    key = make_stop_id(rsc->id, op);
 	}
-	
+	pending = g_hash_table_lookup(pending_ops, key);
+
+	if(pending) {
+	    if(remove && pending->remove == FALSE) {
+		pending->remove = TRUE;
+		crm_debug("Scheduling %s for removal", key);
+	    }
+	    
+	    if(pending->cancelled) {
+		crm_debug("Operation %s already cancelled", key);
+		return TRUE;
+	    }
+
+	    pending->cancelled = TRUE;
+
+	} else {
+	    crm_info("No pending op found for %s", key);
+	}
+
 	crm_debug("Cancelling op %d for %s (%s)", op, rsc->id, key);
 
 	rc = rsc->ops->cancel_op(rsc, op);
 	if(rc != HA_OK) {
 		crm_debug("Op %d for %s (%s): Nothing to cancel", op, rsc->id, key);
-		if(key && remove) {
-			delete_op_entry(NULL, rsc->id, key, op);
-		}
-		/* not doing this will block the node from shutting down */
-		g_hash_table_remove(pending_ops, key);
+		/* The caller needs to make sure the entry is
+		 * removed from the pending_ops list
+		 *
+		 * Usually by returning TRUE inside the worker function
+		 * supplied to g_hash_table_foreach_remove()
+		 *
+		 * Not removing the entry from pending_ops will block
+		 * the node from shutting down
+		 */
+		return FALSE;
 	}
 	
 	return TRUE;
 }
 
-const char *cancel_key = NULL;
-gboolean cancel_done = FALSE;
-lrm_rsc_t *cancel_rsc = NULL;
+struct cancel_data 
+{
+	gboolean done;
+	gboolean remove;
+	const char *key;
+	lrm_rsc_t *rsc;
+};
 
-static void
+static gboolean
 cancel_action_by_key(gpointer key, gpointer value, gpointer user_data)
 {
+	struct cancel_data *data = user_data;
 	struct recurring_op_s *op = (struct recurring_op_s*)value;
 	
-	if(safe_str_eq(op->op_key, cancel_key)) {
-		cancel_done = TRUE;
-		cancel_op(cancel_rsc, key, op->call_id, TRUE);
+	if(safe_str_eq(op->op_key, data->key)) {
+	    data->done = TRUE;
+	    if (cancel_op(data->rsc, key, op->call_id, data->remove) == FALSE) {
+		return TRUE;
+	    }
 	}
+	return FALSE;
 }
 
 static gboolean
 cancel_op_key(lrm_rsc_t *rsc, const char *key, gboolean remove)
 {
+	struct cancel_data data;
+
 	CRM_CHECK(rsc != NULL, return FALSE);
-	
-	cancel_key = key;
-	cancel_rsc = rsc;
-	cancel_done = FALSE;
-
 	CRM_CHECK(key != NULL, return FALSE);
-	
-	g_hash_table_foreach(pending_ops, cancel_action_by_key, NULL);
 
-	if(cancel_done == FALSE && remove) {
-		crm_err("No known %s operation to cancel", key);
-		delete_op_entry(NULL, rsc->id, key, 0);
-	}
+	data.key = key;
+	data.rsc = rsc;
+	data.done = FALSE;
+	data.remove = remove;
 	
-	return cancel_done;
+	g_hash_table_foreach_remove(pending_ops, cancel_action_by_key, &data);
+	return data.done;
 }
 
 static lrm_rsc_t *
@@ -1042,6 +1104,7 @@ do_lrm_invoke(long long action,
 	      enum crmd_fsa_input current_input,
 	      fsa_data_t *msg_data)
 {
+	gboolean done = FALSE;
 	gboolean create_rsc = TRUE;
 	const char *crm_op = NULL;
 	const char *from_sys = NULL;
@@ -1093,8 +1156,10 @@ do_lrm_invoke(long long action,
 
 	} else if(safe_str_eq(operation, CRM_OP_PROBED)
 		  || safe_str_eq(crm_op, CRM_OP_REPROBE)) {
+		int cib_options = cib_inhibit_notify;
 		const char *probed = XML_BOOLEAN_TRUE;
 		if(safe_str_eq(crm_op, CRM_OP_REPROBE)) {
+			cib_options = cib_none;
 			probed = XML_BOOLEAN_FALSE;
 		}
 		
@@ -1173,16 +1238,32 @@ do_lrm_invoke(long long action,
 			op_key = generate_op_key(
 				rsc->id,op_task,crm_parse_int(op_interval,"0"));
 
+			crm_debug("PE requested op %s (call=%s) be cancelled",
+				  op_key, call_id?call_id:"NA");
 			call = crm_parse_int(call_id, "0");
 			if(call == 0) {
-				cancel_op_key(rsc, op_key, TRUE);
+			    /* the normal case when the PE cancels a recurring op */
+			    done = cancel_op_key(rsc, op_key, TRUE);
+
 			} else {
-				cancel_op(rsc, op_key, call, TRUE);
+			    /* the normal case when the PE cancels an orphan op */
+			    done = cancel_op(rsc, NULL, call, TRUE);
 			}
-			
-			op->op_status = LRM_OP_DONE;
+
+			if(done == FALSE) {
+			    crm_debug("Nothing known about operation %d for %s", call, op_key);
+			    delete_op_entry(NULL, rsc->id, op_key, call);
+
+			    /* needed?? surely not otherwise the cancel_op_(_key) wouldn't
+			     * have failed in the first place
+			     */
+			    g_hash_table_remove(pending_ops, op_key);
+			}
+
 			op->rc = EXECRA_OK;
+			op->op_status = LRM_OP_DONE;
 			send_direct_ack(from_host, from_sys, op, rsc->id);
+			
 			crm_free(op_key);
 			free_lrm_op(op);			
 			
@@ -1400,15 +1481,19 @@ send_direct_ack(const char *to_host, const char *to_sys,
 	free_xml(update);
 }
 
-static void
+static gboolean
 stop_recurring_action_by_rsc(gpointer key, gpointer value, gpointer user_data)
 {
 	lrm_rsc_t *rsc = user_data;
 	struct recurring_op_s *op = (struct recurring_op_s*)value;
 	
 	if(op->interval != 0 && safe_str_eq(op->rsc_id, rsc->id)) {
-		cancel_op(rsc, key, op->call_id, FALSE);
+		if (cancel_op(rsc, key, op->call_id, FALSE) == FALSE) {
+			return TRUE;
+		}
 	}
+
+	return FALSE;
 }
 
 void
@@ -1436,8 +1521,10 @@ do_lrm_rsc_op(lrm_rsc_t *rsc, const char *operation,
 
 	/* stop the monitor before stopping the resource */
 	if(crm_str_eq(operation, CRMD_ACTION_STOP, TRUE)
+	   || crm_str_eq(operation, CRMD_ACTION_DEMOTE, TRUE)
+	   || crm_str_eq(operation, CRMD_ACTION_PROMOTE, TRUE)
 	   || crm_str_eq(operation, CRMD_ACTION_MIGRATE, TRUE)) {
-		g_hash_table_foreach(pending_ops, stop_recurring_action_by_rsc, rsc);
+		g_hash_table_foreach_remove(pending_ops, stop_recurring_action_by_rsc, rsc);
 	}
 	
 	/* now do the op */
@@ -1608,12 +1695,16 @@ static void
 cib_rsc_callback(const HA_Message *msg, int call_id, int rc,
 		 crm_data_t *output, void *user_data)
 {
-	if(rc != cib_ok && rc != cib_diff_resync) {
-		crm_err("Resource update %d failed: %s",
-			call_id, cib_error2string(rc));	
-	} else {
-		crm_debug("Resource update %d complete: rc=%d", call_id, rc);
-	}
+    switch(rc) {
+	case cib_ok:
+	case cib_diff_failed:
+	case cib_diff_resync:
+	    crm_debug("Resource update %d complete: rc=%d", call_id, rc);
+	    break;
+	default:
+	    crm_err("Resource update %d failed: (rc=%d) %s",
+		    call_id, rc, cib_error2string(rc));
+    }
 }
 
 
@@ -1713,6 +1804,7 @@ process_lrm_event(lrm_op_t *op)
 	char *op_id = NULL;
 	char *op_key = NULL;
 	int log_level = LOG_ERR;
+	struct recurring_op_s *pending = NULL;
 	CRM_CHECK(op != NULL, return FALSE);
 	CRM_CHECK(op->rsc_id != NULL, return FALSE);
 
@@ -1756,6 +1848,9 @@ process_lrm_event(lrm_op_t *op)
 		crm_info("Result: %s", op->output);
 	}
 	
+	op_id = make_stop_id(op->rsc_id, op->call_id);
+	pending = g_hash_table_lookup(pending_ops, op_id);
+
 	if(op->op_status != LRM_OP_CANCELLED) {
 		do_update_resource(op);
 		if(op->interval != 0) {
@@ -1766,20 +1861,23 @@ process_lrm_event(lrm_op_t *op)
 		/* no known valid reason for this to happen */
 		crm_err("Op %s (call=%d): Cancelled", op_key, op->call_id);
 
-	} else if(op->user_data != NULL) {
-		delete_op_entry(op, NULL, NULL, op->call_id);
-		
-	} else {
-		crm_err("Op %s (call=%d): No user data", op_key, op->call_id);
-	}	
+	} else if(pending == NULL) {
+		crm_err("Op %s (call=%d): No 'pending' entry",
+			op_key, op->call_id);
 
-	op_id = make_stop_id(op->rsc_id, op->call_id);
-	if(g_hash_table_remove(pending_ops, op_id)) {
-		crm_debug("Op %s (call=%d): Confirmed", op_key, op->call_id);
-		goto out;
+	} else if(op->user_data == NULL) {
+		crm_err("Op %s (call=%d): No user data", op_key, op->call_id);
+	    
+	} else if(pending->remove) {
+		delete_op_entry(op, op->rsc_id, op_key, op->call_id);
+
+	} else {
+		crm_debug("Op %s (call=%d): no delete event required", op_key, op->call_id);
 	}
 
-	crm_err("Op %s (call=%d): Not matched", op_key, op->call_id);
+	if(g_hash_table_remove(pending_ops, op_id)) {
+	    crm_debug("Op %s (call=%d, stop_id=%s): Confirmed", op_key, op->call_id, op_id);
+	}
 
   out:
 	crm_free(op_key);
