@@ -527,7 +527,7 @@ relay_message(HA_Message *relay_message, gboolean originated_locally)
 			is_local = 1;
 		}
 			
-	} else if(strcasecmp(fsa_our_uname, host_to) == 0) {
+	} else if(safe_str_eq(fsa_our_uname, host_to)) {
 		is_local=1;
 	}
 
@@ -650,10 +650,11 @@ crmd_authorize_message(ha_msg_input_t *client_msg, crmd_client_t *curr_client)
 		crm_free(minor_version);
 	}
 
-	if (strcasecmp(CRM_SYSTEM_PENGINE, client_name) == 0) {
+	
+	if (safe_str_eq(CRM_SYSTEM_PENGINE, client_name)) {
 		the_subsystem = pe_subsystem;
 		
-	} else if (strcasecmp(CRM_SYSTEM_TENGINE, client_name) == 0) {
+	} else if (safe_str_eq(CRM_SYSTEM_TENGINE, client_name)) {
 		the_subsystem = te_subsystem;
 	}
 	
