@@ -185,19 +185,22 @@ check_action_definition(resource_t *rsc, node_t *active_node, crm_data_t *xml_op
 		if(op_match == NULL && data_set->stop_action_orphans) {
 			/* create a cancel action */
 			action_t *cancel = NULL;
-			char *cancel_key = NULL;
+			char *cancel_key = crm_strdup(key);
 			const char *call_id = crm_element_value(xml_op, XML_LRM_ATTR_CALLID);
 			
 			crm_info("Orphan action will be stopped: %s on %s",
 				 key, active_node->details->uname);
 
-			cancel_key = generate_op_key(
-				rsc->id, CRMD_ACTION_CANCEL, interval);
+			/* cancel_key = generate_op_key( */
+			/* 	rsc->id, CRMD_ACTION_CANCEL, interval); */
 
 			cancel = custom_action(
 				rsc, cancel_key, CRMD_ACTION_CANCEL,
 				active_node, FALSE, TRUE, data_set);
 
+			crm_free(cancel->task);
+			cancel->task = crm_strdup(CRMD_ACTION_CANCEL);
+			
 			add_hash_param(cancel->meta, XML_LRM_ATTR_TASK,     task);
 			add_hash_param(cancel->meta, XML_LRM_ATTR_CALLID,   call_id);
 			add_hash_param(cancel->meta, XML_LRM_ATTR_INTERVAL, interval_s);
@@ -463,7 +466,7 @@ stage0(pe_working_set_t *data_set)
  * Check nodes for resources started outside of the LRM
  */
 gboolean
-stage1(pe_working_set_t *data_set)
+probe_resources(pe_working_set_t *data_set)
 {
 	action_t *probe_complete = NULL;
 	action_t *probe_node_complete = NULL;
@@ -482,13 +485,7 @@ stage1(pe_working_set_t *data_set)
 			continue;
 
 		} else if(probe_complete == NULL) {
-			probe_complete = custom_action(
-				NULL, crm_strdup(CRM_OP_PROBED),
-				CRM_OP_PROBED, NULL, FALSE, TRUE,
-				data_set);
-
-			probe_complete->pseudo = TRUE;
-			probe_complete->optional = TRUE;
+			probe_complete = get_pseudo_op(CRM_OP_PROBED, data_set);
 		}
 
 		if(probed != NULL && crm_is_true(probed) == FALSE) {
@@ -516,6 +513,7 @@ stage1(pe_working_set_t *data_set)
 
 				probe_complete->optional = FALSE;
 				probe_node_complete->optional = FALSE;
+
 				custom_action_order(
 					NULL, NULL, probe_complete,
 					rsc, start_key(rsc), NULL,
@@ -590,9 +588,15 @@ stage5(pe_working_set_t *data_set)
 	slist_iter(
 		rsc, resource_t, data_set->resources, lpc,
 		rsc->cmds->color(rsc, data_set);
+		);
+
+	probe_resources(data_set);
+	
+	slist_iter(
+		rsc, resource_t, data_set->resources, lpc,
 		rsc->cmds->create_actions(rsc, data_set);	
 		);
-	
+
 	return TRUE;
 }
 
@@ -731,7 +735,7 @@ stage7(pe_working_set_t *data_set)
 		order, order_constraint_t, data_set->ordering_constraints, lpc,
 
 		resource_t *rsc = order->lh_rsc;
-		crm_debug_2("Applying ordering constraint: %d", order->id);
+		crm_debug_3("Applying ordering constraint: %d", order->id);
 		
 		if(rsc != NULL) {
 			crm_debug_4("rsc_action-to-*");
@@ -781,6 +785,15 @@ stage8(pe_working_set_t *data_set)
 	value = pe_pref(data_set->config_hash, "cluster-delay");
 	crm_xml_add(data_set->graph, "cluster-delay", value);
 
+	crm_xml_add(data_set->graph, "failed-stop-offset", "INFINITY");
+
+	value = pe_pref(data_set->config_hash, "start-failure-is-fatal");
+	if(crm_is_true(value)) {
+	    crm_xml_add(data_set->graph, "failed-start-offset", "INFINITY");
+	} else {
+	    crm_xml_add(data_set->graph, "failed-start-offset", "1");
+	}
+	
 	value = pe_pref(data_set->config_hash, "batch-limit");
 	crm_xml_add(data_set->graph, "batch-limit", value);
 
