@@ -50,15 +50,16 @@ gboolean
 update_action(action_t *action)
 {
 	int local_type = 0;
-	int log_level = LOG_DEBUG_2;
+	int default_log_level = LOG_DEBUG_3;
+	int log_level = default_log_level;
 	gboolean changed = FALSE;
-	
+
 	do_crm_log(log_level, "Processing action %s: %s %s %s",
 		   action->uuid,
 		   action->optional?"optional":"required",
 		   action->runnable?"runnable":"unrunnable",
 		   action->pseudo?"pseudo":action->task);
-
+	
 	slist_iter(
 		other, action_wrapper_t, action->actions_before, lpc,
 
@@ -70,7 +71,18 @@ update_action(action_t *action)
 		if(other_rsc) {
 		    other_role = other_rsc->fns->state(other_rsc, TRUE);
 		}
-		
+
+		if(other->type & pe_order_test) {
+		    log_level = LOG_NOTICE;
+		    do_crm_log(log_level, "Processing action %s: %s %s %s",
+			       action->uuid,
+			       action->optional?"optional":"required",
+			       action->runnable?"runnable":"unrunnable",
+			       action->pseudo?"pseudo":action->task);
+		} else {
+		    log_level = default_log_level;
+		}
+
 		do_crm_log(log_level, "   Checking action %s: %s %s %s (flags=0x%.6x)",
 			   other->action->uuid,
 			   other->action->optional?"optional":"required",
@@ -80,6 +92,15 @@ update_action(action_t *action)
 
 		local_type = other->type;
 
+		if((local_type & pe_order_demote_stop)
+		   && other->action->pseudo == FALSE
+		   && other_role > RSC_ROLE_SLAVE
+		   && node != NULL
+		   && node->details->online) {
+		    local_type |= pe_order_implies_left;
+		    do_crm_log(log_level,"Upgrading demote->stop constraint to implies_left");
+		}
+
 		if((local_type & pe_order_demote)
 		   && other->action->pseudo == FALSE
 		   && other_role > RSC_ROLE_SLAVE
@@ -89,7 +110,24 @@ update_action(action_t *action)
 		    do_crm_log(log_level,"Upgrading restart constraint to runnable_left");
 		}
 
+		if((local_type & pe_order_complex_right)
+		   && action->optional
+		   && other->action->optional == FALSE
+		   && (local_type ^ pe_order_complex_right) != pe_order_optional) {
+		    local_type |= pe_order_implies_right;
+		    do_crm_log(log_level,"Upgrading complex constraint to implies_right");
+		}
+
+		if((local_type & pe_order_complex_left)
+		   && action->optional == FALSE
+		   && other->action->optional
+		   && (local_type ^ pe_order_complex_left) != pe_order_optional) {
+		    local_type |= pe_order_implies_left;
+		    do_crm_log(log_level,"Upgrading complex constraint to implies_left");
+		}
+		
 		if((local_type & pe_order_shutdown)
+		   && action->optional
 		   && other->action->optional == FALSE
 		   && is_set(other_rsc->flags, pe_rsc_shutdown)) {
 		    action->optional = FALSE;
@@ -152,7 +190,7 @@ update_action(action_t *action)
 				other_changed = TRUE;
 			}
 		}		
-		
+
 		if(local_type & pe_order_implies_left) {
 			if(other->action->optional == FALSE) {
 				/* nothing to do */
@@ -180,10 +218,25 @@ update_action(action_t *action)
 			}
 		}
 		
+		if(local_type & pe_order_implies_left_printed) {
+		    if(other->action->optional == TRUE
+		       && other->action->print_always == FALSE) {
+			if(action->optional == FALSE
+			   || (other->action->pseudo && action->print_always)) {
+			    other_changed = TRUE;
+			    other->action->print_always = TRUE;
+			    do_crm_log(log_level-1,
+				       "   * (implies left) Ensuring action %s is included because of %s",
+				       other->action->uuid, action->uuid);
+			}
+		    }
+		}
+
 		if(local_type & pe_order_implies_right) {
 			if(action->optional == FALSE) {
 				/* nothing to do */
 				do_crm_log(log_level+1, "      Ignoring implies right - redundant");
+
 			} else if(other->action->optional == FALSE) {
 				action->optional = FALSE;
 				do_crm_log(log_level-1,
@@ -196,6 +249,20 @@ update_action(action_t *action)
 			}
 		}
 
+		if(local_type & pe_order_implies_right_printed) {
+		    if(action->optional == TRUE
+		       && action->print_always == FALSE) {
+			if(other->action->optional == FALSE
+			   || (action->pseudo && other->action->print_always)) {
+			    changed = TRUE;
+			    action->print_always = TRUE;
+			    do_crm_log(log_level-1,
+				       "   * (implies right) Ensuring action %s is included because of %s",
+				       action->uuid, other->action->uuid);
+			}
+		    }
+		}
+
 		if(other_changed) {
 			do_crm_log(log_level, "%s changed, processing after list", other->action->uuid);
 			update_action(other->action);
@@ -204,17 +271,29 @@ update_action(action_t *action)
 				do_crm_log(log_level, "%s changed, processing %s", other->action->uuid, before_other->action->uuid);
 				update_action(before_other->action);
 				);
+			slist_iter(
+				before_other, action_wrapper_t, other->action->actions_before, lpc2,
+				do_crm_log(log_level, "%s changed, processing %s", other->action->uuid, before_other->action->uuid);
+				update_action(before_other->action);
+				);
 		}
 		
 		);
 
 	if(changed) {
+		update_action(action);
 		do_crm_log(log_level, "%s changed, processing after list", action->uuid);
 		slist_iter(
 			other, action_wrapper_t, action->actions_after, lpc,
 			do_crm_log(log_level, "%s changed, processing %s", action->uuid, other->action->uuid);
 			update_action(other->action);
 			);
+		do_crm_log(log_level, "%s changed, processing before list", action->uuid);
+		slist_iter(
+			other, action_wrapper_t, action->actions_before, lpc,
+			do_crm_log(log_level, "%s changed, processing %s", action->uuid, other->action->uuid);
+			update_action(other->action);
+			);
 	}
 	
 	return FALSE;
@@ -428,37 +507,32 @@ action2xml(action_t *action, gboolean as_input)
 static gboolean
 should_dump_action(action_t *action) 
 {
-	const char * interval = NULL;
+	int log_filter = LOG_DEBUG_5;
 
 	CRM_CHECK(action != NULL, return FALSE);
 
-	interval = g_hash_table_lookup(action->meta, XML_LRM_ATTR_INTERVAL);
-	if(action->optional) {
-		crm_debug_5("action %d (%s) was optional",
+	if(action->dumped) {
+		do_crm_log(log_filter, "action %d (%s) was already dumped",
 			    action->id, action->uuid);
 		return FALSE;
 
 	} else if(action->runnable == FALSE) {
-		crm_debug_5("action %d (%s) was not runnable",
+		do_crm_log(log_filter, "action %d (%s) was not runnable",
 			    action->id, action->uuid);
 		return FALSE;
 
-	} else if(action->dumped) {
-		crm_debug_5("action %d (%s) was already dumped",
+	} else if(action->optional && action->print_always == FALSE) {
+		do_crm_log(log_filter, "action %d (%s) was optional",
 			    action->id, action->uuid);
 		return FALSE;
 
 	} else if(action->rsc != NULL
 		  && is_not_set(action->rsc->flags, pe_rsc_managed)) {
+		const char * interval = NULL;
+		interval = g_hash_table_lookup(action->meta, XML_LRM_ATTR_INTERVAL);
 
-		/* make sure probes go through */
-		if(safe_str_neq(action->task, CRMD_ACTION_STATUS)) {
-			pe_warn("action %d (%s) was for an unmanaged resource (%s)",
-				action->id, action->uuid, action->rsc->id);
-			return FALSE;
-		}
-		
-		if(interval != NULL && safe_str_neq(interval, "0")) {
+		/* make sure probes and recurring monitors go through */
+		if(safe_str_neq(action->task, CRMD_ACTION_STATUS) && interval == NULL) {
 			pe_warn("action %d (%s) was for an unmanaged resource (%s)",
 				action->id, action->uuid, action->rsc->id);
 			return FALSE;
@@ -519,49 +593,77 @@ static gint sort_action_id(gconstpointer a, gconstpointer b)
 static gboolean
 should_dump_input(int last_action, action_t *action, action_wrapper_t *wrapper)
 {
+    int type = wrapper->type;
+    int log_dump = LOG_DEBUG_3;
+    int log_filter = LOG_DEBUG_3;
+    
+    type &= ~pe_order_implies_left_printed;
+    type &= ~pe_order_implies_right_printed;
+    type &= ~pe_order_optional;
+    
     wrapper->state = pe_link_not_dumped;	
     if(last_action == wrapper->action->id) {
-	crm_debug_2("Input (%d) %s duplicated",
-		    wrapper->action->id,
-		    wrapper->action->uuid);
+	do_crm_log(log_filter, "Input (%d) %s duplicated for %s",
+		   wrapper->action->id, wrapper->action->uuid, action->uuid);
 	wrapper->state = pe_link_dup;
 	return FALSE;
 	
     } else if(wrapper->type == pe_order_none) {
-	crm_debug_2("Input (%d) %s suppressed",
-		    wrapper->action->id,
-		    wrapper->action->uuid);
+	do_crm_log(log_filter, "Input (%d) %s suppressed for %s",
+		   wrapper->action->id, wrapper->action->uuid, action->uuid);
 	return FALSE;
-	
-    } else if(wrapper->action->optional == TRUE) {
-	crm_debug_2("Input (%d) %s optional",
-		    wrapper->action->id,
-		    wrapper->action->uuid);
-	return FALSE;
-	
+
     } else if(wrapper->action->runnable == FALSE
-	      && wrapper->action->pseudo == FALSE
-	      && wrapper->type == pe_order_optional) {
-	crm_debug("Input (%d) %s optional (ordering)",
-		  wrapper->action->id,
-		  wrapper->action->uuid);
+	      && type == pe_order_none) {
+	do_crm_log(log_filter, "Input (%d) %s optional (ordering) for %s",
+		   wrapper->action->id, wrapper->action->uuid, action->uuid);
 	return FALSE;
 
     } else if(action->pseudo
 	      && (wrapper->type & pe_order_stonith_stop)) {
-	crm_debug("Input (%d) %s suppressed",
-		  wrapper->action->id,
-		  wrapper->action->uuid);
+	do_crm_log(log_filter, "Input (%d) %s suppressed for %s",
+		   wrapper->action->id, wrapper->action->uuid, action->uuid);
 	return FALSE;
-    }
-    crm_debug_3("Input (%d) %s n=%p p=%d r=%d f=0x%.6x dumped for %s",
-		wrapper->action->id,
-		wrapper->action->uuid,
-		wrapper->action->node,
-		wrapper->action->pseudo,
-		wrapper->action->runnable,      
-		wrapper->type,      
-		action->uuid);
+
+    } else if(wrapper->action->dumped || should_dump_action(wrapper->action)) {
+	do_crm_log(log_dump, "Input (%d) %s should be dumped for %s",
+		   wrapper->action->id, wrapper->action->uuid, action->uuid);
+	goto dump;
+
+#if 0
+    } if(wrapper->action->runnable
+	 && wrapper->action->pseudo
+	 && wrapper->action->rsc->variant != pe_native) {
+	do_crm_log(LOG_CRIT, "Input (%d) %s should be dumped for %s",
+		   wrapper->action->id, wrapper->action->uuid, action->uuid);
+	goto dump;
+#endif
+    } else if(wrapper->action->optional == TRUE && wrapper->action->print_always == FALSE) {
+	do_crm_log(log_filter, "Input (%d) %s optional for %s",
+		   wrapper->action->id, wrapper->action->uuid, action->uuid);
+    do_crm_log(log_filter, "Input (%d) %s n=%p p=%d r=%d o=%d a=%d f=0x%.6x",
+	       wrapper->action->id,
+	       wrapper->action->uuid,
+	       wrapper->action->node,
+	       wrapper->action->pseudo,
+	       wrapper->action->runnable,
+	       wrapper->action->optional,
+	       wrapper->action->print_always,
+	       wrapper->type);
+	return FALSE;
+    } 
+	
+  dump:
+    do_crm_log(log_dump, "Input (%d) %s n=%p p=%d r=%d o=%d a=%d f=0x%.6x dumped for %s",
+	       wrapper->action->id,
+	       wrapper->action->uuid,
+	       wrapper->action->node,
+	       wrapper->action->pseudo,
+	       wrapper->action->runnable,      
+	       wrapper->action->optional,
+	       wrapper->action->print_always,
+	       wrapper->type,      
+	       action->uuid);
     return TRUE;
 }
 		   
