@@ -27,6 +27,7 @@
 #include <lib/crm/pengine/variant.h>
 
 extern gint sort_clone_instance(gconstpointer a, gconstpointer b);
+void node_list_update_one(GListPtr list1, node_t *other, int score);
 
 extern void clone_create_notifications(
 	resource_t *rsc, action_t *action, action_t *action_complete,
@@ -39,10 +40,6 @@ child_promoting_constraints(
 	clone_variant_data_t *clone_data, enum pe_ordering type,
 	resource_t *rsc, resource_t *child, resource_t *last, pe_working_set_t *data_set)
 {
-/* 	if(clone_data->ordered */
-/* 	   || clone_data->self->restart_type == pe_restart_restart) { */
-/* 		type = pe_order_implies_left; */
-/* 	} */
 	if(child == NULL) {
 		if(clone_data->ordered && last != NULL) {
 			crm_debug_4("Ordered version (last node)");
@@ -52,8 +49,22 @@ child_promoting_constraints(
 				rsc, promoted_key(rsc), NULL,
 				type, data_set);
 		}
-		
-	} else if(clone_data->ordered) {
+		return;
+	}
+
+	/* child promote before global promoted */
+	custom_action_order(
+	    child, promote_key(child), NULL,
+	    rsc, promoted_key(rsc), NULL,
+	    type, data_set);
+	
+	/* global promote before child promote */
+	custom_action_order(
+	    rsc, promote_key(rsc), NULL,
+	    child, promote_key(child), NULL,
+	    type, data_set);
+
+	if(clone_data->ordered) {
 		crm_debug_4("Ordered version");
 		if(last == NULL) {
 			/* global promote before first child promote */
@@ -69,19 +80,6 @@ child_promoting_constraints(
 
 	} else {
 		crm_debug_4("Un-ordered version");
-		
-		/* child promote before global promoted */
-		custom_action_order(
-			child, promote_key(child), NULL,
-			rsc, promoted_key(rsc), NULL,
-			type, data_set);
-                
-		/* global promote before child promote */
-		custom_action_order(
-			rsc, promote_key(rsc), NULL,
-			child, promote_key(child), NULL,
-			type, data_set);
-
 	}
 }
 
@@ -90,11 +88,6 @@ child_demoting_constraints(
 	clone_variant_data_t *clone_data, enum pe_ordering type,
 	resource_t *rsc, resource_t *child, resource_t *last, pe_working_set_t *data_set)
 {
-/* 	if(clone_data->ordered */
-/* 	   || clone_data->self->restart_type == pe_restart_restart) { */
-/* 		type = pe_order_implies_left; */
-/* 	} */
-	
 	if(child == NULL) {
 		if(clone_data->ordered && last != NULL) {
 			crm_debug_4("Ordered version (last node)");
@@ -104,8 +97,22 @@ child_demoting_constraints(
 				last, demote_key(last), NULL,
 				pe_order_implies_left, data_set);
 		}
-		
-	} else if(clone_data->ordered && last != NULL) {
+		return;
+	}
+	
+	/* child demote before global demoted */
+	custom_action_order(
+	    child, demote_key(child), NULL,
+	    rsc, demoted_key(rsc), NULL,
+	    pe_order_implies_right_printed, data_set);
+	
+	/* global demote before child demote */
+	custom_action_order(
+	    rsc, demote_key(rsc), NULL,
+	    child, demote_key(child), NULL,
+	    pe_order_implies_left_printed, data_set);
+	
+	if(clone_data->ordered && last != NULL) {
 		crm_debug_4("Ordered version");
 
 		/* child/child relative demote */
@@ -123,30 +130,25 @@ child_demoting_constraints(
 
 	} else {
 		crm_debug_4("Un-ordered version");
-
-		/* child demote before global demoted */
-		custom_action_order(
-			child, demote_key(child), NULL,
-			rsc, demoted_key(rsc), NULL,
-			type, data_set);
-                        
-		/* global demote before child demote */
-		custom_action_order(
-			rsc, demote_key(rsc), NULL,
-			child, demote_key(child), NULL,
-			type, data_set);
 	}
 }
 
 static void
 master_update_pseudo_status(
-	resource_t *child, gboolean *demoting, gboolean *promoting) 
-{
+	resource_t *rsc, gboolean *demoting, gboolean *promoting) 
+{	
+	if(rsc->children) {
+	    slist_iter(child, resource_t, rsc->children, lpc,
+		       master_update_pseudo_status(child, demoting, promoting)
+		);
+	    return;
+	}
+    
 	CRM_ASSERT(demoting != NULL);
 	CRM_ASSERT(promoting != NULL);
 
 	slist_iter(
-		action, action_t, child->actions, lpc,
+		action, action_t, rsc->actions, lpc,
 
 		if(*promoting && *demoting) {
 			return;
@@ -169,7 +171,7 @@ master_update_pseudo_status(
 		cons, rsc_to_node_t, list, lpc2,			\
 		cons_node = NULL;					\
 		if(cons->role_filter == RSC_ROLE_MASTER) {		\
-			crm_debug("Applying %s to %s",			\
+			crm_debug_2("Applying %s to %s",			\
 				  cons->id, child_rsc->id);		\
 			cons_node = pe_find_node_id(			\
 				cons->node_list_rh, chosen->details->id); \
@@ -177,7 +179,7 @@ master_update_pseudo_status(
 		if(cons_node != NULL) {					\
 			int new_priority = merge_weights(		\
 				child_rsc->priority, cons_node->weight); \
-			crm_debug("\t%s: %d->%d (%d)", child_rsc->id,	\
+			crm_debug_2("\t%s: %d->%d (%d)", child_rsc->id,	\
 				child_rsc->priority, new_priority, cons_node->weight);	\
 			child_rsc->priority = new_priority;		\
 		}							\
@@ -188,8 +190,24 @@ can_be_master(resource_t *rsc)
 {
 	node_t *node = NULL;
 	node_t *local_node = NULL;
+	resource_t *parent = uber_parent(rsc);
 	clone_variant_data_t *clone_data = NULL;
 	int level = LOG_DEBUG_2;
+#if 0
+	enum rsc_role_e role = RSC_ROLE_UNKNOWN;
+	role = rsc->fns->state(rsc, FALSE);
+	crm_info("%s role: %s", rsc->id, role2text(role));
+#endif
+	
+	if(rsc->children) {
+	    slist_iter(
+		child, resource_t, rsc->children, lpc,
+		if(can_be_master(child) == NULL) {
+		    do_crm_log(level, "Child %s of %s can't be promoted", child->id, rsc->id);
+		    return NULL;
+		}
+		);
+	}
 	
 	node = rsc->fns->location(rsc, NULL, FALSE);
 	if(rsc->priority < 0) {
@@ -205,10 +223,10 @@ can_be_master(resource_t *rsc)
 			    node->details->uname);
 		return NULL;
 	}
-
-	get_clone_variant_data(clone_data, rsc->parent);
+	
+	get_clone_variant_data(clone_data, parent);
 	local_node = pe_find_node_id(
-		rsc->parent->allowed_nodes, node->details->id);
+		parent->allowed_nodes, node->details->id);
 
 	if(local_node == NULL) {
 		crm_err("%s cannot run on %s: node not allowed",
@@ -267,14 +285,13 @@ static void master_promotion_order(resource_t *rsc)
 	return;
     }
     clone_data->merged_master_weights = TRUE;
-    crm_info("Merging weights for %s", rsc->id);
+    crm_debug_2("Merging weights for %s", rsc->id);
     slist_iter(
 	child, resource_t, rsc->children, lpc,
 	crm_debug_2("%s: %d", child->id, child->sort_index);
 	);
     dump_node_scores(LOG_DEBUG_3, rsc, "Before", rsc->allowed_nodes);
 
-#if 1
     slist_iter(
 	child, resource_t, rsc->children, lpc,
 
@@ -287,15 +304,17 @@ static void master_promotion_order(resource_t *rsc)
 	node = (node_t*)pe_find_node_id(
 	    rsc->allowed_nodes, chosen->details->id);
 	CRM_ASSERT(node != NULL);
+	/* adds in master preferences and rsc_location.role=Master */
 	node->weight = merge_weights(child->sort_index, node->weight);
 	);
     
     dump_node_scores(LOG_DEBUG_3, rsc, "Middle", rsc->allowed_nodes);
-#endif
     
     slist_iter(
 	constraint, rsc_colocation_t, rsc->rsc_cons_lhs, lpc,
-	
+	/* (re-)adds location preferences of resource that wish to be
+	 * colocated with the master instance
+	 */
 	if(constraint->role_rh == RSC_ROLE_MASTER) {
 	    rsc->allowed_nodes = constraint->rsc_lh->cmds->merge_weights(
 		constraint->rsc_lh, rsc->id, rsc->allowed_nodes,
@@ -331,12 +350,32 @@ int
 master_score(resource_t *rsc, node_t *node, int not_set_value)
 {
 	char *attr_name;
+	char *name = rsc->id;
 	const char *attr_value;
 	int score = not_set_value, len = 0;
 
-	len = 8 + strlen(rsc->id);
+	if(rsc->fns->state(rsc, TRUE) < RSC_ROLE_STARTED) {
+	    return score;
+	}
+
+	if(rsc->running_on) {
+	    node_t *match = pe_find_node_id(rsc->allowed_nodes, node->details->id);
+	    if(match->weight < 0) {
+		crm_debug_2("%s on %s has score: %d - ignoring master pref",
+			    rsc->id, match->details->uname, match->weight);
+		return score;
+	    }
+	}
+	    
+#if 0
+	if(rsc->clone_name) {
+	    name = rsc->clone_name;
+	    crm_err("%s ::= %s", rsc->id, rsc->clone_name);
+	}
+#endif
+	len = 8 + strlen(name);
 	crm_malloc0(attr_name, len);
-	sprintf(attr_name, "master-%s", rsc->id);
+	sprintf(attr_name, "master-%s", name);
 	
 	crm_debug_3("looking for %s on %s", attr_name,
 				node->details->uname);
@@ -397,14 +436,14 @@ apply_master_prefs(resource_t *rsc)
 	    
 	    new_score = merge_weights(node->weight, score);
 	    if(new_score != node->weight) {
-		crm_debug("\t%s: Updating preference for %s (%d->%d)",
+		crm_debug_2("\t%s: Updating preference for %s (%d->%d)",
 			  child_rsc->id, node->details->uname, node->weight, new_score);
 		node->weight = new_score;
 	    }
 	    
 	    new_score = max(child_rsc->priority, score);
 	    if(new_score != child_rsc->priority) {
-		crm_debug("\t%s: Updating priority (%d->%d)",
+		crm_debug_2("\t%s: Updating priority (%d->%d)",
 			  child_rsc->id, child_rsc->priority, new_score);
 		child_rsc->priority = new_score;
 	    }
@@ -415,11 +454,20 @@ apply_master_prefs(resource_t *rsc)
 static void set_role(resource_t *rsc, enum rsc_role_e role, gboolean current) 
 {
     if(current) {
-	rsc->role = role;
+	if(rsc->variant == pe_native && rsc->running_on != NULL && rsc->role > role) {
+	    crm_debug_6("Filtering change %s.role = %s (was %s)", rsc->id, role2text(role), role2text(rsc->role));
+
+	} else if(rsc->role < role) {
+	    crm_debug_5("Set %s.role = %s (was %s)", rsc->id, role2text(role), role2text(rsc->role));
+	    rsc->role = role;
+	}
     } else {
-	rsc->next_role = role;
-	if(role == RSC_ROLE_MASTER) {
-	    add_hash_param(rsc->parameters, crm_meta_name("role"), role2text(role));
+	if(rsc->next_role < role) {
+	    crm_debug_5("Set %s.next_role = %s (was %s)", rsc->id, role2text(role), role2text(rsc->next_role));
+	    rsc->next_role = role;
+	    if(role == RSC_ROLE_MASTER) {
+		add_hash_param(rsc->parameters, crm_meta_name("role"), role2text(role));
+	    }
 	}
     }
     
@@ -518,7 +566,7 @@ master_color(resource_t *rsc, pe_working_set_t *data_set)
 
 	    );
 
-	master_promotion_order(rsc);	
+	master_promotion_order(rsc);
 
 	/* mark the first N as masters */
 	slist_iter(
@@ -526,6 +574,14 @@ master_color(resource_t *rsc, pe_working_set_t *data_set)
 
 		chosen = NULL;
 		crm_debug_2("Processing %s", child_rsc->id);
+
+		chosen = child_rsc->fns->location(child_rsc, NULL, FALSE);
+		do_crm_log(scores_log_level, "%s promotion score on %s: %d",
+			   child_rsc->id, chosen?chosen->details->uname:"none", child_rsc->sort_index);
+		chosen = NULL; /* nuke 'chosen' so that we don't promote more than the
+				* required number of instances
+				*/
+		
 		if(promoted < clone_data->master_max) {
 			chosen = can_be_master(child_rsc);
 		}
@@ -541,7 +597,8 @@ master_color(resource_t *rsc, pe_working_set_t *data_set)
 		}
 
 		chosen->count++;
-		crm_info("Promoting %s", child_rsc->id);
+		crm_info("Promoting %s (%s %s)",
+			 child_rsc->id, role2text(child_rsc->role), chosen->details->uname);
 		set_role(child_rsc, RSC_ROLE_MASTER, FALSE);
 		clone_data->masters_allocated++;
 		promoted++;		
@@ -581,6 +638,7 @@ void master_create_actions(resource_t *rsc, pe_working_set_t *data_set)
 
 		any_demoting = any_demoting || child_demoting;
 		any_promoting = any_promoting || child_promoting;
+		crm_debug_2("Created actions for %s: %d %d", child_rsc->id, child_promoting, child_demoting);
 		);
 	
 	/* promote */
@@ -702,8 +760,7 @@ master_internal_constraints(resource_t *rsc, pe_working_set_t *data_set)
 	
 }
 
-extern resource_t *find_compatible_child(resource_t *local_child, resource_t *rsc, enum rsc_role_e filter);
-static void node_list_update_one(GListPtr list1, node_t *other, int score)
+void node_list_update_one(GListPtr list1, node_t *other, int score)
 {
     node_t *node = NULL;
     
@@ -726,6 +783,7 @@ void master_rsc_colocation_rh(
 	clone_variant_data_t *clone_data = NULL;
 	get_clone_variant_data(clone_data, rsc_rh);
 
+	
 	CRM_CHECK(rsc_rh != NULL, return);
 	if(is_set(rsc_rh->flags, pe_rsc_provisional)) {
 		return;
@@ -740,7 +798,7 @@ void master_rsc_colocation_rh(
 	CRM_CHECK(rsc_lh->variant == pe_native, return);
 	crm_debug_2("Processing constraint %s: %d", constraint->id, constraint->score);
 
-	if(constraint->score < INFINITY) {
+	if(constraint->role_rh == RSC_ROLE_UNKNOWN) {
 		slist_iter(
 			child_rsc, resource_t, rsc_rh->children, lpc,
 			
@@ -750,7 +808,6 @@ void master_rsc_colocation_rh(
 	} else if(is_set(rsc_lh->flags, pe_rsc_provisional)) {
 		GListPtr lhs = NULL, rhs = NULL;
 		lhs = rsc_lh->allowed_nodes;
-		
 		slist_iter(
 			child_rsc, resource_t, rsc_rh->children, lpc,
 			node_t *chosen = child_rsc->fns->location(child_rsc, NULL, FALSE);
@@ -758,9 +815,11 @@ void master_rsc_colocation_rh(
 			crm_debug_3("Processing: %s", child_rsc->id);
 			if(chosen != NULL
 			   && next_role == constraint->role_rh) {
-			    crm_debug_3("Applying: %s %s", child_rsc->id,
-					role2text(next_role));
-			    node_list_update_one(rsc_lh->allowed_nodes, chosen, constraint->score);
+			    crm_debug_3("Applying: %s %s %s %d", child_rsc->id,
+					role2text(next_role), chosen->details->uname, constraint->score);
+			    if(constraint->score < INFINITY) {
+				node_list_update_one(rsc_lh->allowed_nodes, chosen, constraint->score);
+			    }
 			    rhs = g_list_append(rhs, chosen);
 			}
 			);
@@ -769,15 +828,16 @@ void master_rsc_colocation_rh(
 		 * Doing this unconditionally would prevent the slaves from being started
 		 */
 		if(constraint->role_lh != RSC_ROLE_MASTER
-		    || constraint->role_rh != RSC_ROLE_MASTER) {
-		    rsc_lh->allowed_nodes = node_list_and(lhs, rhs, FALSE);
-		    pe_free_shallow(lhs);
+		   || constraint->role_rh != RSC_ROLE_MASTER) {
+		    if(constraint->score > 0) {
+			rsc_lh->allowed_nodes = node_list_exclude(lhs, rhs);
+			pe_free_shallow(lhs);
+		    }
 		}
-		
 		pe_free_shallow_adv(rhs, FALSE);
 
 	} else if(constraint->role_lh == RSC_ROLE_MASTER) {
-	    resource_t *rh_child = find_compatible_child(rsc_lh, rsc_rh, constraint->role_rh);
+	    resource_t *rh_child = find_compatible_child(rsc_lh, rsc_rh, constraint->role_rh, FALSE);
 	    if(rh_child == NULL && constraint->score >= INFINITY) {
 		crm_debug_2("%s can't be promoted %s", rsc_lh->id, constraint->id);
 		rsc_lh->priority = -INFINITY;
