@@ -30,7 +30,7 @@
 #define VARIANT_NATIVE 1
 #include <lib/crm/pengine/variant.h>
 
-resource_t *ultimate_parent(resource_t *rsc);
+gboolean at_stack_bottom(resource_t *rsc);
 
 void node_list_update(GListPtr list1, GListPtr list2, int factor);
 
@@ -94,33 +94,60 @@ native_choose_node(resource_t *rsc)
 	GListPtr nodes = NULL;
 	node_t *chosen = NULL;
 
+	int lpc = 0;
+	int multiple = 0;
+	int length = g_list_length(rsc->allowed_nodes);
+
 	if(is_not_set(rsc->flags, pe_rsc_provisional)) {
 		return rsc->allocated_to?TRUE:FALSE;
 	}
 	
 	crm_debug_3("Choosing node for %s from %d candidates",
-		    rsc->id, g_list_length(rsc->allowed_nodes));
+		    rsc->id, length);
 
 	if(rsc->allowed_nodes) {
-		rsc->allowed_nodes = g_list_sort(
-			rsc->allowed_nodes, sort_node_weight);
-		nodes = rsc->allowed_nodes;
-		chosen = g_list_nth_data(nodes, 0);
+	    rsc->allowed_nodes = g_list_sort(rsc->allowed_nodes, sort_node_weight);
+	    nodes = rsc->allowed_nodes;
+	    chosen = g_list_nth_data(nodes, 0);
+
+	    if(chosen
+	       && chosen->weight > 0
+	       && can_run_resources(chosen)) {
+		node_t *running = g_list_nth_data(rsc->running_on, 0);
+		if(can_run_resources(running) == FALSE) {
+		    running = NULL;
+		}
+		
+		for(lpc = 1; lpc < length; lpc++) {
+		    node_t *tmp = g_list_nth_data(nodes, lpc);
+		    if(tmp->weight == chosen->weight) {
+			multiple++;
+			if(running && tmp->details == running->details) {
+			    /* prefer the existing node if scores are equal */
+			    chosen = tmp;
+			}
+		    }
+		}
+	    }
+	}
+
+	if(multiple > 1) {
+		int log_level = LOG_INFO;
+		char *score = score2char(chosen->weight);
+		if(chosen->weight >= INFINITY) {
+			log_level = LOG_WARNING;
+		}
+		
+		do_crm_log(log_level, "%d nodes with equal score (%s) for"
+			   " running %s resources.  Chose %s.",
+			   multiple, score, rsc->id, chosen->details->uname);
+		crm_free(score);
 	}
 	
+	
 	return native_assign_node(rsc, nodes, chosen);
 }
 
-resource_t *
-ultimate_parent(resource_t *rsc)
-{
-	resource_t *parent = rsc;
-	while(parent->parent) {
-		parent = parent->parent;
-	}
-	return parent;
-}
-
 GListPtr
 native_merge_weights(
     resource_t *rsc, const char *rhs, GListPtr nodes, int factor, gboolean allow_rollback) 
@@ -128,11 +155,11 @@ native_merge_weights(
     GListPtr archive = NULL;
 
     if(is_set(rsc->flags, pe_rsc_merging)) {
-	crm_debug("%s: Breaking dependancy loop", rhs);
+	crm_info("%s: Breaking dependancy loop", rhs);
 	return nodes;
 
-    } else if(is_not_set(rsc->flags, pe_rsc_provisional)
-	      || can_run_any(nodes) == FALSE) {
+    } else if(is_not_set(rsc->flags, pe_rsc_provisional)) {
+	crm_debug_4("%s: not provisional", rsc->id);
 	return nodes;
     }
 
@@ -143,7 +170,35 @@ native_merge_weights(
  	archive = node_list_dup(nodes, FALSE, FALSE);
     }
 
+#if 0
     node_list_update(nodes, rsc->allowed_nodes, factor);
+#else
+    /* turn this off once we switch to migration-threshold */
+    {
+	GListPtr tmp = node_list_dup(rsc->allowed_nodes, FALSE, FALSE);
+	
+	slist_iter(
+	    node, node_t, tmp, lpc,
+	    if(node->weight < 0 && node->weight > -INFINITY) {
+		/* Once a dependant's score goes below zero, force the node score to -INFINITY
+		 *
+		 * This prevents the colocation sets from being partially active in scenarios
+		 *  where it could be fully active elsewhere
+		 *
+		 * If we don't do this, then the next resource's stickiness might bring
+		 *  the combined score above 0 again - which confuses the PE into thinking
+		 *  the whole colocation set can run there but is pointless since the later children
+		 *  are not be able to run if the ones before them can't
+		 */
+		node->weight = -INFINITY;
+	    }
+	    );
+	
+	node_list_update(nodes, tmp, factor);
+	pe_free_shallow_adv(tmp, TRUE);
+    }
+#endif
+    
     if(archive && can_run_any(nodes) == FALSE) {
 	crm_debug("%s: Rolling back scores from %s", rhs, rsc->id);
   	pe_free_shallow_adv(nodes, TRUE);
@@ -169,7 +224,7 @@ native_merge_weights(
 node_t *
 native_color(resource_t *rsc, pe_working_set_t *data_set)
 {
-        int alloc_details = LOG_DEBUG_2;
+        int alloc_details = scores_log_level;
 	if(rsc->parent && is_not_set(rsc->parent->flags, pe_rsc_allocating)) {
 		/* never allocate children on their own */
 		crm_debug("Escalating allocation of %s to its parent: %s",
@@ -187,20 +242,20 @@ native_color(resource_t *rsc, pe_working_set_t *data_set)
 	}
 
 	set_bit(rsc->flags, pe_rsc_allocating);
-	print_resource(alloc_details, "Allocating: ", rsc, FALSE);
-	dump_node_scores(alloc_details, rsc, "Pre-allloc", rsc->allowed_nodes);
+	print_resource(alloc_details+1, "Allocating: ", rsc, FALSE);
+	dump_node_scores(alloc_details+1, rsc, "Pre-allloc", rsc->allowed_nodes);
 
 	slist_iter(
 		constraint, rsc_colocation_t, rsc->rsc_cons, lpc,
 
 		resource_t *rsc_rh = constraint->rsc_rh;
-		crm_debug("%s: Pre-Processing %s (%s)",
-			  rsc->id, constraint->id, rsc_rh->id);
+		crm_debug_2("%s: Pre-Processing %s (%s)",
+			    rsc->id, constraint->id, rsc_rh->id);
 		rsc_rh->cmds->color(rsc_rh, data_set);
 		rsc->cmds->rsc_colocation_lh(rsc, rsc_rh, constraint);	
 	    );	
 
-	dump_node_scores(alloc_details, rsc, "Post-coloc", rsc->allowed_nodes);
+	dump_node_scores(alloc_details+1, rsc, "Post-coloc", rsc->allowed_nodes);
 
 	slist_iter(
 	    constraint, rsc_colocation_t, rsc->rsc_cons_lhs, lpc,
@@ -210,17 +265,31 @@ native_color(resource_t *rsc, pe_working_set_t *data_set)
 		constraint->score/INFINITY, TRUE);
 	    );
 	
-	dump_node_scores(alloc_details, rsc, "Post-merge", rsc->allowed_nodes);
-	
-	print_resource(LOG_DEBUG, "Allocating: ", rsc, FALSE);
+	print_resource(LOG_DEBUG_2, "Allocating: ", rsc, FALSE);
 	if(rsc->next_role == RSC_ROLE_STOPPED) {
 		crm_debug_2("Making sure %s doesn't get allocated", rsc->id);
 		/* make sure it doesnt come up again */
 		resource_location(
 			rsc, NULL, -INFINITY, "target_role", data_set);
 	}
-	
-	if(is_set(rsc->flags, pe_rsc_provisional)
+
+	dump_node_scores(alloc_details, rsc, __PRETTY_FUNCTION__, rsc->allowed_nodes);
+	if(is_not_set(rsc->flags, pe_rsc_managed)) {
+	    const char *reason = NULL;
+	    node_t *assign_to = NULL;
+	    if(rsc->running_on == NULL) {
+		reason = "inactive";
+	    } else if(is_set(rsc->flags, pe_rsc_failed)) {
+		reason = "failed";		
+	    } else {
+		assign_to = rsc->running_on->data;
+		reason = "active";
+	    }
+	    crm_info("Unmanaged resource %s allocated to %s: %s", rsc->id,
+		     assign_to?assign_to->details->uname:"'nowhere'", reason);
+	    native_assign_node(rsc, NULL, assign_to);
+
+	} else if(is_set(rsc->flags, pe_rsc_provisional)
 	   && native_choose_node(rsc) ) {
 		crm_debug_3("Allocated resource %s to %s",
 			    rsc->id, rsc->allocated_to->details->uname);
@@ -228,7 +297,7 @@ native_color(resource_t *rsc, pe_working_set_t *data_set)
 	} else if(rsc->allocated_to == NULL) {
 		if(is_not_set(rsc->flags, pe_rsc_orphan)) {
 			pe_warn("Resource %s cannot run anywhere", rsc->id);
-		} else {
+		} else if(rsc->running_on != NULL) {
 			crm_info("Stopping orphan resource %s", rsc->id);
 		}
 		
@@ -243,6 +312,41 @@ native_color(resource_t *rsc, pe_working_set_t *data_set)
 	return rsc->allocated_to;
 }
 
+static gboolean is_op_dup(
+    resource_t *rsc, const char *name, const char *interval) 
+{
+    gboolean dup = FALSE;
+    const char *id = NULL;
+    const char *value = NULL;
+    xml_child_iter_filter(
+	rsc->ops_xml, operation, "op",
+	value = crm_element_value(operation, "name");
+	if(safe_str_neq(value, name)) {
+	    continue;
+	}
+	
+	value = crm_element_value(operation, XML_LRM_ATTR_INTERVAL);
+	if(value == NULL) {
+	    value = "0";
+	}
+	
+	if(safe_str_neq(value, interval)) {
+	    continue;
+	}
+
+	if(id == NULL) {
+	    id = ID(operation);
+	    
+	} else {
+	    crm_config_err("Operation %s is a duplicate of %s", ID(operation), id);
+	    crm_config_err("Do not use the same (name, interval) combination more than once per resource");
+	    dup = TRUE;
+	}
+	);
+    
+    return dup;
+}
+
 void
 RecurringOp(resource_t *rsc, action_t *start, node_t *node,
 	    crm_data_t *operation, pe_working_set_t *data_set) 
@@ -258,7 +362,9 @@ RecurringOp(resource_t *rsc, action_t *start, node_t *node,
 	gboolean is_optional = TRUE;
 	GListPtr possible_matches = NULL;
 	
-	crm_debug_2("Creating recurring actions for %s", rsc->id);
+	crm_debug_2("Creating recurring action %s for %s",
+		    ID(operation), rsc->id);
+	
 	if(node != NULL) {
 		node_uname = node->details->uname;
 	}
@@ -281,6 +387,10 @@ RecurringOp(resource_t *rsc, action_t *start, node_t *node,
 	}
 	
 	name = crm_element_value(operation, "name");
+	if(is_op_dup(rsc, name, interval)) {
+	    return;
+	}
+
 	key = generate_op_key(rsc->id, name, interval_ms);
 	if(start != NULL) {
 		crm_debug_3("Marking %s %s due to %s",
@@ -291,7 +401,7 @@ RecurringOp(resource_t *rsc, action_t *start, node_t *node,
 		crm_debug_2("Marking %s optional", key);
 		is_optional = TRUE;
 	}
-	
+
 	/* start a monitor for an already active resource */
 	possible_matches = find_actions_exact(rsc->actions, key, node);
 	if(possible_matches == NULL) {
@@ -320,11 +430,31 @@ RecurringOp(resource_t *rsc, action_t *start, node_t *node,
 			mon->task = crm_strdup(CRMD_ACTION_CANCEL);
 			add_hash_param(mon->meta, XML_LRM_ATTR_INTERVAL, interval);
 			add_hash_param(mon->meta, XML_LRM_ATTR_TASK, name);
-			
-			custom_action_order(
-				rsc, NULL, mon,
-				rsc, promote_key(rsc), NULL,
-				pe_order_runnable_left, data_set);
+
+			local_key = NULL;
+
+			switch(rsc->role) {
+			    case RSC_ROLE_SLAVE:
+			    case RSC_ROLE_STARTED:
+				if(rsc->next_role == RSC_ROLE_MASTER) {
+				    local_key = promote_key(rsc);
+
+				} else if(rsc->next_role == RSC_ROLE_STOPPED) {
+				    local_key = stop_key(rsc);
+				}
+				
+				break;
+			    case RSC_ROLE_MASTER:
+				local_key = demote_key(rsc);
+				break;
+			    default:
+				break;
+			}
+
+			if(local_key) {
+			    custom_action_order(rsc, NULL, mon, rsc, local_key, NULL,
+						pe_order_runnable_left, data_set);
+			}
 			
 			mon = NULL;
 		}
@@ -359,7 +489,7 @@ RecurringOp(resource_t *rsc, action_t *start, node_t *node,
 		mon->runnable = FALSE;
 		
 	} else if(mon->optional == FALSE) {
-		crm_notice("%s\t   %s", crm_str(node_uname),mon->uuid);
+	    crm_notice(" Start recurring %s (%ds) for %s on %s", mon->task, interval_ms/1000, rsc->id, crm_str(node_uname));
 	}
 	
 	custom_action_order(rsc, start_key(rsc), NULL,
@@ -374,7 +504,13 @@ RecurringOp(resource_t *rsc, action_t *start, node_t *node,
 			rsc, NULL, mon,
 			pe_order_optional|pe_order_runnable_left, data_set);
 		crm_free(running_master);
-	}		
+
+	} else if(rsc->role == RSC_ROLE_MASTER) {
+		custom_action_order(
+			rsc, demote_key(rsc), NULL,
+			rsc, NULL, mon,
+			pe_order_optional|pe_order_runnable_left, data_set);
+	}
 }
 
 void
@@ -413,6 +549,9 @@ void native_create_actions(resource_t *rsc, pe_working_set_t *data_set)
 	if(g_list_length(rsc->running_on) > 1) {
  		if(rsc->recovery_type == recovery_stop_start) {
 			pe_proc_err("Attempting recovery of resource %s", rsc->id);
+			if(rsc->role == RSC_ROLE_MASTER) {
+			    DemoteRsc(rsc, NULL, FALSE, data_set);
+			}
 			StopRsc(rsc, NULL, FALSE, data_set);
 			rsc->role = RSC_ROLE_STOPPED;
 		}
@@ -432,8 +571,16 @@ void native_create_actions(resource_t *rsc, pe_working_set_t *data_set)
 		g_list_free(possible_matches);
 		crm_debug_2("Stopping a stopped resource");
 		crm_free(key);
-		return;
-	} 
+		goto do_recurring;
+		
+	} else if(rsc->role != RSC_ROLE_STOPPED) {
+	    /* A cheap trick to account for the fact that Master/Slave groups may not be
+	     * completely running when we set their role to Slave
+	     */
+	    crm_debug_2("Resetting %s.role = %s (was %s)",
+			rsc->id, role2text(RSC_ROLE_STOPPED), role2text(rsc->role));
+	    rsc->role = RSC_ROLE_STOPPED;
+	}
 
 	role = rsc->role;
 
@@ -448,7 +595,8 @@ void native_create_actions(resource_t *rsc, pe_working_set_t *data_set)
 		role = next_role;
 	}
 
-	if(rsc->next_role != RSC_ROLE_STOPPED && is_set(rsc->flags, pe_rsc_managed)) {
+  do_recurring:
+	if(rsc->next_role != RSC_ROLE_STOPPED || is_set(rsc->flags, pe_rsc_managed) == FALSE) {
 		start = start_action(rsc, chosen, TRUE);
 		Recurring(rsc, start, chosen, data_set);
 	}
@@ -464,7 +612,7 @@ void native_internal_constraints(resource_t *rsc, pe_working_set_t *data_set)
 		type |= pe_order_implies_right;
 	}
 
-	if(rsc->parent == NULL) {
+	if(rsc->parent == NULL || rsc->parent->variant == pe_group) {
 		type |= pe_order_restart;
 	}
 	
@@ -474,7 +622,7 @@ void native_internal_constraints(resource_t *rsc, pe_working_set_t *data_set)
 
 	custom_action_order(rsc, demote_key(rsc), NULL,
 			    rsc, stop_key(rsc), NULL,
-			    pe_order_optional, data_set);
+			    pe_order_demote_stop, data_set);
 
 	custom_action_order(rsc, start_key(rsc), NULL,
 			    rsc, promote_key(rsc), NULL,
@@ -506,13 +654,7 @@ void native_internal_constraints(resource_t *rsc, pe_working_set_t *data_set)
 		return;
 	} 
 
-	if(safe_str_eq(class, "stonith")) {
-	    custom_action_order(
-		NULL, crm_strdup(all_stopped->task), all_stopped,
-		rsc, stop_key(rsc), NULL,
-		pe_order_implies_left|pe_order_stonith_stop, data_set);
-	    
-	} else if(rsc->variant == pe_native) {
+	if(rsc->variant == pe_native && safe_str_neq(class, "stonith")) {
 	    custom_action_order(
 		rsc, stop_key(rsc), NULL,
 		NULL, crm_strdup(all_stopped->task), all_stopped,
@@ -543,23 +685,43 @@ static gboolean
 filter_colocation_constraint(
 	resource_t *rsc_lh, resource_t *rsc_rh, rsc_colocation_t *constraint)
 {
+	int level = LOG_DEBUG_4;
 	if(constraint->score == 0){
 		return FALSE;
 	}
 
-	if(constraint->role_lh != RSC_ROLE_UNKNOWN
+	if(constraint->score > 0
+	   && constraint->role_lh != RSC_ROLE_UNKNOWN
 	   && constraint->role_lh != rsc_lh->next_role) {
-		crm_debug_4("RH: Skipping constraint: \"%s\" state filter",
+		do_crm_log(level, "LH: Skipping constraint: \"%s\" state filter",
 			    role2text(constraint->role_rh));
 		return FALSE;
 	}
 	
-	if(constraint->role_rh != RSC_ROLE_UNKNOWN
+	if(constraint->score > 0
+	   && constraint->role_rh != RSC_ROLE_UNKNOWN
 	   && constraint->role_rh != rsc_rh->next_role) {
-		crm_debug_4("RH: Skipping constraint: \"%s\" state filter",
+		do_crm_log(level, "RH: Skipping constraint: \"%s\" state filter",
 			    role2text(constraint->role_rh));
 		return FALSE;
 	}
+
+	if(constraint->score < 0
+	   && constraint->role_lh != RSC_ROLE_UNKNOWN
+	   && constraint->role_lh == rsc_lh->next_role) {
+		do_crm_log(level, "LH: Skipping -ve constraint: \"%s\" state filter",
+			    role2text(constraint->role_rh));
+		return FALSE;
+	}
+	
+	if(constraint->score < 0
+	   && constraint->role_rh != RSC_ROLE_UNKNOWN
+	   && constraint->role_rh == rsc_rh->next_role) {
+		do_crm_log(level, "RH: Skipping -ve constraint: \"%s\" state filter",
+			    role2text(constraint->role_rh));
+		return FALSE;
+	}
+
 	return TRUE;
 }
 
@@ -592,10 +754,12 @@ colocation_match(
 		node, node_t, rsc_lh->allowed_nodes, lpc,
 		tmp = g_hash_table_lookup(node->details->attrs, attribute);
 		if(do_check && safe_str_eq(tmp, value)) {
+		    if(constraint->score < INFINITY) {
 			crm_debug_2("%s: %s.%s += %d", constraint->id, rsc_lh->id,
 				  node->details->uname, constraint->score);
 			node->weight = merge_weights(
 				constraint->score, node->weight);
+		    }
 
 		} else if(do_check == FALSE || constraint->score >= INFINITY) {
 			crm_debug_2("%s: %s.%s = -INFINITY (%s)", constraint->id, rsc_lh->id,
@@ -664,8 +828,7 @@ node_list_update(GListPtr list1, GListPtr list2, int factor)
 			continue;
 		}
 
-		other_node = (node_t*)pe_find_node_id(
-			list2, node->details->id);
+		other_node = (node_t*)pe_find_node_id(list2, node->details->id);
 
 		if(other_node != NULL) {
 			crm_debug_2("%s: %d + %d",
@@ -677,21 +840,46 @@ node_list_update(GListPtr list1, GListPtr list2, int factor)
 		);	
 }
 
+static GListPtr find_actions_by_task(GListPtr actions, resource_t *rsc, const char *original_key)
+{
+    GListPtr list = NULL;
+
+    list = find_actions(actions, original_key, NULL);
+    if(list == NULL) {
+	/* we're potentially searching a child of the original resource */
+	char *key = NULL;
+	char *tmp = NULL;
+	char *task = NULL;
+	int interval = 0;
+	CRM_CHECK(parse_op_key(original_key, &tmp, &task, &interval),
+		  crm_err("search key: %s", original_key); return NULL);
+	
+	key = generate_op_key(rsc->id, task, interval);
+	list = find_actions(actions, key, NULL);
+
+	crm_free(key);
+	crm_free(tmp);
+	crm_free(task);
+    }
+
+    return list;
+}
+
 void native_rsc_order_lh(resource_t *lh_rsc, order_constraint_t *order, pe_working_set_t *data_set)
 {
 	GListPtr lh_actions = NULL;
 	action_t *lh_action = order->lh_action;
 	resource_t *rh_rsc = order->rh_rsc;
 
-	crm_debug_2("Processing LH of ordering constraint %d", order->id);
+	crm_debug_3("Processing LH of ordering constraint %d", order->id);
 	CRM_ASSERT(lh_rsc != NULL);
 	
 	if(lh_action != NULL) {
 		lh_actions = g_list_append(NULL, lh_action);
 
 	} else if(lh_action == NULL) {
-		lh_actions = find_actions(
-			lh_rsc->actions, order->lh_action_task, NULL);
+		lh_actions = find_actions_by_task(
+		    lh_rsc->actions, lh_rsc, order->lh_action_task);
 	}
 
 	if(lh_actions == NULL && lh_rsc != rh_rsc) {
@@ -760,8 +948,8 @@ void native_rsc_order_rh(
 		rh_actions = g_list_append(NULL, rh_action);
 
 	} else if(rsc != NULL) {
-		rh_actions = find_actions(
-			rsc->actions, order->rh_action_task, NULL);
+		rh_actions = find_actions_by_task(
+		    rsc->actions, rsc, order->rh_action_task);
 	}
 
 	if(rh_actions == NULL) {
@@ -810,7 +998,7 @@ void native_rsc_location(resource_t *rsc, rsc_to_node_t *constraint)
 	} else if(constraint->role_filter > 0
 		  && constraint->role_filter != rsc->next_role) {
 		crm_debug("Constraint (%s) is not active (role : %s)",
-			    constraint->id, role2text(constraint->role_filter));
+			  constraint->id, role2text(constraint->role_filter));
 		return;
 		
 	} else if(is_active(constraint) == FALSE) {
@@ -860,10 +1048,10 @@ register_activity(resource_t *rsc, enum action_tasks task, node_t *node, notify_
 {
 	notify_entry_t *entry = NULL;
 
-	CRM_CHECK(node != NULL,
-		  crm_err("%s has no node for required action %s",
-			  rsc->id, task2text(task));
-		  return);
+	if(node == NULL) {
+	    pe_proc_warn("%s has no node for required action %s", rsc->id, task2text(task));
+	    return;
+	}
 
 	crm_malloc0(entry, sizeof(notify_entry_t));
 	entry->rsc = rsc;
@@ -1034,7 +1222,7 @@ pe_notify(resource_t *rsc, node_t *node, action_t *op, action_t *confirm,
 	value = g_hash_table_lookup(op->meta, "notify_type");
 	task = g_hash_table_lookup(op->meta, "notify_operation");
 
-	crm_debug("Creating notify actions for %s: %s (%s-%s)",
+	crm_debug_2("Creating notify actions for %s: %s (%s-%s)",
 		    op->uuid, rsc->id, value, task);
 	
 	key = generate_notify_key(rsc->id, value, task);
@@ -1116,21 +1304,64 @@ NoRoleChange(resource_t *rsc, node_t *current, node_t *next,
 	action_t *start = NULL;		
 	GListPtr possible_matches = NULL;
 
-	crm_debug_2("Executing: %s (role=%s)",rsc->id, role2text(rsc->next_role));
+	crm_debug_2("Executing: %s (role=%s)", rsc->id, role2text(rsc->next_role));
 
-	if(current == NULL || next == NULL) {
-		return;
+	if(current == NULL && next == NULL) {
+	    crm_notice("Leave resource %s\t(%s)",
+		       rsc->id, role2text(rsc->role));
+	    return;
+
+	} else if(next == NULL) {
+	    crm_notice("Stop resource %s\t(%s %s)",
+		       rsc->id, role2text(rsc->role), current->details->uname);
+	    return;
+
+	} else if(current == NULL && rsc->next_role > RSC_ROLE_SLAVE) {
+	    crm_notice("Promote %s\t(%s -> %s %s)", rsc->id,
+		       role2text(rsc->role), role2text(rsc->next_role),
+		       next->details->uname);
+	    return;
+
+	} else if(current == NULL) {
+	    crm_notice("Start %s\t(%s -> %s %s)", rsc->id,
+		       role2text(rsc->role), role2text(rsc->next_role),
+		       next->details->uname);
+	    return;
 	}
 
+	if(rsc->role == rsc->next_role) {
+	    start = start_action(rsc, next, TRUE);
+	    if(start->optional) {
+		crm_notice("Leave resource %s\t(%s %s)",
+			   rsc->id, role2text(rsc->role), next->details->uname);
+
+	    } else if(safe_str_eq(current->details->id, next->details->id)) {
+		if(is_set(rsc->flags, pe_rsc_failed)) {
+		    crm_notice("Recover resource %s\t(%s %s)",
+			       rsc->id, role2text(rsc->role), next->details->uname);
+		} else {
+		    crm_notice("Restart resource %s\t(%s %s)",
+			       rsc->id, role2text(rsc->role), next->details->uname);
+		}
+		
+	    } else {
+		crm_notice("Move resource %s\t(%s %s -> %s)",
+			   rsc->id, role2text(rsc->role), current->details->uname, next->details->uname);
+	    }
+
+	} else if(rsc->role < rsc->next_role) {
+	    crm_notice("Promote %s\t(%s -> %s %s)", rsc->id,
+		       role2text(rsc->role), role2text(rsc->next_role),
+		       next->details->uname);
+	    
+	} else if(rsc->role > rsc->next_role) {
+	    crm_notice("Demote %s\t(%s -> %s %s)", rsc->id,
+		       role2text(rsc->role), role2text(rsc->next_role),
+		       next->details->uname);
+	}
+	
 	if(is_set(rsc->flags, pe_rsc_failed)
 	   || safe_str_neq(current->details->id, next->details->id)) {
-		if(is_set(rsc->flags, pe_rsc_failed)) {
-			crm_notice("Recover resource %s\t(%s)",
-				   rsc->id, next->details->uname);
-		} else {
-			crm_notice("Move  resource %s\t(%s -> %s)", rsc->id,
-				   current->details->uname, next->details->uname);
-		}
 
 		if(rsc->next_role > RSC_ROLE_STARTED) {
 		    gboolean optional = TRUE;
@@ -1159,7 +1390,7 @@ NoRoleChange(resource_t *rsc, node_t *current, node_t *next,
 		g_list_free(possible_matches);
 		
 	} else if(is_set(rsc->flags, pe_rsc_start_pending)) {
-		action_t *start = start_action(rsc, next, TRUE);
+		start = start_action(rsc, next, TRUE);
 		if(start->runnable) {
 			/* wait for StartRsc() to be called */
 			rsc->role = RSC_ROLE_STOPPED;
@@ -1169,6 +1400,7 @@ NoRoleChange(resource_t *rsc, node_t *current, node_t *next,
 		}
 		
 	} else {
+	    
 		stop = stop_action(rsc, current, TRUE);
 		start = start_action(rsc, next, TRUE);
 		stop->optional = start->optional;
@@ -1183,13 +1415,6 @@ NoRoleChange(resource_t *rsc, node_t *current, node_t *next,
 		
 		if(start->runnable == FALSE) {
 			rsc->next_role = RSC_ROLE_STOPPED;
-			
-		} else if(start->optional) {
-			crm_notice("Leave resource %s\t(%s)",
-				   rsc->id, next->details->uname);
-		} else {
-			crm_notice("Restart resource %s\t(%s)",
-				   rsc->id, next->details->uname);
 		}
 	}
 }
@@ -1199,16 +1424,28 @@ gboolean
 StopRsc(resource_t *rsc, node_t *next, gboolean optional, pe_working_set_t *data_set)
 {
 	action_t *stop = NULL;
-	
+	const char *class = crm_element_value(rsc->xml, XML_AGENT_ATTR_CLASS);
+
 	crm_debug_2("Executing: %s", rsc->id);
+
+	if(rsc->next_role == RSC_ROLE_STOPPED
+	   && rsc->variant == pe_native
+	   && safe_str_eq(class, "stonith")) {
+	    action_t *all_stopped = get_pseudo_op(ALL_STOPPED, data_set);
+	    custom_action_order(
+		NULL, crm_strdup(all_stopped->task), all_stopped,
+		rsc, stop_key(rsc), NULL,
+		pe_order_implies_left|pe_order_stonith_stop, data_set);
+	}
 	
 	slist_iter(
 		current, node_t, rsc->running_on, lpc,
 		stop = stop_action(rsc, current, optional);
-		if(stop->runnable && stop->optional == FALSE) {
-			crm_notice("  %s\tStop %s", current->details->uname, rsc->id);
-		}
 
+		if(stop->runnable && stop->optional == FALSE) {
+		    crm_notice("  %s\tStop %s",current->details->uname,rsc->id);
+		}
+		
 		if(data_set->remove_after_stop) {
 			DeleteRsc(rsc, current, optional, data_set);
 		}
@@ -1244,6 +1481,8 @@ PromoteRsc(resource_t *rsc, node_t *next, gboolean optional, pe_working_set_t *d
 		  crm_err("Next role: %s", role2text(rsc->next_role));
 		  return FALSE);
 
+	CRM_CHECK(next != NULL, return FALSE);
+
 	key = start_key(rsc);
 	action_list = find_actions_exact(rsc->actions, key, next);
 	crm_free(key);
@@ -1259,7 +1498,7 @@ PromoteRsc(resource_t *rsc, node_t *next, gboolean optional, pe_working_set_t *d
 	if(runnable) {
 		promote_action(rsc, next, optional);
 		if(optional == FALSE) {
-			crm_notice("%s\tPromote %s", next->details->uname, rsc->id);
+			crm_debug("%s\tPromote %s", next->details->uname, rsc->id);
 		}
 		return TRUE;
 	} 
@@ -1286,7 +1525,7 @@ DemoteRsc(resource_t *rsc, node_t *next, gboolean optional, pe_working_set_t *da
 /* 	CRM_CHECK(rsc->next_role == RSC_ROLE_SLAVE, return FALSE); */
 	slist_iter(
 		current, node_t, rsc->running_on, lpc,
-		crm_notice("%s\tDemote %s", current->details->uname, rsc->id);
+		do_crm_log(optional?LOG_DEBUG:LOG_NOTICE, "%s\tDemote %s", current->details->uname, rsc->id);
 		demote_action(rsc, current, optional);
 		);
 	return TRUE;
@@ -1303,7 +1542,7 @@ RoleError(resource_t *rsc, node_t *next, gboolean optional, pe_working_set_t *da
 gboolean
 NullOp(resource_t *rsc, node_t *next, gboolean optional, pe_working_set_t *data_set)
 {
-	crm_debug("Executing: %s", rsc->id);
+	crm_debug_2("Executing: %s", rsc->id);
 	return FALSE;
 }
 
@@ -1407,7 +1646,7 @@ native_create_probe(resource_t *rsc, node_t *node, action_t *complete,
 	
 	crm_debug_2("Probing %s on %s (%s)", rsc->id, node->details->uname, role2text(rsc->role));
 	
-	custom_action_order(rsc, NULL, probe, rsc, NULL, complete,
+	custom_action_order(rsc, NULL, probe, NULL, NULL, complete,
 			    pe_order_implies_right, data_set);
 
 	return TRUE;
@@ -1436,15 +1675,10 @@ native_start_constraints(
 			       order_actions(all_stopped, action, pe_order_implies_left);
 
 			   } else if(target != NULL
-			      && target->details->expected_up
 			      && safe_str_eq(action->task, CRMD_ACTION_START)
 			      && NULL == pe_find_node_id(
 				      rsc->known_on, target->details->id)) {
-				   /* if expected_up == TRUE, then we've seen
-				    *   the node before and it has failed (as
-				    *   opposed to just hasn't started up yet)
-				    *
-				    * if known == NULL, then we dont know if
+				   /* if known == NULL, then we dont know if
 				    *   the resource is active on the node
 				    *   we're about to shoot
 				    *
@@ -1464,7 +1698,8 @@ native_start_constraints(
 					    action->uuid, target->details->uname);
 				   order_actions(all_stopped, action,
 						 pe_order_implies_left|pe_order_runnable_left);
-			   }   
+			   }
+			   
 			);
 	}
 }
@@ -1523,7 +1758,7 @@ native_stop_constraints(
 	    }
 	    
 	    if(parent) {
-		crm_info("Re-creating actions for %s", parent->id);
+		crm_debug_2("Re-creating actions for %s", parent->id);
 		parent->cmds->create_actions(parent, data_set);
 		
 		/* make sure we dont mess anything up in create_actions */
@@ -1619,11 +1854,124 @@ complex_stonith_ordering(
 	native_stop_constraints(rsc,  stonith_op, is_stonith, data_set);
 }
 
+#define ALLOW_WEAK_MIGRATION 0
+
+static gboolean
+check_stack_element(resource_t *rsc, resource_t *other_rsc, const char *type) 
+{
+    char *key = NULL;
+    int level = LOG_DEBUG;
+    GListPtr action_list = NULL;
+
+    if(other_rsc == NULL || other_rsc == rsc) {
+	return TRUE;
+    }
+
+    do_crm_log(level+1, "%s: processing %s (%s)", rsc->id, other_rsc->id, type);
+    
+    if(other_rsc->variant == pe_native) {
+	do_crm_log(level, "%s: depends on %s (mid-stack) %s",
+		   rsc->id, other_rsc->id, type);
+	return FALSE;
+	
+    } else if(other_rsc->variant == pe_group) {
+	if(at_stack_bottom(other_rsc) == FALSE) {
+	    do_crm_log(level, "%s: depends on group %s (mid-stack) %s",
+		       rsc->id, other_rsc->id, type);
+	    return FALSE;
+	}
+	return TRUE;
+    }
+    
+    /* is the clone also moving moved around?
+     *
+     * if so, then we can't yet be completely sure the
+     *   resource can safely migrate since the node we're
+     *   moving too may not have the clone instance started
+     *   yet
+     *
+     * in theory we can figure out if the clone instance we
+     *   will run on is already there, but there that would
+     *   involve too much knowledge of internal clone code.
+     *   maybe later...
+     */
+    
+    do_crm_log(level+1,"%s: start depends on clone %s",
+	       rsc->id, other_rsc->id);
+    key = stop_key(other_rsc);
+    action_list = find_actions(other_rsc->actions, key, NULL);
+    crm_free(key);
+    
+    slist_iter(
+	other_stop, action_t, action_list,lpc,
+	if(other_stop && other_stop->optional == FALSE) {
+	    do_crm_log(level, "%s: start depends on %s",
+		       rsc->id, other_stop->uuid);
+	    
+	    g_list_free(action_list);
+	    return FALSE;
+	}
+	);
+    g_list_free(action_list);
+    return TRUE;
+}
+
+gboolean
+at_stack_bottom(resource_t *rsc) 
+{
+    char *key = NULL;
+    action_t *start = NULL;
+    action_t *other = NULL;
+    GListPtr action_list = NULL;
+    
+    key = start_key(rsc);
+    action_list = find_actions(rsc->actions, key, NULL);
+    crm_free(key);
+    
+    crm_debug_3("%s: processing", rsc->id);
+    CRM_CHECK(action_list != NULL, return FALSE);
+    
+    start = action_list->data;
+    g_list_free(action_list);
+
+    slist_iter(
+	constraint, rsc_colocation_t, rsc->rsc_cons, lpc,
+
+	resource_t *target = constraint->rsc_rh;
+	crm_debug_4("%s == %s (%d)", rsc->id, target->id, constraint->score);
+	if(constraint->score > 0
+	   && check_stack_element(rsc, target, "coloc") == FALSE) {
+	    return FALSE;
+	}
+	);
+
+    slist_iter(
+	other_w, action_wrapper_t, start->actions_before, lpc,
+	other = other_w->action;
+
+#if ALLOW_WEAK_MIGRATION
+	if((other_w->type & pe_order_implies_right) == 0) {
+	    crm_debug_3("%s: depends on %s (optional ordering)",
+			rsc->id, other->uuid);
+	    continue;
+	}	
+#endif 
+
+	if(other->optional == FALSE
+	   && check_stack_element(rsc, other->rsc, "order") == FALSE) {
+	    return FALSE;
+	}
+	
+	);
+
+    return TRUE;
+}
+
 void
 complex_migrate_reload(resource_t *rsc, pe_working_set_t *data_set)
 {
 	char *key = NULL;
-	int level = LOG_DEBUG_2;
+	int level = LOG_DEBUG;
 	GListPtr action_list = NULL;
 	
 	action_t *stop = NULL;
@@ -1638,17 +1986,19 @@ complex_migrate_reload(resource_t *rsc, pe_working_set_t *data_set)
 		
 		child_rsc->cmds->migrate_reload(child_rsc, data_set);
 		);
+	    other = NULL;
 	    return;
 	}
 
+	do_crm_log(level+1, "Processing %s", rsc->id);
 	CRM_CHECK(rsc->variant == pe_native, return);
 	
 	if(is_not_set(rsc->flags, pe_rsc_managed)
 	   || is_set(rsc->flags, pe_rsc_failed)
 	   || is_set(rsc->flags, pe_rsc_start_pending)
-	   || rsc->next_role != RSC_ROLE_STARTED		   
+	   || rsc->next_role != RSC_ROLE_STARTED
 	   || g_list_length(rsc->running_on) != 1) {
-		do_crm_log(level, "%s: resource", rsc->id);
+		do_crm_log(level+1, "%s: general resource state", rsc->id);
 		return;
 	}
 	
@@ -1663,15 +2013,15 @@ complex_migrate_reload(resource_t *rsc, pe_working_set_t *data_set)
 	
 	start = action_list->data;
 	g_list_free(action_list);
-	
+
 	value = g_hash_table_lookup(rsc->meta, "allow_migrate");
 	if(crm_is_true(value)) {
 	    set_bit(rsc->flags, pe_rsc_can_migrate);	
-	}
-	
+	}	
+
 	if(is_not_set(rsc->flags, pe_rsc_can_migrate)
 	   && start->allow_reload_conversion == FALSE) {
-		do_crm_log(level, "%s: no need to continue", rsc->id);
+		do_crm_log(level+1, "%s: no need to continue", rsc->id);
 		return;
 	}
 	
@@ -1705,76 +2055,23 @@ complex_migrate_reload(resource_t *rsc, pe_working_set_t *data_set)
 		return;
 	}
 	
-	slist_iter(
-		other_w, action_wrapper_t, start->actions_before, lpc,
-		gboolean can_migrate = TRUE;
-		resource_t *parent = NULL;
-		other = other_w->action;
-		parent = uber_parent(other->rsc);
+	if(is_set(rsc->flags, pe_rsc_can_migrate)) {
+	    if(start->node == NULL
+	       || stop->node == NULL
+	       || stop->node->details == start->node->details) {
+		clear_bit(rsc->flags, pe_rsc_can_migrate);
 
-		if(other->optional == TRUE
-		   || other->rsc == rsc
-		   || parent == NULL) {
-			continue;
-		}
+	    } else if(at_stack_bottom(rsc) == FALSE) {
+		crm_notice("Cannot migrate %s from %s to %s"
+			   " - %s is not at the bottom of the resource stack",
+			   rsc->id, stop->node->details->uname,
+			   start->node->details->uname, rsc->id);
+		clear_bit(rsc->flags, pe_rsc_can_migrate);
+	    }
+	}
 
-		if(parent->variant == pe_native
-		   || parent->variant == pe_group) {
-			/* clones are the only ones that can be "moved"
-			 * and still allow resources sitting on top of
-			 * them (ie. us) to be migrated 
-			 */
-			can_migrate = FALSE;
-			
-		} else if(safe_str_eq(other->task, CRMD_ACTION_MIGRATE)
-			  || safe_str_eq(other->task, CRMD_ACTION_MIGRATED)) {
-			/* we depend on something that is already migrating...
-			 * we cant both migrate
-			 */
-			can_migrate = FALSE;
-			
-		} else {
-			/* is the clone also moving moved around?
-			 *
-			 * if so, then we can't yet be completely sure the
-			 *   resource can safely migrate since the node we're
-			 *   moving too may not have the clone instance started
-			 *   yet
-			 *
-			 * in theory we can figure out if the clone instance we
-			 *   will run on is already there, but there that would
-			 *   involve too much knowledge of internal clone code.
-			 *   maybe later...
-			 */
-			do_crm_log(level,
-				   "%s: start depends on clone %s",
-				   rsc->id, parent->id);
-			key = stop_key(parent);
-			action_list = find_actions(parent->actions, key, NULL);
-			crm_free(key);
-			
-			slist_iter(
-				other_stop, action_t, action_list,lpc,
-				if(other_stop && other_stop->optional == FALSE) {
-					do_crm_log(LOG_INFO,
-						   "%s: start depends on %s",
-						   rsc->id, other_stop->uuid);
-					can_migrate = FALSE;
-				}
-				);
-			g_list_free(action_list);
-		}
-		
-		if(can_migrate == FALSE) {
-			do_crm_log(LOG_INFO, "%s: start depends on %s",
-				   rsc->id, other->uuid);
-			return;
-		}
-		);
-
-	if(is_set(rsc->flags, pe_rsc_can_migrate)
-	   && stop->node->details != start->node->details) {
-		crm_info("Migrating %s from %s to %s", rsc->id,
+	if(is_set(rsc->flags, pe_rsc_can_migrate)) {
+		crm_notice("Migrating %s from %s to %s", rsc->id,
 			 stop->node->details->uname,
 			 start->node->details->uname);
 		
@@ -1787,17 +2084,33 @@ complex_migrate_reload(resource_t *rsc, pe_working_set_t *data_set)
 		add_hash_param(stop->meta, "migrate_target",
 			       start->node->details->uname);
 
+		/* Hook up to the all_stopped and shutdown actions */
+		slist_iter(
+			other_w, action_wrapper_t, stop->actions_after, lpc,
+			other = other_w->action;
+			if(other->optional == FALSE
+			   && other->rsc == NULL) {
+				order_actions(start, other, other_w->type);
+			}
+			);
 
 		slist_iter(
 			other_w, action_wrapper_t, start->actions_before, lpc,
 			other = other_w->action;
 			if(other->optional == FALSE
+#if ALLOW_WEAK_MIGRATION
+			   && (other_w->type & pe_order_implies_right)
+#endif
 			   && other->rsc != NULL
+			   && other->rsc != rsc->parent
 			   && other->rsc != rsc) {
+				do_crm_log(level, "Ordering %s before %s",
+					   other->uuid, stop->uuid);
+			    
 				order_actions(other, stop, other_w->type);
 			}
 			);
-		
+
 		crm_free(start->uuid);
 		crm_free(start->task);
 		start->task = crm_strdup(CRMD_ACTION_MIGRATED);
@@ -1821,6 +2134,6 @@ complex_migrate_reload(resource_t *rsc, pe_working_set_t *data_set)
 		stop->pseudo = TRUE; /* easier than trying to delete it from the graph */
 		
 	} else {
-		do_crm_log(level, "%s nothing to do", rsc->id);
+		do_crm_log(level+1, "%s nothing to do", rsc->id);
 	}
 }
