@@ -56,8 +56,15 @@ enum pe_ordering {
 	pe_order_demote			= 0x800, /* stop-start constraint */
 
 	pe_order_shutdown		= 0x1000, /* combines with pe_order_restart to make a complex resource shut down */
+	pe_order_demote_stop		= 0x2000, /* upgrades to implies_left if the resource is a master */
+
+	pe_order_complex_left		= 0x10000, /* upgrades to implies left */
+	pe_order_complex_right		= 0x20000, /* upgrades to implies right */
+
+	pe_order_implies_left_printed	= 0x40000, /* Like implies left but only ensures the action is printed, not manditory */
+	pe_order_implies_right_printed	= 0x80000, /* Like implies right but only ensures the action is printed, not manditory */
 	
-	pe_order_test		        = 0x10000 /* test marker */
+	pe_order_test		        = 0x100000 /* test marker */
 };
 
 struct rsc_colocation_s { 
@@ -116,7 +123,7 @@ struct action_wrapper_s
 };
 
 extern gboolean stage0(pe_working_set_t *data_set);
-extern gboolean stage1(pe_working_set_t *data_set);
+extern gboolean probe_resources(pe_working_set_t *data_set);
 extern gboolean stage2(pe_working_set_t *data_set);
 extern gboolean stage3(pe_working_set_t *data_set);
 extern gboolean stage4(pe_working_set_t *data_set);
@@ -170,6 +177,7 @@ extern int custom_action_order(
 extern void graph_element_from_action(
 	action_t *action, pe_working_set_t *data_set);
 
+extern int scores_log_level;
 extern const char* transition_idle_timeout;
 
 #endif
