@@ -256,21 +256,10 @@ gint sort_node_weight(gconstpointer a, gconstpointer b)
 gboolean
 native_assign_node(resource_t *rsc, GListPtr nodes, node_t *chosen)
 {
-	int multiple = 0;
 	CRM_ASSERT(rsc->variant == pe_native);
 
 	clear_bit(rsc->flags, pe_rsc_provisional);
 	
-	slist_iter(candidate, node_t, nodes, lpc, 
-		   crm_debug("Color %s, Node[%d] %s: %d", rsc->id, lpc,
-			     candidate->details->uname, candidate->weight);
-		   if(chosen->weight > 0
-		      && candidate->details->unclean == FALSE
-		      && candidate->weight == chosen->weight) {
-			   multiple++;
-		   }
-		);
-
 	if(chosen == NULL) {
 		crm_debug("Could not allocate a node for %s", rsc->id);
 		rsc->next_role = RSC_ROLE_STOPPED;
@@ -278,7 +267,8 @@ native_assign_node(resource_t *rsc, GListPtr nodes, node_t *chosen)
 
 	} else if(can_run_resources(chosen) == FALSE || chosen->weight < 0) {
 		crm_debug("All nodes for resource %s are unavailable"
-			  ", unclean or shutting down", rsc->id);
+			  ", unclean or shutting down (%s: %d, %d)",
+			  rsc->id, chosen->details->uname, can_run_resources(chosen), chosen->weight);
 		rsc->next_role = RSC_ROLE_STOPPED;
 		return FALSE;
 	}
@@ -287,19 +277,6 @@ native_assign_node(resource_t *rsc, GListPtr nodes, node_t *chosen)
 		rsc->next_role = RSC_ROLE_STARTED;
 	}
 	
-	if(multiple > 1) {
-		int log_level = LOG_INFO;
-		char *score = score2char(chosen->weight);
-		if(chosen->weight >= INFINITY) {
-			log_level = LOG_WARNING;
-		}
-		
-		do_crm_log(log_level, "%d nodes with equal score (%s) for"
-			      " running the listed resources (chose %s):",
-			      multiple, score, chosen->details->uname);
-		crm_free(score);
-	}
-	
 	/* todo: update the old node for each resource to reflect its
 	 * new resource count
 	 */
@@ -324,7 +301,7 @@ native_assign_node(resource_t *rsc, GListPtr nodes, node_t *chosen)
 }
 
 void
-convert_non_atomic_task(resource_t *rsc, order_constraint_t *order)
+convert_non_atomic_task(resource_t *rsc, order_constraint_t *order, gboolean with_notify)
 {
 	int interval = 0;
 	char *rid = NULL;
@@ -332,7 +309,7 @@ convert_non_atomic_task(resource_t *rsc, order_constraint_t *order)
 	int task = no_action;
 	char *old_uuid = order->lh_action_task;
 
-	crm_debug("Processing %s", old_uuid);
+	crm_debug_3("Processing %s", old_uuid);
 	
 	if(order->lh_action_task == NULL
 	   || strstr(order->lh_action_task, "notify") != NULL) {
@@ -369,7 +346,7 @@ convert_non_atomic_task(resource_t *rsc, order_constraint_t *order)
 	}
 	
 	if(task != no_action) {
-		if(is_set(rsc->flags, pe_rsc_notify)) {
+		if(with_notify && is_set(rsc->flags, pe_rsc_notify)) {
 			order->lh_action_task = generate_notify_key(
 				rsc->id, "confirmed-post",
 				task2text(task));
@@ -377,7 +354,7 @@ convert_non_atomic_task(resource_t *rsc, order_constraint_t *order)
 			order->lh_action_task = generate_op_key(
 				rsc->id, task2text(task+1), 0);
 		}
-		crm_debug("Converted %s -> %s",
+		crm_debug_2("Converted %s -> %s",
 			  old_uuid, order->lh_action_task);
 		crm_free(old_uuid);
 	}
@@ -394,7 +371,7 @@ order_actions(
 	action_wrapper_t *wrapper = NULL;
 	GListPtr list = NULL;
 	
-	crm_debug_2("Ordering Action %s before %s",
+	crm_debug_3("Ordering Action %s before %s",
 		    lh_action->uuid, rh_action->uuid);
 
 	log_action(LOG_DEBUG_4, "LH (order_actions)", lh_action, FALSE);
@@ -503,16 +480,6 @@ log_action(unsigned int log_level, const char *pre_text, action_t *action, gbool
 	}
 }
 
-resource_t *uber_parent(resource_t *rsc) 
-{
-	resource_t *parent = rsc;
-	while(parent != NULL && parent->parent != NULL) {
-		parent = parent->parent;
-	}
-	return parent;
-}
-
-
 action_t *get_pseudo_op(const char *name, pe_working_set_t *data_set) 
 {
     action_t *op = NULL;
