@@ -149,7 +149,7 @@ te_fence_node(crm_graph_t *graph, crm_action_t *action)
 	st_op->node_uuid = crm_strdup(uuid);
 	
 	st_op->private_data = generate_transition_key(
-		transition_graph->id, action->id, te_uuid);
+	    transition_graph->id, action->id, 0, te_uuid);
 	
 	CRM_ASSERT(stonithd_input_IPC_channel() != NULL);
 		
@@ -160,6 +160,17 @@ te_fence_node(crm_graph_t *graph, crm_action_t *action)
 	return TRUE;
 }
 
+static int get_target_rc(crm_action_t *action) 
+{
+	const char *target_rc_s = g_hash_table_lookup(
+	    action->params, crm_meta_name(XML_ATTR_TE_TARGET_RC));
+	
+	if(target_rc_s != NULL) {
+		return crm_parse_int(target_rc_s, "0");
+	}
+	return 0;
+}
+
 static gboolean
 te_crm_command(crm_graph_t *graph, crm_action_t *action)
 {
@@ -189,7 +200,7 @@ te_crm_command(crm_graph_t *graph, crm_action_t *action)
 			     CRM_SYSTEM_TENGINE, NULL);
 	
 	counter = generate_transition_key(
-		transition_graph->id, action->id, te_uuid);
+	    transition_graph->id, action->id, get_target_rc(action), te_uuid);
 	crm_xml_add(cmd, XML_ATTR_TRANSITION_KEY, counter);
 	ret = send_ipc_message(crm_ch, cmd);
 	crm_free(counter);
@@ -329,7 +340,8 @@ cib_action_update(crm_action_t *action, int status)
 
 	crm_free(code);
 
-	code = generate_transition_key(transition_graph->id, action->id,te_uuid);
+	code = generate_transition_key(
+	    transition_graph->id, action->id, get_target_rc(action), te_uuid);
 	crm_xml_add(xml_op, XML_ATTR_TRANSITION_KEY, code);
 	crm_free(code);
 
@@ -397,11 +409,11 @@ send_rsc_command(crm_action_t *action)
 	task_uuid = crm_element_value(action->xml, XML_LRM_ATTR_TASK_KEY);
 	on_node = crm_element_value(rsc_op, XML_LRM_ATTR_TARGET);
 	counter = generate_transition_key(
-		transition_graph->id, action->id, te_uuid);
+	    transition_graph->id, action->id, get_target_rc(action), te_uuid);
 	crm_xml_add(rsc_op, XML_ATTR_TRANSITION_KEY, counter);
 
-	crm_info("Initiating action %d: %s on %s",
-		 action->id, task_uuid, on_node);
+	crm_info("Initiating action %d: %s %s on %s",
+		 action->id, task, task_uuid, on_node);
 
 	crm_free(counter);
 	
