@@ -243,8 +243,9 @@ process_te_message(HA_Message *msg, crm_data_t *xml_data, IPC_Channel *sender)
 				INFINITY, tg_restart, "Transition Active", NULL);
 
 		}  else {
+			const char *value = NULL;
 			crm_data_t *graph_data = xml_data;
-			crm_debug("Processing graph derived from %s", graph_input);
+			crm_info("Processing graph derived from %s", graph_input);
 
 			if(graph_file != NULL) {
 				FILE *graph_fd = fopen(graph_file, "r");
@@ -264,6 +265,16 @@ process_te_message(HA_Message *msg, crm_data_t *xml_data, IPC_Channel *sender)
 			start_global_timer(transition_timer,
 					   transition_graph->transition_timeout);
 
+			value = crm_element_value(graph_data, "failed-stop-offset");
+			if(value) {
+			    failed_stop_offset = crm_strdup(value);
+			}
+			
+			value = crm_element_value(graph_data, "failed-start-offset");
+			if(value) {
+			    failed_start_offset = crm_strdup(value);
+			}
+			
 			trigger_graph();
 			print_graph(LOG_DEBUG_2, transition_graph);
 
@@ -291,6 +302,7 @@ void
 tengine_stonith_callback(stonith_ops_t * op)
 {
 	const char *allow_fail  = NULL;
+	int target_rc = -1;
 	int stonith_id = -1;
 	int transition_id = -1;
 	char *uuid = NULL;
@@ -311,7 +323,7 @@ tengine_stonith_callback(stonith_ops_t * op)
 	/* filter out old STONITH actions */
 
 	CRM_CHECK(decode_transition_key(
-			  op->private_data, &uuid, &transition_id, &stonith_id),
+		      op->private_data, &uuid, &transition_id, &stonith_id, &target_rc),
 		  crm_err("Invalid event detected");
 		  goto bail;
 		);
@@ -429,6 +441,16 @@ cib_action_updated(const HA_Message *msg, int call_id, int rc,
 	}
 }
 
+void
+cib_failcount_updated(const HA_Message *msg, int call_id, int rc,
+		   crm_data_t *output, void *user_data)
+{
+	trigger_graph();
+
+	if(rc < cib_ok) {
+		crm_err("Update %d FAILED: %s", call_id, cib_error2string(rc));
+	}
+}
 
 gboolean
 action_timer_callback(gpointer data)
@@ -556,14 +578,10 @@ global_timer_callback(gpointer data)
 gboolean
 te_graph_trigger(gpointer user_data) 
 {
-	int timeout = 0;
-	enum transition_status graph_rc = -1;
-
-	if(transition_graph->complete) {
-		notify_crmd(transition_graph);
-		return TRUE;
-	}
+    int timeout = 0;
+    enum transition_status graph_rc = -1;
 
+    if(transition_graph->complete == FALSE) {
 	graph_rc = run_graph(transition_graph);
 	timeout = transition_graph->transition_timeout;
 	print_graph(LOG_DEBUG_3, transition_graph);
@@ -583,11 +601,12 @@ te_graph_trigger(gpointer user_data)
 		crm_err("Transition failed: %s", transition_status(graph_rc));
 		print_graph(LOG_WARNING, transition_graph);
 	}
-	
-	transition_graph->complete = TRUE;
-	notify_crmd(transition_graph);
-
-	return TRUE;	
+    }
+    
+    transition_graph->complete = TRUE;
+    notify_crmd(transition_graph);
+    
+    return TRUE;	
 }
 
 
