@@ -29,10 +29,13 @@
 #include <clplumbing/Gmain_timeout.h>
 #include <lrm/lrm_api.h>
 
+char *failed_stop_offset = NULL;
+char *failed_start_offset = NULL;
+
 crm_data_t *need_abort(crm_data_t *update);
 void process_graph_event(crm_data_t *event, const char *event_node);
 int match_graph_event(int action_id, crm_data_t *event, const char *event_node,
-		      int op_status, int op_rc);
+		      int op_status, int op_rc, int target_rc);
 
 crm_data_t *
 need_abort(crm_data_t *update)
@@ -106,7 +109,7 @@ fail_incompletable_actions(crm_graph_t *graph, const char *down_node)
 				continue;
 			}
 			
-			target = crm_element_value(action->xml, XML_LRM_ATTR_TARGET);
+			target = crm_element_value(action->xml, XML_LRM_ATTR_TARGET_UUID);
 			if(safe_str_eq(target, down_node)) {
 				action->failed = TRUE;
 				last_action = action->xml;
@@ -214,7 +217,7 @@ extract_event(crm_data_t *msg)
 }
 
 static void
-update_failcount(crm_data_t *event, const char *event_node, int rc) 
+update_failcount(crm_data_t *event, const char *event_node, int rc, int target_rc) 
 {
 	int interval = 0;
 	char *task = NULL;
@@ -222,13 +225,24 @@ update_failcount(crm_data_t *event, const char *event_node, int rc)
 	char *attr_name = NULL;
 	const char *id  = ID(event);
 	const char *on_uuid  = event_node;
-	const char *value = XML_NVPAIR_ATTR_VALUE"++";
+	const char *value = NULL;
 
 	if(rc == 99) {
 		/* this is an internal code for "we're busy, try again" */
 		return;
+
+	} else if(rc == target_rc) {
+	    return;
 	}
 
+	if(failed_stop_offset == NULL) {
+	    failed_stop_offset = crm_strdup(INFINITY_S);
+	}
+
+	if(failed_start_offset == NULL) {
+	    failed_start_offset = crm_strdup(INFINITY_S);
+	}
+	
 	CRM_CHECK(on_uuid != NULL, return);
 
 	CRM_CHECK(parse_op_key(id, &rsc_id, &task, &interval),
@@ -237,20 +251,30 @@ update_failcount(crm_data_t *event, const char *event_node, int rc)
 	CRM_CHECK(task != NULL, goto bail);
 	CRM_CHECK(rsc_id != NULL, goto bail);
 
-	if(safe_str_eq(task, CRMD_ACTION_START)
-	   || safe_str_eq(task, CRMD_ACTION_STOP)) {
+	if(safe_str_eq(task, CRMD_ACTION_START)) {
 	    interval = 1;
-	    value = "INFINITY";
+	    value = failed_start_offset;
+
+	} else if(safe_str_eq(task, CRMD_ACTION_STOP)) {
+	    interval = 1;
+	    value = failed_stop_offset;
 	}
-	
+
+	if(value == NULL || safe_str_neq(value, INFINITY_S)) {
+	    value = XML_NVPAIR_ATTR_VALUE"++";
+	}
+
 	if(interval > 0) {
+		int call_id = 0;
 		attr_name = crm_concat("fail-count", rsc_id, '-');
-		crm_warn("Updating failcount for %s on %s after failed %s: rc=%d",
-			 rsc_id, on_uuid, task, rc);
-	
-		update_attr(te_cib_conn, cib_none, XML_CIB_TAG_STATUS,
-			    on_uuid, NULL,NULL, attr_name,
-			    XML_NVPAIR_ATTR_VALUE"++", FALSE);
+		crm_warn("Updating failcount for %s on %s after failed %s: rc=%d (update=%s)",
+			 rsc_id, on_uuid, task, rc, value);
+
+		call_id = update_attr(te_cib_conn, cib_inhibit_notify, XML_CIB_TAG_STATUS,
+			    on_uuid, NULL,NULL, attr_name, value, FALSE);
+
+		/* don't let notificatios of these updates cause new transitions */
+		add_cib_op_callback(call_id, FALSE, NULL, cib_failcount_updated);
 		crm_free(attr_name);
 	}
 
@@ -260,18 +284,9 @@ update_failcount(crm_data_t *event, const char *event_node, int rc)
 }
 
 static int
-status_from_rc(crm_action_t *action, int orig_status, int rc)
+status_from_rc(crm_action_t *action, int orig_status, int rc, int target_rc)
 {
-	int target_rc = 0;
 	int status = orig_status;
-	const char *target_rc_s = g_hash_table_lookup(
-		action->params, crm_meta_name(XML_ATTR_TE_TARGET_RC));
-
-	if(target_rc_s != NULL) {
-		crm_debug_2("Target rc: %s vs. %d", target_rc_s, rc);
-		target_rc = crm_parse_int(target_rc_s, NULL);
-	}
-
 	if(target_rc == rc) {
 	    crm_debug_2("Target rc: == %d", rc);
 	    if(status != LRM_OP_DONE) {
@@ -294,8 +309,8 @@ status_from_rc(crm_action_t *action, int orig_status, int rc)
 		const char *task, *uname;
 		task = crm_element_value(action->xml, XML_LRM_ATTR_TASK);
 		uname  = crm_element_value(action->xml, XML_LRM_ATTR_TARGET);
-		crm_warn("Action %s on %s failed (target: %s vs. rc: %d): %s",
-			 task, uname, crm_str(target_rc_s), rc, op_status2text(status));
+		crm_warn("Action %d (%s) on %s failed (target: %d vs. rc: %d): %s",
+			 action->id, task, uname, target_rc, rc, op_status2text(status));
 	}
 
 	return status;
@@ -309,7 +324,7 @@ status_from_rc(crm_action_t *action, int orig_status, int rc)
  */
 int
 match_graph_event(int action_id, crm_data_t *event, const char *event_node,
-		  int op_status, int op_rc)
+		  int op_status, int op_rc, int target_rc)
 {
 	const char *target = NULL;
 	const char *allow_fail = NULL;
@@ -321,9 +336,9 @@ match_graph_event(int action_id, crm_data_t *event, const char *event_node,
 		return -1;
 	}
 	
-	op_status = status_from_rc(action, op_status, op_rc);
+	op_status = status_from_rc(action, op_status, op_rc, target_rc);
 	if(op_status != LRM_OP_DONE) {
-		update_failcount(event, event_node, op_rc);
+	    update_failcount(event, event_node, op_rc, target_rc);
 	}
 	
 	/* Process OP status */
@@ -477,6 +492,7 @@ process_graph_event(crm_data_t *event, const char *event_node)
 	int status = -1;
 
 	int action = -1;
+	int target_rc = -1;
 	int transition_num = -1;
 	char *update_te_uuid = NULL;
 
@@ -496,7 +512,7 @@ process_graph_event(crm_data_t *event, const char *event_node)
 	
 	CRM_CHECK(decode_transition_magic(
 			  magic, &update_te_uuid, &transition_num, &action,
-			  &status, &rc),
+			  &status, &rc, &target_rc),
 		  crm_err("Invalid event %s detected", id);
 		  abort_transition(INFINITY, tg_restart,"Bad event", event);
 		);
@@ -506,11 +522,13 @@ process_graph_event(crm_data_t *event, const char *event_node)
 	}
 	
 	if(transition_num == -1) {
-		crm_err("Action %s initiated outside of a transition", id);
+		crm_err("Action %s (%s) initiated outside of a transition",
+			id, magic);
 		abort_transition(INFINITY, tg_restart,"Unexpected event",event);
 
 	} else if(action < 0 || safe_str_neq(update_te_uuid, te_uuid)) {
-		crm_info("Action %s initiated by a different transitioner", id);
+		crm_info("Action %s (%s) initiated by a different transitioner",
+			 id, magic);
 		abort_transition(INFINITY, tg_restart,"Foreign event", event);
 		
 	} else if(transition_graph->id != transition_num) {
@@ -523,7 +541,7 @@ process_graph_event(crm_data_t *event, const char *event_node)
 		abort_transition(INFINITY, tg_restart, "Inactive graph", event);
 
 	} else if(match_graph_event(
-			  action, event, event_node, status, rc) < 0) {
+		      action, event, event_node, status, rc, target_rc) < 0) {
 		crm_err("Unknown graph action %s", id);
 		abort_transition(INFINITY, tg_restart, "Unknown event", event);
 
@@ -533,7 +551,7 @@ process_graph_event(crm_data_t *event, const char *event_node)
 	}
 
 	if(passed == FALSE && rc != EXECRA_OK) {
-		update_failcount(event, event_node, rc);
+	    update_failcount(event, event_node, rc, target_rc);
 	}
 
   bail:
