@@ -68,5 +68,8 @@ extern GTRIGSource *transition_trigger;
 extern GTRIGSource *stonith_reconnect;
 extern crm_action_timer_t *transition_timer;
 
+extern char *failed_stop_offset;
+extern char *failed_start_offset;
+
 #endif
 
