@@ -69,22 +69,22 @@ class LinuxHAv2(HeartbeatCM):
             "StableTime"     : 30,
             "StartCmd"       : "/etc/init.d/heartbeat start > /dev/null 2>&1",
             "StopCmd"        : "/etc/init.d/heartbeat stop  > /dev/null 2>&1",
-            "ElectionCmd"    : "/usr/sbin/crmadmin -E %s",
-            "StatusCmd"      : "/usr/sbin/crmadmin -S %s 2>/dev/null",
-            "EpocheCmd"      : "/usr/sbin/ccm_tool -e",
-            "QuorumCmd"      : "/usr/sbin/ccm_tool -q",
-            "CibQuery"       : "/usr/sbin/cibadmin -Ql",
-            "ParitionCmd"    : "/usr/sbin/ccm_tool -p",
-            "IsRscRunning"   : "/usr/lib/heartbeat/lrmadmin -E %s monitor 0 0 EVERYTIME 2>/dev/null|grep return",
-            "ExecuteRscOp"   : "/usr/lib/heartbeat/lrmadmin -n %s -E %s %s 0 %d EVERYTIME 2>/dev/null",
-            "CIBfile"        : "%s:/var/lib/heartbeat/crm/cib.xml",
+            "ElectionCmd"    : "/usr/local/sbin/crmadmin -E %s",
+            "StatusCmd"      : "/usr/local/sbin/crmadmin -S %s 2>/dev/null",
+            "EpocheCmd"      : "/usr/local/sbin/ccm_tool -e",
+            "QuorumCmd"      : "/usr/local/sbin/ccm_tool -q",
+            "CibQuery"       : "/usr/local/sbin/cibadmin -Ql",
+            "ParitionCmd"    : "/usr/local/sbin/ccm_tool -p",
+            "IsRscRunning"   : "/usr/local/lib/heartbeat/lrmadmin -E %s monitor 0 0 EVERYTIME 2>/dev/null|grep return",
+            "ExecuteRscOp"   : "/usr/local/lib/heartbeat/lrmadmin -n %s -E %s %s 0 %d EVERYTIME 2>/dev/null",
+            "CIBfile"        : "%s:/usr/local/var/lib/heartbeat/crm/cib.xml",
             "TmpDir"         : "/tmp",
-            "BreakCommCmd2"  : "/usr/share/heartbeat/TestHeartbeatComm break-communication %s>/dev/null 2>&1",
+            "BreakCommCmd2"  : "/usr/local/share/heartbeat/TestHeartbeatComm break-communication %s>/dev/null 2>&1",
             "IsIPAddrRscRunning"   : "",
 
-            "StandbyCmd"   : "/usr/sbin/crm_standby -U %s -v %s 2>/dev/null",
-            "UUIDQueryCmd"   : "/usr/sbin/crmadmin -N",
-            "StandbyQueryCmd"    : "/usr/sbin/crm_standby -GQ -U %s 2>/dev/null",
+            "StandbyCmd"   : "/usr/local/sbin/crm_standby -U %s -v %s 2>/dev/null",
+            "UUIDQueryCmd"   : "/usr/local/sbin/crmadmin -N",
+            "StandbyQueryCmd"    : "/usr/local/sbin/crm_standby -GQ -U %s 2>/dev/null",
 
             # Patterns to look for in the log files for various occasions...
             "Pat:DC_IDLE"      : "crmd.*State transition.*-> S_IDLE",
@@ -160,10 +160,10 @@ class LinuxHAv2(HeartbeatCM):
 
         if not self.CIBsync.has_key(node) and self.Env["ClobberCIB"] == 1:
             self.CIBsync[node] = 1
-            self.rsh.remote_py(node, "os", "system", "rm -f /var/lib/heartbeat/crm/cib.xml")
-            self.rsh.remote_py(node, "os", "system", "rm -f /var/lib/heartbeat/crm/cib.xml.sig")
-            self.rsh.remote_py(node, "os", "system", "rm -f /var/lib/heartbeat/crm/cib.xml.last")
-            self.rsh.remote_py(node, "os", "system", "rm -f /var/lib/heartbeat/crm/cib.xml.sig.last")
+            self.rsh.remote_py(node, "os", "system", "rm -f /usr/local/var/lib/heartbeat/crm/cib.xml")
+            self.rsh.remote_py(node, "os", "system", "rm -f /usr/local/var/lib/heartbeat/crm/cib.xml.sig")
+            self.rsh.remote_py(node, "os", "system", "rm -f /usr/local/var/lib/heartbeat/crm/cib.xml.last")
+            self.rsh.remote_py(node, "os", "system", "rm -f /usr/local/var/lib/heartbeat/crm/cib.xml.sig.last")
 
             # Only install the CIB on the first node, all the other ones will pick it up from there
             if self.cib_installed == 1:
@@ -178,7 +178,7 @@ class LinuxHAv2(HeartbeatCM):
                 os.system("rm -f "+cib_file)
                 self.debug("Creating new CIB for " + node + " in: " + cib_file)
                 os.system("echo \'" + self.default_cts_cib + "\' > " + cib_file)
-                if 0!=self.rsh.echo_cp(None, cib_file, node, "/var/lib/heartbeat/crm/cib.xml"):
+                if 0!=self.rsh.echo_cp(None, cib_file, node, "/usr/local/var/lib/heartbeat/crm/cib.xml"):
                     raise ValueError("Can not create CIB on %s "%node)
 
                 os.system("rm -f "+cib_file)
@@ -187,7 +187,7 @@ class LinuxHAv2(HeartbeatCM):
                 if 0!=self.rsh.cp(self.Env["CIBfilename"], "root@" + (self["CIBfile"]%node)):
                     raise ValueError("Can not scp file to %s "%node)
         
-            self.rsh.remote_py(node, "os", "system", "chown hacluster /var/lib/heartbeat/crm/cib.xml")
+            self.rsh.remote_py(node, "os", "system", "chown hacluster /usr/local/var/lib/heartbeat/crm/cib.xml")
 
     def prepare(self):
         '''Finish the Initialization process. Prepare to test...'''
@@ -314,7 +314,7 @@ class LinuxHAv2(HeartbeatCM):
 
     def active_resources(self, node):
         (rc, output) = self.rsh.remote_py(
-            node, "os", "system", """/usr/sbin/crm_mon -1 | grep "Started %s" """ % node)
+            node, "os", "system", """/usr/local/sbin/crm_mon -1 | grep "Started %s" """ % node)
 
         resources = []
         for line in output:
@@ -365,7 +365,7 @@ class LinuxHAv2(HeartbeatCM):
                 os.system("cp "+self.Env["CIBfilename"]+" "+cib_file)
         else:            
             if 0 != self.rsh.echo_cp(
-                self.Env["nodes"][0], "/var/lib/heartbeat/crm/cib.xml", None, cib_file):
+                self.Env["nodes"][0], "/usr/local/var/lib/heartbeat/crm/cib.xml", None, cib_file):
                 raise ValueError("Can not copy file to %s, maybe permission denied"%cib_file)
 
         self.config = parse(cib_file)
