@@ -43,7 +43,7 @@ class HeartbeatCM(ClusterManager):
 
     def __init__(self, Environment, randseed=None):
 
-        self.ResourceDirs = ["/etc/ha.d/resource.d", "/etc/rc.d/init.d", "/etc/rc.d/"]
+        self.ResourceDirs = ["/usr/local/etc/ha.d/resource.d", "/usr/local/etc/rc.d/init.d", "/usr/local/etc/rc.d/"]
         self.ResourceFile = Environment["HAdir"] + "/haresources"
         self.ConfigFile = Environment["HAdir"]+ "/ha.cf"
         ClusterManager.__init__(self, Environment, randseed=randseed)
@@ -51,23 +51,23 @@ class HeartbeatCM(ClusterManager):
             "Name"           : "heartbeat",
             "DeadTime"       : 30,
             "StableTime"     : 30,
-            "StartCmd"       : "/usr/lib/heartbeat/ha_logd -d >/dev/null 2>&1; MALLOC_CHECK_=2 /usr/lib/heartbeat/heartbeat >/dev/null 2>&1",
-            "StopCmd"        : "/usr/lib/heartbeat/heartbeat -k",
-            "StatusCmd"      : "/usr/lib/heartbeat/heartbeat -s",
-            "RereadCmd"      : "/usr/lib/heartbeat/heartbeat -r",
-            "StartDRBDCmd"   : "/etc/init.d/drbd start >/dev/null 2>&1",
-            "StopDRBDCmd"    : "/etc/init.d/drbd stop",
-            "StatusDRBDCmd"  : "/etc/init.d/drbd status",
-            "DRBDCheckconf"  : "/etc/init.d/drbd checkconfig >/var/run/drbdconf 2>&1",
-            "BreakCommCmd"   : "/usr/share/heartbeat/TestHeartbeatComm break-communication >/dev/null 2>&1",
-            "FixCommCmd"     : "/usr/share/heartbeat/TestHeartbeatComm fix-communication >/dev/null 2>&1",
-            "DelFileCommCmd" : "/usr/share/heartbeat/TestHeartbeatComm delete-testingfile >/dev/null 2>&1",
-            "SaveFileCmd"    : "/usr/share/heartbeat/TestHeartbeatComm save-testingfile /tmp/OnlyForTesting >/dev/null 2>&1",
-            "ReduceCommCmd"  : "/usr/share/heartbeat/TestHeartbeatComm reduce-communication %s %s>/dev/null 2>&1",
-            "RestoreCommCmd" : "/usr/share/heartbeat/TestHeartbeatComm restore-communication /tmp/OnlyForTesting >/dev/null 2>&1",
-            "IPaddrCmd"      : "/etc/ha.d/resource.d/IPaddr %s status",
-            "Standby"        : "/usr/share/heartbeat/hb_standby >/dev/null 2>&1",
-            "TestConfigDir"  : "/etc/ha.d/testconfigs",
+            "StartCmd"       : "/usr/local/lib/heartbeat/ha_logd -d >/dev/null 2>&1; MALLOC_CHECK_=2 /usr/local/lib/heartbeat/heartbeat >/dev/null 2>&1",
+            "StopCmd"        : "/usr/local/lib/heartbeat/heartbeat -k",
+            "StatusCmd"      : "/usr/local/lib/heartbeat/heartbeat -s",
+            "RereadCmd"      : "/usr/local/lib/heartbeat/heartbeat -r",
+            "StartDRBDCmd"   : "/usr/local/etc/init.d/drbd start >/dev/null 2>&1",
+            "StopDRBDCmd"    : "/usr/local/etc/init.d/drbd stop",
+            "StatusDRBDCmd"  : "/usr/local/etc/init.d/drbd status",
+            "DRBDCheckconf"  : "/usr/local/etc/init.d/drbd checkconfig >/var/run/drbdconf 2>&1",
+            "BreakCommCmd"   : "/usr/local/share/heartbeat/TestHeartbeatComm break-communication >/dev/null 2>&1",
+            "FixCommCmd"     : "/usr/local/share/heartbeat/TestHeartbeatComm fix-communication >/dev/null 2>&1",
+            "DelFileCommCmd" : "/usr/local/share/heartbeat/TestHeartbeatComm delete-testingfile >/dev/null 2>&1",
+            "SaveFileCmd"    : "/usr/local/share/heartbeat/TestHeartbeatComm save-testingfile /tmp/OnlyForTesting >/dev/null 2>&1",
+            "ReduceCommCmd"  : "/usr/local/share/heartbeat/TestHeartbeatComm reduce-communication %s %s>/dev/null 2>&1",
+            "RestoreCommCmd" : "/usr/local/share/heartbeat/TestHeartbeatComm restore-communication /tmp/OnlyForTesting >/dev/null 2>&1",
+            "IPaddrCmd"      : "/usr/local/etc/ha.d/resource.d/IPaddr %s status",
+            "Standby"        : "/usr/local/share/heartbeat/hb_standby >/dev/null 2>&1",
+            "TestConfigDir"  : "/usr/local/etc/ha.d/testconfigs",
             "LogFileName"    : Environment["LogFileName"],
 
             # Patterns to look for in the log files for various occasions...
@@ -108,15 +108,15 @@ class HeartbeatCM(ClusterManager):
         for j in *
         do
           if
-            [ -f "/etc/ha.d/$j" ];
+            [ -f "/usr/local/etc/ha.d/$j" ];
           then
             if
-              cmp $j /etc/ha.d/$j >/dev/null 2>&1;
+              cmp $j /usr/local/etc/ha.d/$j >/dev/null 2>&1;
             then
               : Config file $j is already up to correct.
             else
               echo "Touching $j"
-              cp $j /etc/ha.d/$j
+              cp $j /usr/local/etc/ha.d/$j
             fi
           fi
         done
