@@ -195,7 +195,7 @@ class RemoteExec:
            If the call fail, lastrc == 1 and return the reason (string)
         '''
         encode_args = binascii.b2a_base64(pickle.dumps(args))
-        encode_cmd = string.join(["/usr/share/heartbeat/cts/CTSproxy.py",module,func,encode_args])
+        encode_cmd = string.join(["/usr/local/share/heartbeat/cts/CTSproxy.py",module,func,encode_args])
 
         #print "%s: %s.%s %s" % (node, module, func, repr(args))
         result = self.readlines(node, encode_cmd)
@@ -431,7 +431,7 @@ class ClusterManager(UserDict):
         self.data = {
             "up"             : "up",        # Status meaning up
             "down"           : "down",  # Status meaning down
-            "StonithCmd"     : "/usr/sbin/stonith -t baytech -p '10.10.10.100 admin admin' %s",
+            "StonithCmd"     : "/usr/local/sbin/stonith -t baytech -p '10.10.10.100 admin admin' %s",
             "DeadTime"       : 30,        # Max time to detect dead node...
             "StartTime"      : 90,        # Max time to start up
     #
@@ -507,7 +507,7 @@ class ClusterManager(UserDict):
                 if self.ShouldBeStatus[node] == self["down"]:
                     self.debug("Removing cache file on: "+node)
                     self.rsh.remote_py(node, "os", "system", 
-                                       "rm -f /var/lib/heartbeat/hostcache")
+                                       "rm -f /usr/local/var/lib/heartbeat/hostcache")
                 else:
                     self.debug("NOT Removing cache file on: "+node)
 
@@ -546,7 +546,7 @@ class ClusterManager(UserDict):
         if self.clear_cache:
             self.debug("Removing cache file on: "+node)
             self.rsh.remote_py(node, "os", "system", 
-                               "rm -f /var/lib/heartbeat/hostcache")
+                               "rm -f /usr/local/var/lib/heartbeat/hostcache")
 
         if self.rsh(node, self["StartCmd"]) != 0:
             self.log ("Warn: Start command failed on node %s" %(node))
@@ -580,7 +580,7 @@ class ClusterManager(UserDict):
         if self.clear_cache:
             self.debug("Removing cache file on: "+node)
             self.rsh.remote_py(node, "os", "system", 
-                               "rm -f /var/lib/heartbeat/hostcache")
+                               "rm -f /usr/local/var/lib/heartbeat/hostcache")
 
         self.rsh.noBlock(node, self["StartCmd"])
         self.ShouldBeStatus[node]=self["up"]
@@ -951,7 +951,7 @@ class Process(Component):
         
     def kill(self, node):
         if self.CM.rsh(node, self.KillCmd) != 0:
-            self.CM.log ("ERROR: Kill %s failed on node %s" %(name,node))
+            self.CM.log ("ERROR: Kill %s failed on node %s" %(self.name,node))
             return None
         return 1
 
