@@ -454,7 +454,7 @@ class CIBAudit(ClusterAudit):
                         #self.CM.debug("Retrieved CIB: %s" % first_host_xml) 
                 else:
                         a_host_xml = self.store_remote_cib(a_host)
-                        diff_cmd="/usr/sbin/crm_diff -c -VV -f -N \'%s\' -O '%s'" % (a_host_xml, first_host_xml)
+                        diff_cmd="/usr/local/sbin/crm_diff -c -VV -f -N \'%s\' -O '%s'" % (a_host_xml, first_host_xml)
 
                         infile, outfile, errfile = os.popen3(diff_cmd)
                         diff_lines = outfile.readlines()
