@@ -51,7 +51,7 @@ class ResetMechanism:
 
 class Stonith(ResetMechanism):
     def __init__(self, sttype="external/ssh", pName=None, pValue=None
-        ,        path="/usr/sbin/stonith"):
+        ,        path="/usr/local/sbin/stonith"):
         self.pathname=path
         self.configName=pName
         self.configValue=pValue
@@ -71,15 +71,15 @@ class Stonithd(ResetMechanism):
         self.sttype = sttype
         self.nodes = nodes
 
-        self.query_cmd_pat   = '/usr/lib/heartbeat/stonithdtest/apitest 0 %s 60000 0'
-        self.reset_cmd_pat   = '/usr/lib/heartbeat/stonithdtest/apitest 1 %s 60000 0'
-        self.poweron_cmd_pat = '/usr/lib/heartbeat/stonithdtest/apitest 2 %s 60000 0'
-        self.poweroff_cmd_pat= '/usr/lib/heartbeat/stonithdtest/apitest 3 %s 60000 0'
+        self.query_cmd_pat   = '/usr/local/lib/heartbeat/stonithdtest/apitest 0 %s 60000 0'
+        self.reset_cmd_pat   = '/usr/local/lib/heartbeat/stonithdtest/apitest 1 %s 60000 0'
+        self.poweron_cmd_pat = '/usr/local/lib/heartbeat/stonithdtest/apitest 2 %s 60000 0'
+        self.poweroff_cmd_pat= '/usr/local/lib/heartbeat/stonithdtest/apitest 3 %s 60000 0'
 
-        self.lrmd_add_pat    = '/usr/lib/heartbeat/lrmadmin -A %s stonith ' + sttype + ' NULL hostlist=%s livedangerously=yes'
-        self.lrmd_start_pat  = '/usr/lib/heartbeat/lrmadmin -E %s start 0 0 EVERYTIME'
-        self.lrmd_stop_pat   = '/usr/lib/heartbeat/lrmadmin -E %s stop 0 0 EVERYTIME'
-        self.lrmd_del_pat    = '/usr/lib/heartbeat/lrmadmin -D %s'
+        self.lrmd_add_pat    = '/usr/local/lib/heartbeat/lrmadmin -A %s stonith ' + sttype + ' NULL hostlist=%s livedangerously=yes'
+        self.lrmd_start_pat  = '/usr/local/lib/heartbeat/lrmadmin -E %s start 0 0 EVERYTIME'
+        self.lrmd_stop_pat   = '/usr/local/lib/heartbeat/lrmadmin -E %s stop 0 0 EVERYTIME'
+        self.lrmd_del_pat    = '/usr/local/lib/heartbeat/lrmadmin -D %s'
 
         self.rsc_id          = 'my_stonithd_id'
 
@@ -347,7 +347,7 @@ class CtsLab(UserDict):
         if not self.has_key("CMclass"):
             self["CMclass"] = HeartbeatCM
         if not self.has_key("LogFileName"):
-            self["LogFileName"] = "/var/log/ha-log"
+            self["LogFileName"] = "/usr/local/var/log/ha-log"
         if not self.has_key("logrestartcmd"):
             self["logrestartcmd"] = "/etc/init.d/syslog restart"
         if not self.has_key("logfacility"):
@@ -509,8 +509,8 @@ if __name__ == '__main__':
     from CTS import Scenario, InitClusterManager, PingFest, PacketLoss, BasicSanityCheck
     import CM_hb
 
-    HAdir = "/etc/ha.d"
-    LogFile = "/var/log/ha-log-"+DefaultFacility
+    HAdir = "/usr/local/etc/ha.d"
+    LogFile = "/usr/local/var/log/ha-log-"+DefaultFacility
     DoStonith = 1
     DoStandby = 1
     DoFencing = 1
@@ -779,7 +779,7 @@ if __name__ == '__main__':
     cm.log("Cluster nodes: ")
     for node in config.Parameters["node"]:
         (rc, lines) = cm.rsh.remote_py(node, "os", "system", 
-                                     "/usr/sbin/crm_uuid")
+                                     "/usr/local/sbin/crm_uuid")
         if not lines:
             cm.log("    * %s: __undefined_uuid__" % node)
         else:
