@@ -1220,7 +1220,7 @@ class Fastdetection(CTSTest):
             if stopwatch.look():
                 Stoptime = os.times()[4]
 #        This test is CM-specific - FIXME!!
-                self.CM.rsh(node, "killall -9 /usr/lib/heartbeat/ccm /usr/lib/heartbeat/ipfail >/dev/null 2>&1; true")
+                self.CM.rsh(node, "killall -9 /usr/local/lib/heartbeat/ccm /usr/local/lib/heartbeat/ipfail >/dev/null 2>&1; true")
                 Detectiontime = Stoptime-Starttime
                 detectms = int(Detectiontime*1000+0.5)
                 self.CM.log("...failure detection time: %d ms" % detectms)
@@ -1236,7 +1236,7 @@ class Fastdetection(CTSTest):
                 return self.success()
             else:
 #        This test is CM-specific - FIXME!!
-                self.CM.rsh(node, "killall -9 /usr/lib/heartbeat/ccm /usr/lib/heartbeat/ipfail >/dev/null 2>&1; true")
+                self.CM.rsh(node, "killall -9 /usr/local/lib/heartbeat/ccm /usr/local/lib/heartbeat/ipfail >/dev/null 2>&1; true")
                 self.CM.ShouldBeStatus[node] = self.CM["down"]
                 ret=self.start(node)
                 return self.failure("Didn't find the log message")
@@ -1698,7 +1698,7 @@ class ResourceRecover(CTSTest):
         
         # fail a resource by calling an action it doesn't support
         self.CM.rsh.remote_py(node, "os", "system",
-                              "/usr/sbin/crm_resource -F -r %s -H %s &>/dev/null" % (self.rid, node))
+                              "/usr/local/sbin/crm_resource -F -r %s -H %s &>/dev/null" % (self.rid, node))
 
         watch.lookforall()
 
@@ -2279,7 +2279,7 @@ class BSC_AddResource(CTSTest):
         CTSTest.__init__(self, cm)
         self.name="AddResource"
         self.resource_offset = 0
-        self.cib_cmd="""/usr/sbin/cibadmin -C -o %s -X '%s' """
+        self.cib_cmd="""/usr/local/sbin/cibadmin -C -o %s -X '%s' """
 
     def __call__(self, node):
         self.resource_offset =         self.resource_offset  + 1
