@@ -1,4 +1,3 @@
-/* $Id: module.c,v 1.53.2.1 2004/04/20 05:05:28 alan Exp $ */
 /*
  * module: Dynamic module support code
  *
@@ -7,23 +6,23 @@
  * 
  * Thanks to Conectiva S.A. for sponsoring Marcelo Tosatti work
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- * 
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
-#include <portability.h>
+#include <lha_internal.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -48,7 +47,7 @@
 #include <pils/generic.h>
 #include <HBcomm.h>
 #include <hb_config.h>
-#include <clplumbing/ttylock.h>
+#include <stonith/st_ttylock.h>
 
 #ifndef RTLD_NOW
 #	define RTLD_NOW 0
@@ -70,8 +69,8 @@ static void		RegisterCleanup(void(*)(void));
 struct hb_media_imports	CommImports =
 {	GetParameterValue	/* So plugins can get option values */
 ,	RegisterNewMedium
-,	ttylock
-,	ttyunlock
+,	st_ttylock
+,	st_ttyunlock
 ,	StringToBaud
 ,	RegisterCleanup
 ,	hb_signal_process_pending
@@ -102,8 +101,10 @@ module_init(void)
 	if (initialised) {
 		return HA_FAIL;
 	}
+#ifdef DLPREOPEN
 	/* Initialize libltdl's list of preloaded modules */
 	LTDL_SET_PRELOADED_SYMBOLS();
+#endif
 
 #if 0
 	/* Initialize ltdl */
@@ -118,7 +119,7 @@ module_init(void)
 	}
 
 	if (DEBUGDETAILS) {
- 		PILSetDebugLevel(PluginLoadingSystem, NULL, NULL, debug);
+ 		PILSetDebugLevel(PluginLoadingSystem, NULL, NULL, debug_level);
 	}
 
 
@@ -132,7 +133,7 @@ module_init(void)
 		,	PIL_strerror(rc));
 		return HA_FAIL;
 	}
- 	PILSetDebugLevel(PluginLoadingSystem, NULL, NULL, debug);
+ 	PILSetDebugLevel(PluginLoadingSystem, NULL, NULL, debug_level);
 
 	/* init completed */
 	++initialised;
@@ -154,6 +155,7 @@ RegisterNewMedium(struct hb_media* mp)
  * file so they can be passed to the plugins their use. This avoids coupling
  * through global variables which is problematic for plugins on some platforms.
  */
+#define		PREFIX "HA_"
 void
 SetParameterValue(const char * name, const char * value)
 {
@@ -161,6 +163,7 @@ SetParameterValue(const char * name, const char * value)
 	char *	valdup;
 	void *	gname;
 	void *	gval;
+	int	name_len = strlen(name?name:"");
 
 	if (Parameters == NULL) {
 		Parameters = g_hash_table_new(g_str_hash, g_str_equal);
@@ -179,6 +182,25 @@ SetParameterValue(const char * name, const char * value)
 	namedup = g_strdup(name);
 	valdup = g_strdup(value);
 	g_hash_table_insert(Parameters, namedup, valdup);
+
+	if(name_len > 0) {
+		char *  env_name = cl_malloc(name_len + STRLEN_CONST(PREFIX)+1);		
+		if (env_name == NULL){
+			cl_log(LOG_ERR, "SetParameterValue():"
+			       "setenv() memory allocation failed.");
+			return;
+		}
+		snprintf(env_name, name_len+4, PREFIX "%s", name);
+		env_name[name_len+STRLEN_CONST(PREFIX)] = EOS;
+		/*
+		 * It is unclear whether any given version of setenv
+		 * makes a copy of the name or value, or both.
+		 * Therefore it is UNSAFE to free either one.
+		 * Fortunately the size of the resulting potential memory leak
+		 * is small for this particular situation.
+		 */
+		setenv(env_name, value, 1);
+	}
 }
 /* 
  * GetParameterValue() provides information from the configuration file
