@@ -1,4 +1,3 @@
-/* $Id: oc_event.h,v 1.1.4.1 2004/04/20 07:57:54 alan Exp $ */
 /*
  * oc_event.h
  *
@@ -50,7 +49,7 @@ typedef uint32_t  oc_ed_t;
 
 #define OC_EV_CLASS_SHIFT	22
 #define OC_EV_EVENT_SHIFT	10
-#define	OC_EV_EVENT_MASK	(~ ((uint)~0 << OC_EV_CLASS_SHIFT))
+#define	OC_EV_EVENT_MASK	(~ (~((uint)0) << OC_EV_CLASS_SHIFT))
 
 #define	OC_EV_GET_CLASS(ed)	((uint)(ed) >> OC_EV_CLASS_SHIFT)
 #define	OC_EV_GET_EVENT(ed)	((uint)(ed) & OC_EV_EVENT_MASK)
@@ -84,9 +83,28 @@ typedef enum {
 } oc_conn_event_t;
 
 
-/*
- * Node Membership Events
+/* Node Membership Events
+ * (see http://wiki.linux-ha.org/CCM/MembershipCallback for more info)
+ *
+ *OC_EV_MS_NEW_MEMBERSHIP 
+ *	CCM: membership with quorum 
+ *
+ *OC_EV_MS_MS_INVALID 
+ *	CCM: membership without quorum 
+ *
+ *OC_EV_MS_NOT_PRIMARY 
+ *	CCM: old membership (not valid any longer) 
+ *
+ *OC_EV_MS_PRIMARY_RESTORED 
+ *	This event mean the cluster restores to a stable state that has the same membership as before. 
+ *	It also implies it has the same quorum as before. 
+ *	CCM: old membership restored (same membership as before) 
+ *
+ *OC_EV_MS_EVICTED 
+ *	CCM: the client is evicted from ccm. 
+ *
  */
+
 typedef enum {
 	OC_EV_MS_INVALID = OC_EV_SET_CLASS(OC_EV_MEMB_CLASS, 0),
 	OC_EV_MS_NEW_MEMBERSHIP,
