@@ -367,7 +367,7 @@ typedef PIL_rc(*PILInterfaceFun)(PILInterface*, void* ud_interface);
 #define IS_PILINTERFACEUNIV(s)	((s)->MagicNum == PIL_MAGIC_INTERFACEUNIV)
 
 /* This is the place where PILS installs its own plugins */
-#define	PILS_BASE_PLUGINDIR	"/usr/lib/pils/plugins"
+#define	PILS_BASE_PLUGINDIR	"/usr/local/lib/pils/plugins"
 
 /* The type of a Plugin Initialization Function */
 typedef PIL_rc (*PILPluginInitFun) (PILPlugin*us
