@@ -989,10 +989,10 @@ G_main_del_SignalHandler(GSIGSource* sig_src)
 	
 	CL_SIGNAL(sig_src->signal, NULL);
 
-	sig_src->gsourceid = 0;
 	sig_src->signal_triggered = FALSE;
 	g_source_remove(sig_src->gsourceid);
 	G_main_signal_list[sig_src->signal] = NULL;
+	sig_src->gsourceid = 0;
 	g_source_unref(source);
 	
 	return TRUE;
@@ -1436,9 +1436,9 @@ static gboolean
 Gmain_timeout_dispatch(GSource* src, GSourceFunc func, gpointer user_data);
 
 static GSourceFuncs Gmain_timeout_funcs = {
-	prepare: Gmain_timeout_prepare,
-	check: Gmain_timeout_check,
-	dispatch: Gmain_timeout_dispatch,
+	Gmain_timeout_prepare,
+	Gmain_timeout_check,
+	Gmain_timeout_dispatch,
 };
 
 
