@@ -103,6 +103,18 @@ IPC_Channel* get_log_chan(void){
 }
 /*************************/
 
+/**************************
+ * check if the fd is in use for logging
+ **************************/
+int
+cl_log_is_logd_fd(int fd)
+{
+	return logging_daemon_chan && (
+		fd == logging_daemon_chan->ops->get_send_select_fd(logging_daemon_chan)
+		||
+		fd == logging_daemon_chan->ops->get_recv_select_fd(logging_daemon_chan)
+		);
+}
 
 void
 cl_log_enable_stderr(int truefalse)
@@ -566,7 +578,6 @@ cl_log(int priority, const char * fmt, ...)
 	va_list		ap;
 	char		buf[MAXLINE];
 	ssize_t		nbytes;
-	int	needprivs = !cl_have_full_privs();
 
 	cl_process_pid = (int)getpid();
 
@@ -581,10 +592,6 @@ cl_log(int priority, const char * fmt, ...)
 		nbytes =  sizeof(buf) -1 ;
 	}
 
-	if (needprivs) {
-		return_to_orig_privs();
-	}
-
 	if (stderr_enabled) {
 		print_logline(stderr, cl_log_entity,cl_process_pid,
 			NULLTIME, prio2str(priority), buf);
