@@ -1131,12 +1131,19 @@ cib_config_changed(crm_data_t *old_cib, crm_data_t *new_cib, crm_data_t **result
 		if(xml_has_child(dest, "status")) {
 			cl_msg_remove(dest, "status");
 		}
+
+		xml_prop_iter(dest, name, value,
+			      if(safe_str_neq(name, XML_ATTR_NUMUPDATES)) {
+				  config_changes = TRUE;
+				  break;
+			      }
+		    );
+		
 		if(xml_has_children(dest)) {
 			config_changes = TRUE;
 		}
 	}
 
-	/* TODO: Check cib attributes */
 	
 	if(result) {
 		*result = diff;
