@@ -530,6 +530,11 @@ cib_native_perform_op(
 		rc = cib_return_code;
 	}	
 
+	if(rc == cib_diff_resync) {
+	    /* This is an internal value that clients do not and should not care about */
+	    rc = cib_ok;
+	}
+	
 	if(rc == cib_ok || rc == cib_not_master || rc == cib_master_timeout) {
 		crm_log_message(LOG_MSG, op_reply);
 
@@ -706,6 +711,11 @@ cib_native_callback(cib_t *cib, struct ha_msg *msg)
 	}
 
 	ha_msg_value_int(msg, F_CIB_RC, &rc);
+	if(rc == cib_diff_resync) {
+	    /* This is an internal value that clients do not and should not care about */
+	    rc = cib_ok;
+	}
+
 	output = get_message_xml(msg, F_CIB_CALLDATA);
 	
 	if(local_blob.callback != NULL
