@@ -63,6 +63,7 @@ gboolean crm_assert_failed = FALSE;
 unsigned int crm_log_level = LOG_INFO;
 gboolean crm_config_error = FALSE;
 gboolean crm_config_warning = FALSE;
+const char *crm_system_name = "unknown";
 
 void crm_set_env_options(void);
 
@@ -182,8 +183,8 @@ cluster_option(GHashTable* options, gboolean(*validate)(const char*),
 	}
 
 	if(value == NULL) {
-		crm_debug("Using default value '%s' for cluster option '%s'",
-			  def_value, name);
+		crm_debug_2("Using default value '%s' for cluster option '%s'",
+			    def_value, name);
 
 		if(options == NULL) {
 			return def_value;
@@ -440,6 +441,7 @@ crm_log_init(
 	/* and for good measure... - this enum is a bit field (!) */
 	g_log_set_always_fatal((GLogLevelFlags)0); /*value out of range*/
 	
+	crm_system_name = entity;
 	cl_log_set_entity(entity);
 	cl_log_set_facility(HA_LOG_FACILITY);
 
@@ -515,6 +517,11 @@ crm_version_helper(const char *text, char **end_text)
 }
 
 
+/*
+ * version1 < version2 : -1
+ * version1 = version2 :  0
+ * version1 > version2 :  1
+ */
 int
 compare_version(const char *version1, const char *version2)
 {
@@ -1158,47 +1165,35 @@ generate_transition_magic(const char *transition_key, int op_status, int op_rc)
 gboolean
 decode_transition_magic(
 	const char *magic, char **uuid, int *transition_id, int *action_id,
-	int *op_status, int *op_rc)
+	int *op_status, int *op_rc, int *target_rc)
 {
-	char *rc = NULL;
-	char *key = NULL;
-	char *magic2 = NULL;
-	char *status = NULL;
-
-	gboolean result = TRUE;
-	
-	if(decodeNVpair(magic, ':', &status, &magic2) == FALSE) {
-		crm_err("Couldn't find ':' in: %s", magic);
-		result = FALSE;
-		goto bail;
-	}
-
-	if(decodeNVpair(magic2, ';', &rc, &key) == FALSE) {
-		crm_err("Couldn't find ';' in: %s", magic2);
-		result = FALSE;
-		goto bail;
-	}
-
-	
-	CRM_CHECK(decode_transition_key(key, uuid, transition_id, action_id),
-		  result = FALSE;
-		  goto bail;
-		);
-	
-	*op_rc = crm_parse_int(rc, NULL);
-	*op_status = crm_parse_int(status, NULL);
+    int res = 0;
+    char *key = NULL;
+    gboolean result = TRUE;
 
+    CRM_CHECK(magic != NULL, return FALSE);
+    CRM_CHECK(op_rc != NULL, return FALSE);
+    CRM_CHECK(op_status != NULL, return FALSE);
+    
+    crm_malloc0(key, strlen(magic));
+    res = sscanf(magic, "%d:%d;%s", op_status, op_rc, key);
+    if(res != 3) {
+	crm_crit("Only found %d items in: %s", res, magic);
+	return FALSE;
+    }
+    
+    CRM_CHECK(decode_transition_key(key, uuid, transition_id, action_id, target_rc),
+	      result = FALSE;
+	      goto bail;
+	);
+    
   bail:
-	crm_free(rc);
-	crm_free(key);
-	crm_free(magic2);
-	crm_free(status);
-	
-	return result;
+    crm_free(key);
+    return result;
 }
 
 char *
-generate_transition_key(int transition_id, int action_id, const char *node)
+generate_transition_key(int transition_id, int action_id, int target_rc, const char *node)
 {
 	int len = 40;
 	char *fail_state = NULL;
@@ -1209,8 +1204,8 @@ generate_transition_key(int transition_id, int action_id, const char *node)
 	
 	crm_malloc0(fail_state, len);
 	if(fail_state != NULL) {
-		snprintf(fail_state, len, "%d:%d:%s",
-			 action_id, transition_id, node);
+		snprintf(fail_state, len, "%d:%d:%d:%s",
+			 action_id, transition_id, target_rc, node);
 	}
 	return fail_state;
 }
@@ -1218,38 +1213,69 @@ generate_transition_key(int transition_id, int action_id, const char *node)
 
 gboolean
 decode_transition_key(
-	const char *key, char **uuid, int *transition_id, int *action_id)
+	const char *key, char **uuid, int *transition_id, int *action_id, int *target_rc)
 {
-	char *tmp = NULL;
-	char *action = NULL;
-	char *transition = NULL;
+	int res = 0;
+	gboolean done = FALSE;
 
-	*uuid = NULL;
-	*action_id = -1;
-	*transition_id = -1;
+	CRM_CHECK(uuid != NULL, return FALSE);
+	CRM_CHECK(target_rc != NULL, return FALSE);
+	CRM_CHECK(action_id != NULL, return FALSE);
+	CRM_CHECK(transition_id != NULL, return FALSE);
 	
-	if(decodeNVpair(key, ':', &action, &tmp) == FALSE) {
-		crm_err("Couldn't find ':' in: %s", key);
-		return FALSE;
-	}
+	crm_malloc0(*uuid, strlen(key));
+	res = sscanf(key, "%d:%d:%d:%s", action_id, transition_id, target_rc, *uuid);
+	switch(res) {
+	    case 4:
+		/* Post Pacemaker 0.6 */
+		done = TRUE;
+		break;
+	    case 3:
+	    case 2:
+		/* this can be tricky - the UUID might start with an integer */
 
-	*action_id = crm_parse_int(action, NULL);
-	crm_free(action);
+		/* Until Pacemaker 0.6 */
+		done = TRUE;
+		*target_rc = -1;
+		res = sscanf(key, "%d:%d:%s", action_id, transition_id, *uuid);
+		if(res == 2) {
+		    *action_id = -1;
+		    res = sscanf(key, "%d:%s", transition_id, *uuid);
+		    CRM_CHECK(res == 2, done = FALSE);
 
-	if(decodeNVpair(tmp, ':', &transition, uuid) == FALSE) {
-		/* this would be an error but some versions dont
-		 * have the action
-		 */
-		*transition_id = *action_id;
+		} else if(res != 3) {
+		    CRM_CHECK(res == 3, done = FALSE);
+		}
+		break;
+		
+	    case 1:
+		/* Prior to Heartbeat 2.0.8 */
+		done = TRUE;
 		*action_id = -1;
-		*uuid = tmp;
-
-	} else {
-		*transition_id = crm_parse_int(transition, NULL);
-		crm_free(transition);
-		crm_free(tmp);
+		*target_rc = -1;
+		res = sscanf(key, "%d:%s", transition_id, *uuid);
+		CRM_CHECK(res == 2, done = FALSE);
+		break;
+	    default:
+		crm_crit("Unhandled sscanf result (%d) for %s", res, key);
+		
 	}
-	return TRUE;
+
+	if(strlen(*uuid) != 36) {
+	    crm_warn("Bad UUID (%s) in sscanf result (%d) for %s", *uuid, res, key);		    
+	}
+	
+	if(done == FALSE) {
+	    crm_err("Cannot decode '%s' rc=%d", key, res);
+	    
+	    crm_free(*uuid);
+	    *uuid = NULL;
+	    *target_rc = -1;
+	    *action_id = -1;
+	    *transition_id = -1;
+	}
+	
+	return done;
 }
 
 void
@@ -1321,7 +1347,7 @@ filter_action_parameters(crm_data_t *param_set, const char *version)
 	}
 
 #if CRM_DEPRECATED_SINCE_2_0_5
- 	if(version == NULL || compare_version("1.0.5", version)) {
+ 	if(version == NULL || compare_version("1.0.5", version) > 0) {
 		for(lpc = 0; lpc < DIMOF(filter_205); lpc++) {
 			xml_remove_prop(param_set, filter_205[lpc]); 
 		}
@@ -1349,7 +1375,7 @@ filter_action_parameters(crm_data_t *param_set, const char *version)
 		      }
 		);
 
-	if(crm_get_msec(interval) && compare_version(version, "1.0.8")) {
+	if(crm_get_msec(interval) && compare_version(version, "1.0.8") > 0) {
 		/* Re-instate the operation's timeout value */
 		if(timeout != NULL) {
 			crm_xml_add(param_set, CRM_META"_timeout", timeout);
@@ -1411,7 +1437,7 @@ crm_abort(const char *file, const char *function, int line,
 	    pid=fork();
 
 	} else {
-	    do_crm_log(LOG_ERR, "%s: Triggered non-fatal assert at %s:%d : %s",
+	    do_crm_log(LOG_ERR, "%s: Triggered fatal assert at %s:%d : %s",
 		       function, file, line, assert_condition);
 	}
 	
