@@ -29,14 +29,24 @@
 void clone_create_notifications(
 	resource_t *rsc, action_t *action, action_t *action_complete,
 	pe_working_set_t *data_set);
+gboolean create_child_clone(resource_t *rsc, int sub_id, pe_working_set_t *data_set);
 
+static void mark_as_orphan(resource_t *rsc) 
+{
+    set_bit(rsc->flags, pe_rsc_orphan);
+    slist_iter(
+	child, resource_t, rsc->children, lpc,
+	mark_as_orphan(child);
+	);
+}
 
-static gboolean
+gboolean
 create_child_clone(resource_t *rsc, int sub_id, pe_working_set_t *data_set) 
 {
 	gboolean rc = TRUE;
 	char *inc_num = NULL;
 	char *inc_max = NULL;
+	gboolean as_orphan = FALSE;
 	resource_t *child_rsc = NULL;
 	crm_data_t * child_copy = NULL;
 	clone_variant_data_t *clone_data = NULL;
@@ -44,6 +54,10 @@ create_child_clone(resource_t *rsc, int sub_id, pe_working_set_t *data_set)
 
 	CRM_CHECK(clone_data->xml_obj_child != NULL, return FALSE);
 
+	if(sub_id < 0) {
+	    as_orphan = TRUE;
+	    sub_id = clone_data->total_clones;
+	}
 	inc_num = crm_itoa(sub_id);
 	inc_max = crm_itoa(clone_data->clone_max);	
 
@@ -58,9 +72,13 @@ create_child_clone(resource_t *rsc, int sub_id, pe_working_set_t *data_set)
 		goto bail;
 	}
 /* 	child_rsc->globally_unique = rsc->globally_unique; */
-	
-	crm_debug_3("Setting clone attributes for: %s", child_rsc->id);
+
+	clone_data->total_clones += 1;
+	crm_debug_2("Setting clone attributes for: %s", child_rsc->id);
 	rsc->children = g_list_append(rsc->children, child_rsc);
+	if(as_orphan) {
+	    mark_as_orphan(child_rsc);
+	}
 	
 	add_hash_param(child_rsc->meta, XML_RSC_ATTR_INCARNATION_MAX, inc_max);
 	
@@ -133,6 +151,11 @@ gboolean clone_unpack(resource_t *rsc, pe_working_set_t *data_set)
 	if(crm_is_true(ordered)) {
 		clone_data->ordered = TRUE;
 	}
+	if((rsc->flags & pe_rsc_unique) == 0 && clone_data->clone_node_max > 1) {
+	    crm_config_err("Anonymous clones (%s) may only support one copy"
+			   " per node", rsc->id);
+	    clone_data->clone_node_max = 1;
+	}
 
 	crm_debug_2("Options for %s", rsc->id);
 	crm_debug_2("\tClone max: %d", clone_data->clone_max);
@@ -248,6 +271,11 @@ void clone_print(
 	
 	slist_iter(
 		child_rsc, resource_t, rsc->children, lpc,
+
+		if(is_set(child_rsc->flags, pe_rsc_orphan)
+		   && child_rsc->fns->active(child_rsc, TRUE) == FALSE) {
+		    continue;
+		}
 		
 		if(options & pe_print_html) {
 			status_print("<li>\n");
