@@ -222,7 +222,9 @@ common_unpack(crm_data_t * xml_obj, resource_t **rsc,
 	    cl_str_to_boolean(value, &bool_value);
 	    if(bool_value == FALSE) {
 		clear_bit((*rsc)->flags, pe_rsc_managed); 
-	    } 
+	    } else {
+		set_bit((*rsc)->flags, pe_rsc_managed); 
+	    }
 	}
 
 	crm_debug_2("Options for %s", (*rsc)->id);
@@ -315,6 +317,15 @@ void common_update_score(resource_t *rsc, const char *id, int score)
     }
 }
 
+resource_t *uber_parent(resource_t *rsc) 
+{
+	resource_t *parent = rsc;
+	while(parent != NULL && parent->parent != NULL) {
+		parent = parent->parent;
+	}
+	return parent;
+}
+
 void
 common_apply_stickiness(resource_t *rsc, node_t *node, pe_working_set_t *data_set) 
 {
@@ -356,16 +367,25 @@ common_apply_stickiness(resource_t *rsc, node_t *node, pe_working_set_t *data_se
 	value = g_hash_table_lookup(node->details->attrs, fail_attr);
 	if(value != NULL) {
 		crm_debug("%s: %s", fail_attr, value);
-		fail_count = crm_parse_int(value, "0");
+		fail_count = char2score(value);
 	}
 	crm_free(fail_attr);
 	
 	if(fail_count > 0 && rsc->fail_stickiness != 0) {
-		resource_location(rsc, node, fail_count * rsc->fail_stickiness,
-				  "fail_stickiness", data_set);
+		resource_t *failed = rsc;
+		int score = fail_count * rsc->fail_stickiness;
+		if(is_not_set(rsc->flags, pe_rsc_unique)) {
+		    failed = uber_parent(rsc);
+		}
+
+		/* detect and prevent score underflows */
+		if(rsc->fail_stickiness < 0 && (score > 0 || score < -INFINITY)) {
+		    score = -INFINITY;
+		}
+
+		resource_location(failed, node, score, "fail_stickiness", data_set);
 		crm_info("Setting failure stickiness for %s on %s: %d",
-			  rsc->id, node->details->uname,
-			  fail_count * rsc->fail_stickiness);
+			  failed->id, node->details->uname, score);
 	}
 	g_hash_table_destroy(meta_hash);
 }
@@ -387,7 +407,7 @@ void common_free(resource_t *rsc)
 	if(rsc->meta != NULL) {
 		g_hash_table_destroy(rsc->meta);
 	}
-	if(is_set(rsc->flags, pe_rsc_orphan)) {
+	if(rsc->parent == NULL && is_set(rsc->flags, pe_rsc_orphan)) {
 		free_xml(rsc->xml);
 	}
 	if(rsc->running_on) {
