@@ -617,11 +617,14 @@ create_fake_resource(const char *rsc_id, crm_data_t *rsc_entry, pe_working_set_t
 	return rsc;
 }
 
+extern gboolean create_child_clone(resource_t *rsc, int sub_id, pe_working_set_t *data_set);
+
 static resource_t *
 unpack_find_resource(
 	pe_working_set_t *data_set, node_t *node, const char *rsc_id, crm_data_t *rsc_entry)
 {
 	resource_t *rsc = NULL;
+	resource_t *clone_parent = NULL;
 	gboolean is_duped_clone = FALSE;
 	char *alt_rsc_id = crm_strdup(rsc_id);
 	
@@ -631,14 +634,23 @@ unpack_find_resource(
 		/* no match */
 		if(rsc == NULL) {
 			crm_debug_2("%s not found: %d", alt_rsc_id, is_duped_clone);
-			if(is_duped_clone) {
-				/* create one */
-				rsc = create_fake_resource(alt_rsc_id, rsc_entry, data_set);
-				crm_info("Making sure orphan %s/%s is stopped on %s",
-					 rsc_id, rsc->id, node->details->uname);
-				resource_location(rsc, NULL, -INFINITY, "__orphan_clone_dont_run__", data_set);
+			if(is_duped_clone == FALSE) {
+			    break;
 			}
+			    
+			/* create one */
+#if 1
+			create_child_clone(clone_parent, -1, data_set);
+			crm_debug("Looking again for %s", alt_rsc_id);
+			rsc = pe_find_resource(data_set->resources, alt_rsc_id);
+			CRM_CHECK(rsc != NULL, crm_err("%s stil not found", alt_rsc_id); continue);
+#else
+			rsc = create_fake_resource(alt_rsc_id, rsc_entry, data_set);
+			crm_info("Making sure orphan %s/%s of %s is stopped on %s",
+				 rsc_id, rsc->id, clone_parent->id, node->details->uname);
+			resource_location(rsc, NULL, -INFINITY, "__orphan_clone_dont_run__", data_set);
 			break;
+#endif
 			
 			/* not running anywhere else */
 		} else if(rsc->running_on == NULL) {
@@ -655,6 +667,7 @@ unpack_find_resource(
 			 */
 		} else {
 			crm_debug_3("find another one");
+			clone_parent = uber_parent(rsc);
 			rsc = NULL;
 			is_duped_clone = TRUE;
 			alt_rsc_id = increment_clone(alt_rsc_id);
@@ -734,11 +747,23 @@ process_rsc_state(resource_t *rsc, node_t *node,
 		native_add_running(rsc, node, data_set);
 
 		if(is_set(rsc->flags, pe_rsc_managed) && rsc->stickiness != 0) {
-			resource_location(rsc, node, rsc->stickiness,
+		    node_t *match = pe_find_node_id(rsc->allowed_nodes, node->details->id);
+
+		    if(match != NULL || data_set->symmetric_cluster) {
+			resource_t *sticky_rsc = rsc;
+			if(rsc->parent && rsc->parent->variant == pe_group) {
+			    sticky_rsc = rsc->parent;
+			}
+			
+			resource_location(sticky_rsc, node, rsc->stickiness,
 					  "stickiness", data_set);
 			crm_debug_2("Resource %s: preferring current location"
-				    " (node=%s, weight=%d)", rsc->id,
+				    " (node=%s, weight=%d)", sticky_rsc->id,
 				    node->details->uname, rsc->stickiness);
+		    } else {
+			crm_debug("Ignoring stickiness for %s: the cluster is asymmetric and node %s is no longer explicitly allowed",
+				  rsc->id, node->details->uname);
+		    }
 		}
 	
 		if(on_fail == action_fail_ignore) {
@@ -994,8 +1019,9 @@ gboolean
 unpack_rsc_op(resource_t *rsc, node_t *node, crm_data_t *xml_op,
 	      int *max_call_id, enum action_fail_response *on_fail,
 	      pe_working_set_t *data_set) 
-{
+{    
 	const char *id          = NULL;
+	const char *key        = NULL;
 	const char *task        = NULL;
  	const char *task_id     = NULL;
  	const char *actual_rc   = NULL;
@@ -1009,10 +1035,13 @@ unpack_rsc_op(resource_t *rsc, node_t *node, crm_data_t *xml_op,
 	int task_id_i = -1;
 	int task_status_i = -2;
 	int actual_rc_i = 0;
+	int target_rc = -1;
 	
 	action_t *action = NULL;
+	node_t *effective_node = NULL;
+	resource_t *failed = NULL;
+
 	gboolean is_probe = FALSE;
-	gboolean is_stop_action = FALSE;
 	
 	CRM_CHECK(rsc    != NULL, return FALSE);
 	CRM_CHECK(node   != NULL, return FALSE);
@@ -1024,6 +1053,7 @@ unpack_rsc_op(resource_t *rsc, node_t *node, crm_data_t *xml_op,
 	task_status = crm_element_value(xml_op, XML_LRM_ATTR_OPSTATUS);
 	op_digest   = crm_element_value(xml_op, XML_LRM_ATTR_OP_DIGEST);
 	op_version  = crm_element_value(xml_op, XML_ATTR_CRM_VERSION);
+	key	    = crm_element_value(xml_op, XML_ATTR_TRANSITION_KEY);
 
 	CRM_CHECK(id != NULL, return FALSE);
 	CRM_CHECK(task != NULL, return FALSE);
@@ -1050,10 +1080,6 @@ unpack_rsc_op(resource_t *rsc, node_t *node, crm_data_t *xml_op,
 		is_probe = TRUE;
 	}
 	
-	if(safe_str_eq(task, CRMD_ACTION_STOP)) {
-		is_stop_action = TRUE;
-	}
-	
 	if(task_status_i != LRM_OP_PENDING) {
 		task_id_i = crm_parse_int(task_id, "-1");
 
@@ -1076,61 +1102,140 @@ unpack_rsc_op(resource_t *rsc, node_t *node, crm_data_t *xml_op,
 	CRM_CHECK(actual_rc != NULL, return FALSE);	
 	actual_rc_i = crm_parse_int(actual_rc, NULL);
 
-	if(EXECRA_NOT_INSTALLED == actual_rc_i) {
-		resource_location(rsc, node, -INFINITY, "not-installed", data_set);
-		if(is_probe) {
-			/* treat these like stops */
-			is_stop_action = TRUE;
-			task_status_i = LRM_OP_DONE;
-			actual_rc_i = EXECRA_NOT_RUNNING;
-			
- 		} else {
-			task_status_i = LRM_OP_ERROR;
-		}
+	if(key) {
+	    int dummy = 0;
+	    char *dummy_string = NULL;
+	    decode_transition_key(key, &dummy_string, &dummy, &dummy, &target_rc);
+	    crm_free(dummy_string);
+	}
+	
+	if(task_status_i == LRM_OP_DONE && target_rc >= 0) {
+	    if(target_rc == actual_rc_i) {
+		task_status_i = LRM_OP_DONE;
 		
-	} else if(EXECRA_NOT_RUNNING == actual_rc_i) {
-		if(is_probe) {
-			/* treat these like stops */
-			is_stop_action = TRUE;
-		}
-		if(is_stop_action) {
-			task_status_i = LRM_OP_DONE;
- 		} else {
-			task_status_i = LRM_OP_ERROR;
-		}
-		
-	} else if(EXECRA_RUNNING_MASTER == actual_rc_i) {
-		if(is_probe
-		   || (rsc->role == RSC_ROLE_MASTER
-		       && safe_str_eq(task, CRMD_ACTION_STATUS))) {
-			task_status_i = LRM_OP_DONE;
+	    } else {
+		task_status_i = LRM_OP_ERROR;
+		crm_info("Remapping %s (rc=%d) on %s to an ERROR (expected %d)",
+			 id, actual_rc_i, node->details->uname, target_rc);
+	    }
 
+	} else if(task_status_i == LRM_OP_ERROR) {
+	    /* let us decide that */
+	    crm_debug("Remapping %s (rc=%d, status=%d) on %s to DONE",
+		      id, actual_rc_i, task_status_i, node->details->uname);
+ 	    task_status_i = LRM_OP_DONE;
+	}
+	
+	if(task_status_i == LRM_OP_NOTSUPPORTED) {
+	    actual_rc_i = EXECRA_UNIMPLEMENT_FEATURE;
+	}
+
+	/* we could clean this up significantly except for old LRMs and CRMs that
+	 * didnt include target_rc and liked to remap status
+	 */
+	switch(actual_rc_i) {
+	    case EXECRA_NOT_RUNNING:
+		if(is_probe || target_rc == actual_rc_i) {
+		    task_status_i = LRM_OP_DONE;
+		    rsc->role = RSC_ROLE_STOPPED;
+		    
+		    /* clear any previous failure actions */
+		    *on_fail = action_fail_ignore;
+		    rsc->next_role = RSC_ROLE_UNKNOWN;
+		    
+		} else if(safe_str_neq(task, CRMD_ACTION_STOP)) {
+		    task_status_i = LRM_OP_ERROR;
+		}
+		break;
+		
+	    case EXECRA_RUNNING_MASTER:
+		if(is_probe) {
+		    task_status_i = LRM_OP_DONE;
+		    crm_warn("%s found active %s in master mode on %s",
+			     id, rsc->id, node->details->uname);
+
+		} else if(target_rc == actual_rc_i) {
+		    /* nothing to do */
+
+		} else if(target_rc >= 0) {
+		    task_status_i = LRM_OP_ERROR;
+
+		    /* legacy code for pre-0.6.5 operations */
+		} else if(safe_str_neq(task, CRMD_ACTION_STATUS)
+			  || rsc->role != RSC_ROLE_MASTER) {
+		    task_status_i = LRM_OP_ERROR;
+		    if(rsc->role != RSC_ROLE_MASTER) {
+			crm_err("%s reported %s in master mode on %s",
+				id, rsc->id,
+				node->details->uname);
+		    }
+		}
+		rsc->role = RSC_ROLE_MASTER;
+		break;
+		
+	    case EXECRA_FAILED_MASTER:
+		rsc->role = RSC_ROLE_MASTER;
+		task_status_i = LRM_OP_ERROR;
+		break;
+
+	    case EXECRA_UNIMPLEMENT_FEATURE:
+		if(interval > 0) {
+		    task_status_i = LRM_OP_NOTSUPPORTED;
+		    break;
+		}
+		/* else: fall through */
+	    case EXECRA_INSUFFICIENT_PRIV:
+	    case EXECRA_NOT_INSTALLED:
+	    case EXECRA_INVALID_PARAM:
+		effective_node = node;
+		/* fall through */
+	    case EXECRA_NOT_CONFIGURED:
+		failed = rsc;
+		if(is_not_set(rsc->flags, pe_rsc_unique)) {
+		    failed = uber_parent(failed);
+		}
+		
+		crm_err("Hard error: %s failed with rc=%d.", id, actual_rc_i);
+		if(effective_node) {
+		    crm_err("  Preventing %s from re-starting on %s",
+			    failed->id, effective_node->details->uname);
 		} else {
-			task_status_i = LRM_OP_ERROR;
-			if(rsc->role != RSC_ROLE_MASTER) {
-				crm_err("%s reported %s in master mode on %s",
-					id, rsc->id,
-					node->details->uname);
-			}
+		    crm_err("  Preventing %s from re-starting anywhere in the cluster",
+			    failed->id);
 		}
-		rsc->role = RSC_ROLE_MASTER;
 
-	} else if(EXECRA_FAILED_MASTER == actual_rc_i) {
-		rsc->role = RSC_ROLE_MASTER;
-		task_status_i = LRM_OP_ERROR;
+		resource_location(failed, effective_node, -INFINITY, "hard-error", data_set);
+		if(is_probe) {
+			/* treat these like stops */
+			task = CRMD_ACTION_STOP;
+			task_status_i = LRM_OP_DONE;
+		}
+		break;
 
-	} else if(EXECRA_OK == actual_rc_i
-		  && interval > 0
-		  && rsc->role == RSC_ROLE_MASTER) {
-		/* catch status ops that return 0 instead of 8 while they
-		 *   are supposed to be in master mode
-		 */
-		task_status_i = LRM_OP_ERROR;
+	    case EXECRA_OK:
+		if(is_probe && target_rc == 7) {
+		    task_status_i = LRM_OP_DONE;
+		    crm_warn("%s found active %s on %s",
+			     id, rsc->id, node->details->uname);
 
-	} else if(task_status_i == LRM_OP_DONE && EXECRA_OK != actual_rc_i) {
-		crm_err("Remapping %s (rc=%d) on %s to an ERROR",
-			id, actual_rc_i, node->details->uname);
-		task_status_i = LRM_OP_ERROR;
+		    /* legacy code for pre-0.6.5 operations */
+		} else if(target_rc < 0
+		   && interval > 0
+		   && rsc->role == RSC_ROLE_MASTER) {
+		    /* catch status ops that return 0 instead of 8 while they
+		     *   are supposed to be in master mode
+		     */
+		    task_status_i = LRM_OP_ERROR;
+		}
+		
+		break;
+		
+	    default:
+		if(task_status_i == LRM_OP_DONE) {
+		    crm_info("Remapping %s (rc=%d) on %s to an ERROR",
+			     id, actual_rc_i, node->details->uname);
+		    task_status_i = LRM_OP_ERROR;
+		}
 	}
 
 	if(task_status_i == LRM_OP_ERROR
@@ -1158,13 +1263,16 @@ unpack_rsc_op(resource_t *rsc, node_t *node, crm_data_t *xml_op,
 			crm_debug_3("%s/%s completed on %s",
 				    rsc->id, task, node->details->uname);
 
-			if(is_stop_action) {
+			if(actual_rc_i == EXECRA_NOT_RUNNING) {
+			    /* nothing to do */
+			    
+			} else if(safe_str_eq(task, CRMD_ACTION_STOP)) {
 				rsc->role = RSC_ROLE_STOPPED;
-
+			    
 				/* clear any previous failure actions */
 				*on_fail = action_fail_ignore;
 				rsc->next_role = RSC_ROLE_UNKNOWN;
-				
+
 			} else if(safe_str_eq(task, CRMD_ACTION_PROMOTE)) {
 				rsc->role = RSC_ROLE_MASTER;
 
@@ -1191,28 +1299,35 @@ unpack_rsc_op(resource_t *rsc, node_t *node, crm_data_t *xml_op,
 				*on_fail = action->on_fail;
 			}
 
-			if(task_status_i == LRM_OP_NOTSUPPORTED) {
+			if(safe_str_eq(task, CRMD_ACTION_STOP)) {
 			    resource_location(
-				rsc, node, -INFINITY, "__not_supported__", data_set);
-
-			} else if(data_set->start_failure_fatal
-				  || compare_version("2.0", op_version)) {			    
-			    if(is_stop_action || safe_str_eq(task, CRMD_ACTION_START)) {
-				crm_warn("Compatability handling for failed op %s on %s",
-					 id, node->details->uname);
-				resource_location(
-				    rsc, node, -INFINITY, "__legacy_stopstart__", data_set);
-			    }
-			}
-
-			if(safe_str_eq(task, CRMD_ACTION_PROMOTE)) {
-				rsc->role = RSC_ROLE_MASTER;
+				rsc, node, -INFINITY, "__stop_fail__", data_set);
+			    
+			} else if(safe_str_eq(task, CRMD_ACTION_PROMOTE)) {
+			    rsc->role = RSC_ROLE_MASTER;
 
 			} else if(safe_str_eq(task, CRMD_ACTION_DEMOTE)) {
-				rsc->role = RSC_ROLE_MASTER;
+			    /*
+			     * staying in role=master ends up putting the PE/TE into a loop
+			     * setting role=slave is not dangerous because no master will be
+			     * promoted until the failed resource has been fully stopped
+			     */
+			    crm_warn("Forcing %s to stop after a failed demote action", rsc->id);
+			    rsc->next_role = RSC_ROLE_STOPPED;
+			    rsc->role = RSC_ROLE_SLAVE;
 				
-			} else if(rsc->role < RSC_ROLE_STARTED) {
-				rsc->role = RSC_ROLE_STARTED;
+			} else if((data_set->start_failure_fatal
+				   || compare_version("2.0", op_version) > 0)
+				  && safe_str_eq(task, CRMD_ACTION_START)) {
+			    crm_warn("Compatability handling for failed op %s on %s",
+				     id, node->details->uname);
+			    resource_location(
+				rsc, node, -INFINITY, "__legacy_start__", data_set);
+
+			}
+
+			if(rsc->role < RSC_ROLE_STARTED) {
+			    rsc->role = RSC_ROLE_STARTED;
 			}
 
 			crm_debug_2("Resource %s: role=%s, unclean=%s, on_fail=%s, fail_role=%s",
