@@ -135,8 +135,43 @@ node_list_eq(GListPtr list1, GListPtr list2, gboolean filter)
 	return TRUE;
 }
 
-/* the intersection of list1 and list2 
- */
+/* any node in list1 or list2 and not in the other gets a score of -INFINITY */
+GListPtr
+node_list_exclude(GListPtr list1, GListPtr list2)
+{
+    node_t *other_node = NULL;
+    GListPtr result = NULL;
+    
+    result = node_list_dup(list1, FALSE, FALSE);
+    
+    slist_iter(
+	node, node_t, result, lpc,
+	
+	other_node = pe_find_node_id(list2, node->details->id);
+	
+	if(other_node == NULL) {
+	    node->weight = -INFINITY;
+	} else {
+	    node->weight = merge_weights(node->weight, other_node->weight);
+	}
+	);
+    
+    slist_iter(
+	node, node_t, list2, lpc,
+	
+	other_node = pe_find_node_id(result, node->details->id);
+	
+	if(other_node == NULL) {
+	    node_t *new_node = node_copy(node);
+	    new_node->weight = -INFINITY;
+	    result = g_list_append(result, new_node);
+	}
+	);
+
+    return result;
+}
+
+/* the intersection of list1 and list2 */
 GListPtr
 node_list_and(GListPtr list1, GListPtr list2, gboolean filter)
 {
@@ -315,14 +350,27 @@ node_list_dup(GListPtr list1, gboolean reset, gboolean filter)
 
 void dump_node_scores(int level, resource_t *rsc, const char *comment, GListPtr nodes) 
 {
+    GListPtr list = nodes;
+    if(rsc) {
+	list = rsc->allowed_nodes;
+    }
+    
     slist_iter(
-	node, node_t, nodes, lpc,
+	node, node_t, list, lpc,
 	if(rsc) {
-	    do_crm_log(level, "%s: %s.%s = %d", comment, rsc->id, node->details->uname, node->weight);
+	    do_crm_log(level, "%s: %s allocation score on %s: %d",
+		       comment, rsc->id, node->details->uname, node->weight);
 	} else {
 	    do_crm_log(level, "%s: %s = %d", comment, node->details->uname, node->weight);
 	}
 	);
+
+    if(rsc && rsc->children) {
+	slist_iter(
+	    child, resource_t, rsc->children, lpc,
+	    dump_node_scores(level, child, comment, nodes);
+	    );
+    }
 }
 
 gint sort_rsc_index(gconstpointer a, gconstpointer b)
@@ -398,7 +446,7 @@ custom_action(resource_t *rsc, char *key, const char *task,
 
 	if(action == NULL) {
 		if(save_action) {
-			crm_debug_2("Creating%s action %d: %s for %s on %s",
+			crm_debug_4("Creating%s action %d: %s for %s on %s",
 				    optional?"":" manditory", data_set->action_id, key, rsc?rsc->id:"<NULL>",
 				    on_node?on_node->details->uname:"<NULL>");
 		}
@@ -482,7 +530,8 @@ custom_action(resource_t *rsc, char *key, const char *task,
 		} else if(action->node == NULL) {
 			action->runnable = FALSE;
 			
-		} else if(is_not_set(rsc->flags, pe_rsc_managed)) {
+		} else if(g_hash_table_lookup(action->meta, XML_LRM_ATTR_INTERVAL) == NULL
+			  && is_not_set(rsc->flags, pe_rsc_managed)) {
 			do_crm_log(warn_level, "Action %s (unmanaged)",
 				 action->uuid);
 			action->optional = TRUE;
@@ -654,7 +703,8 @@ unpack_operation(
 		    value = "stop resource";
 		}
 		
-	} else if(safe_str_eq(value, "ignore")) {
+	} else if(safe_str_eq(value, "ignore")
+		|| safe_str_eq(value, "nothing")) {
 		action->on_fail = action_fail_ignore;
 		value = "ignore";
 
@@ -667,8 +717,7 @@ unpack_operation(
 		action->fail_role = RSC_ROLE_STOPPED;
 		value = "stop resource";
 		
-	} else if(safe_str_eq(value, "restart")
-		|| safe_str_eq(value, "nothing")) {
+	} else if(safe_str_eq(value, "restart")) {
 		action->on_fail = action_fail_recover;
 		value = "restart (and possibly migrate)";
 		
@@ -1033,6 +1082,14 @@ static void
 resource_node_score(resource_t *rsc, node_t *node, int score, const char *tag) 
 {
 	node_t *match = NULL;
+
+	if(rsc->children) {
+	    slist_iter(
+		child_rsc, resource_t, rsc->children, lpc,
+		resource_node_score(child_rsc, node, score, tag);
+		);
+	}
+	
 	crm_debug_2("Setting %s for %s on %s: %d",
 		    tag, rsc->id, node->details->uname, score);
 	match = pe_find_node_id(rsc->allowed_nodes, node->details->id);
@@ -1139,10 +1196,10 @@ sort_op_by_callid(gconstpointer a, gconstpointer b)
 	/* now process pending ops */
 	CRM_CHECK(a_key != NULL && b_key != NULL, sort_return(0));
 	CRM_CHECK(decode_transition_magic(
-			  a_key, &a_uuid, &a_id, &dummy, &a_status, &a_rc),
+		      a_key, &a_uuid, &a_id, &dummy, &a_status, &a_rc, &dummy),
 		  sort_return(0));
 	CRM_CHECK(decode_transition_magic(
-			  b_key, &b_uuid, &b_id, &dummy, &b_status, &b_rc),
+		      b_key, &b_uuid, &b_id, &dummy, &b_status, &b_rc, &dummy),
 		  sort_return(0));
 
 	/* try and determin the relative age of the operation...
