@@ -93,6 +93,8 @@ static int lrm_add_rsc (ll_lrm_t*, const char* id, const char* class
 			,const char* type, const char* provider
 			,GHashTable* parameter);
 static int lrm_delete_rsc (ll_lrm_t*, const char* id);
+static int lrm_fail_rsc (ll_lrm_t* lrm, const char* rsc_id, const int fail_rc
+			,const char* fail_reason);
 static IPC_Channel* lrm_ipcchan (ll_lrm_t*);
 static int lrm_msgready (ll_lrm_t*);
 static int lrm_rcvmsg (ll_lrm_t*, int blocking);
@@ -111,6 +113,7 @@ static struct lrm_ops lrm_ops_instance =
 	lrm_get_rsc,
 	lrm_add_rsc,
 	lrm_delete_rsc,
+	lrm_fail_rsc,
 	lrm_ipcchan,
 	lrm_msgready,
 	lrm_rcvmsg
@@ -688,6 +691,53 @@ lrm_get_rsc (ll_lrm_t* lrm, const char* rsc_id)
 	return rsc;
 }
 
+static int
+lrm_fail_rsc (ll_lrm_t* lrm, const char* rsc_id, const int fail_rc
+,		 const char* fail_reason)
+{
+	struct ha_msg* msg;
+
+	/* check whether the rsc_id is available */
+	if (NULL == rsc_id || RID_LEN <= strlen(rsc_id))	{
+		cl_log(LOG_ERR, "%s: wrong parameter rsc_id.", __FUNCTION__);
+		return HA_FAIL;
+	}
+
+	/* check whether the channel to lrmd is available */
+	if (NULL == ch_cmd)	{
+		cl_log(LOG_ERR, "%s: ch_mod is null.", __FUNCTION__);
+		return HA_FAIL;
+	}
+
+	/* create the message */
+	msg = create_lrm_rsc_msg(rsc_id,FAILRSC);
+	if (NULL == msg) {
+		LOG_FAIL_create_lrm_rsc_msg(FAILRSC);
+		return HA_FAIL;
+	}
+	if ((fail_reason && HA_OK != ha_msg_add(msg,F_LRM_FAIL_REASON,fail_reason))
+		|| HA_OK != ha_msg_add_int(msg, F_LRM_ASYNCMON_RC, fail_rc)
+	) {
+		ha_msg_del(msg);
+		LOG_BASIC_ERROR("ha_msg_add");
+		return HA_FAIL;
+	}
+	/* send to lrmd */
+	if (HA_OK != msg2ipcchan(msg,ch_cmd)) {
+		ha_msg_del(msg);
+		LOG_FAIL_SEND_MSG(FAILRSC, "ch_cmd");
+		return HA_FAIL;
+	}
+	ha_msg_del(msg);
+	/* check the result */
+	if (HA_OK != get_ret_from_ch(ch_cmd)) {
+		LOG_GOT_FAIL_RET(LOG_ERR, FAILRSC);
+		return HA_FAIL;
+	}
+
+	return HA_OK;
+}
+
 static int
 lrm_add_rsc (ll_lrm_t* lrm, const char* rsc_id, const char* class
 , 	     const char* type, const char* provider, GHashTable* parameter)
@@ -1151,6 +1201,7 @@ msg_to_op(struct ha_msg* msg)
 	const char* op_type;
 	const char* app_name;
 	const char* rsc_id;
+	const char* fail_reason;
 	const char* output;
 	const void* user_data;
 
@@ -1190,10 +1241,6 @@ msg_to_op(struct ha_msg* msg)
 		else {
 			op->output = NULL;
 		}
-		/* check the rc to revise the op_status */
-		if (0 != op->rc) {
-			op->op_status = LRM_OP_ERROR;
-		}
 	} else if(op->op_status == LRM_OP_PENDING) {
 		op->rc = EXECRA_STATUS_UNKNOWN;
 		
@@ -1228,9 +1275,14 @@ msg_to_op(struct ha_msg* msg)
 		free_op(op);
 		return NULL;
 	}
-
 	op->rsc_id = g_strdup(rsc_id);
 
+	/* op->fail_reason present only on async failures */
+	fail_reason = ha_msg_value(msg, F_LRM_FAIL_REASON);
+	if (fail_reason) {
+		op->fail_reason = g_strdup(fail_reason);
+	}
+
 	/* op->user_data */
 	user_data = cl_get_string(msg, F_LRM_USERDATA);
 	
@@ -1243,9 +1295,10 @@ msg_to_op(struct ha_msg* msg)
 	   || ha_msg_value_ul(msg, F_LRM_T_RCCHANGE, &op->t_rcchange) != HA_OK
 	   || ha_msg_value_ul(msg, F_LRM_EXEC_TIME, &op->exec_time) != HA_OK
 	   || ha_msg_value_ul(msg, F_LRM_QUEUE_TIME, &op->queue_time) != HA_OK) {
-		cl_log(LOG_WARNING
-		, "%s(%d): failed to get the timing information"
+		/* cl_log(LOG_WARNING
+		, "%s:%d: failed to get the timing information"
 		, __FUNCTION__, __LINE__);
+		*/
 	}
 	
 	/* op->params */
@@ -1435,6 +1488,10 @@ execra_code2string(uniform_ret_execra_t code)
 		case EXECRA_NOT_RUNNING:
 			return "not running";
 		/* For status command only */
+		case EXECRA_RUNNING_MASTER:
+			return "master";
+		case EXECRA_FAILED_MASTER:
+			return "master (failed)";
 		case EXECRA_RA_DEAMON_DEAD1:
 			return "status: deamon dead";
 		case EXECRA_RA_DEAMON_DEAD2:
