@@ -585,7 +585,7 @@ external_getinfo(StonithPlugin * s, int reqtype)
 	}
 
 	if (rc == 0) {
-		if (sd->outputbuf == NULL) {
+		if (sd->outputbuf != NULL) {
 			FREE(sd->outputbuf);
 		}
 		sd->outputbuf =  ret;
@@ -762,7 +762,7 @@ external_run_cmd(struct pluginDevice *sd, const char *op, char **output)
 		LOG(PIL_INFO, "%s: Calling '%s' returned %d", __FUNCTION__, cmd, rc);
 	}
 	if (Debug && data) {
-		LOG(PIL_INFO, "%s: '%s' output: %s", __FUNCTION__, cmd, data);
+		LOG(PIL_DEBUG, "%s: '%s' output: %s", __FUNCTION__, cmd, data);
 	}
 
 	if (output) {
