@@ -26,6 +26,7 @@
 #include <sys/types.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <time.h>
 #ifndef __USE_GNU
 #define __USE_GNU
 /* For strnlen protype */ 
@@ -40,12 +41,13 @@
 #endif /* HAVE_GETOPT_H */
 #include <clplumbing/cl_log.h>
 #include <lrm/lrm_api.h>
+#include <lrm/lrm_msg.h>
 #include <lrm/raexec.h>
 #include <clplumbing/lsb_exitcodes.h>
 #include <clplumbing/GSource.h>
 #include <clplumbing/Gmain_timeout.h>
 
-const char * optstring = "AD:dEF:d:sg:M:O:P:c:S:LI:CT:n:h";
+const char * optstring = "AD:X:dEF:d:sg:M:O:P:c:S:LI:CT:n:h";
 
 #ifdef HAVE_GETOPT_H
 static struct option long_options[] = {
@@ -53,11 +55,12 @@ static struct option long_options[] = {
 	{"executera", 1, 0, 'E'},
 	{"flush",1,0,'F'},
 	{"monitor",0,0,'M'},
-	{"status",1,0,'S'},
+	{"state",1,0,'S'},
 	{"listall",0,0,'L'},
 	{"information",1,0,'I'},
 	{"add",1,0,'A'},
 	{"delete",1,0,'D'},
+	{"fail",1,0,'X'},
 	{"raclass_supported",1,0,'C'},
 	{"ratype_supported",1,0,'T'},
 	{"all_type_metadata",1,0,'O'},
@@ -85,6 +88,7 @@ typedef enum {
 	INF_RSC,
 	ADD_RSC,
 	DEL_RSC,
+	FAIL_RSC,
 	RACLASS_SUPPORTED,
 	RATYPE_SUPPORTED,
 	RA_METADATA,
@@ -136,6 +140,7 @@ const char * simple_help_screen =
 "         {-A|--add} <rscid> <raclass> <ratype> <provider|NULL> [<rsc_params_list>]\n"
 "         {-D|--delete} <rscid>\n"
 "         {-F|--flush} <rscid>\n"
+"         {-X|--fail} <rscid> [<fail_rc> [<fail_reason>]]\n"
 "         {-E|--execute} <rscid> <operator> <timeout> <interval> <target_rc|EVERYTIME|CHANGED> [<operator_parameters_list>]\n"
 "         {-S|--state} <rscid> [-n <fake_name>]\n"
 "         {-L|--listall}\n"
@@ -157,6 +162,7 @@ const char * simple_help_screen =
 static int resource_operation(ll_lrm_t * lrmd, int argc, int optind, 
 			      char * argv[]);
 static int add_resource(ll_lrm_t * lrmd, int argc, int optind, char * argv[]);
+static int fail_resource(ll_lrm_t * lrmd, char *rsc_id, int optc, char *opts[]);
 static int transfer_cmd_params(int amount, int start, char * argv[], 
 			   const char * class, GHashTable ** params_ht);
 static void g_print_stringitem_and_free(gpointer data, gpointer user_data);
@@ -244,6 +250,12 @@ int main(int argc, char **argv)
 				}
 				break;
 
+			case 'X':
+				OPTION_OBSCURE_CHECK 
+				lrmadmin_cmd = FAIL_RSC;
+				strncpy(rscid_arg_tmp, optarg, RID_LEN-1);
+				break;
+
 			case 'C':
 				OPTION_OBSCURE_CHECK 
 				lrmadmin_cmd = RACLASS_SUPPORTED;
@@ -427,6 +439,19 @@ int main(int argc, char **argv)
 			ASYN_OPS = FALSE;
 			break;	
 
+		case FAIL_RSC:
+			/* Return value: HA_OK = 1 Or  HA_FAIL = 0 */
+			if (fail_resource(lrmd, rscid_arg_tmp,
+				argc-optind, argv+optind) == 1)
+			{
+				printf("Succeeded in failing the resource.\n");
+			} else {
+				printf("Failed to fail the resource.\n");
+				ret_value = -3;
+			}
+			ASYN_OPS = FALSE;
+			break;	
+
 		case FLUSH:
 			lrm_rsc = get_lrm_rsc(lrmd, rscid_arg_tmp);
 			if (!(lrm_rsc)) {
@@ -606,7 +631,7 @@ lrm_op_done_callback(lrm_op_t* op)
 	printf("return code: %d\n", op->rc);
 	printf("output data: \n%s\n", (op->output ? op->output : "[null]"));
 	printf("---------------------------------------\n\n");
-	ret_value = op->op_status;	
+	ret_value = op->rc;	
 }
 
 static int 
@@ -760,6 +785,25 @@ add_resource(ll_lrm_t * lrmd, int argc, int optind, char * argv[])
 	return (tmp_ret ? 0 : -1); /* tmp_ret is HA_OK=1 or HA_FAIL=0 */
 }
 
+static int 
+fail_resource(ll_lrm_t * lrmd, char *rsc_id, int optc, char *opts[])
+{
+	int fail_rc = 0;
+	const char * reason = NULL;
+
+	if (optc > 2) {
+		cl_log(LOG_ERR,"Bad usage.");
+		return -2;
+	}
+
+	if (optc >= 1)
+		fail_rc = atoi(opts[0]);
+	if (optc == 2)
+		reason = opts[1];
+
+	return lrmd->lrm_ops->fail_rsc(lrmd, rsc_id, fail_rc, reason);
+}
+
 static int
 transfer_cmd_params(int amount, int start, char * argv[], const char * class, 
 GHashTable ** params_ht)
@@ -784,11 +828,11 @@ GHashTable ** params_ht)
 				goto error_return; /* Have to */
 			}
 
-			len_tmp = strnlen(delimit+1, 80) + 1;
+			len_tmp = strnlen(delimit+1, MAX_PARAM_LEN) + 1;
 			value = g_new(gchar, len_tmp);
 			strncpy(value, delimit+1, len_tmp);
 
-			len_tmp = strnlen(argv[i], 80) - strnlen(delimit, 80);
+			len_tmp = strnlen(argv[i], MAX_PARAM_LEN) - strnlen(delimit, MAX_PARAM_LEN);
 			key = g_new(gchar, len_tmp+1);
 			key[len_tmp] = '\0';
 			strncpy(key, argv[i], len_tmp);
@@ -889,6 +933,7 @@ g_print_ops(gpointer data, gpointer user_data)
 {
 	lrm_op_t* op = (lrm_op_t*)data;
 	GString * param_gstr;
+	time_t run_at=0, rcchange_at=0;
 
 	if (NULL == op) {
 		cl_log(LOG_ERR, "%s:%d: op==NULL"
@@ -899,17 +944,29 @@ g_print_ops(gpointer data, gpointer user_data)
 	param_gstr = g_string_new("");
 	g_hash_table_foreach(op->params, ocf_params_hash_to_str, &param_gstr);
 
+	if( op->t_run )
+		run_at=(time_t)op->t_run;
+	if( op->t_rcchange )
+		rcchange_at=(time_t)op->t_rcchange;
 	printf("   operation '%s' [call_id=%d]:\n"
 	       "      start_delay=%d, interval=%d, timeout=%d, app_name=%s\n"
 	       "      rc=%d (%s), op_status=%d (%s)\n"
-	       "      parameters: %s\n"
 		, nullcheck(op->op_type), op->call_id
 		, op->start_delay, op->interval, op->timeout
 		, nullcheck(op->app_name), op->rc
 		, rc_msg[(op->rc-EXECRA_EXEC_UNKNOWN_ERROR) % DIMOF(rc_msg)]
 		, op->op_status
 		, status_msg[(op->op_status-LRM_OP_PENDING) % DIMOF(status_msg)]
-		, param_gstr->str);
+	);
+	if( op->t_run || op->t_rcchange )
+		printf("      run at: %s"
+			   "      last rc change at: %s"
+			   "      queue time: %lums, exec time: %lums\n"
+			, op->t_run ? ctime(&run_at) : "N/A\n"
+			, op->t_rcchange ? ctime(&rcchange_at) : "N/A\n"
+			, op->queue_time, op->exec_time
+		);
+	printf("      parameters: %s\n", param_gstr->str);
 	g_string_free(param_gstr, TRUE);
 }
 
