@@ -97,6 +97,7 @@ struct msg_map msg_maps[] = {
 	{GETLASTOP,	NO_MSG,	on_msg_get_last_op},
 	{GETALLRCSES,	NO_MSG,	on_msg_get_all},
 	{DELRSC,	REPLY_NOW,	on_msg_del_rsc},
+	{FAILRSC,	REPLY_NOW,	on_msg_fail_rsc},
 	{PERFORMOP,	REPLY_NOW,	on_msg_perform_op},
 	{FLUSHOPS,	REPLY_NOW,	on_msg_flush_all},
 	{CANCELOP,	REPLY_NOW,	on_msg_cancel_op},
@@ -170,7 +171,7 @@ ra_pipe_op_new(int child_stdout, int child_stderr, lrmd_op_t * lrmd_op)
 	}
 	rapop = cl_calloc(sizeof(ra_pipe_op_t), 1);
 	if ( rapop == NULL) {
-		lrmd_log(LOG_ERR, "%s:%d out of memory." 
+		lrmd_log(LOG_ERR, "%s:%d out of memory" 
 			, __FUNCTION__, __LINE__);
 		return NULL;
 	}
@@ -1218,7 +1219,10 @@ init_start ()
 	 * Calling this function accomplishes that.
 	 */
 	cl_set_all_coredump_signal_handlers();
-	drop_privs(0, 0); /* become "nobody" */
+	if( drop_privs(0, 0) ) { /* become "nobody" */
+		lrmd_log(LOG_WARNING,"%s: failed to drop privileges: %s"
+		, __FUNCTION__, strerror(errno));
+	}
 
 	/*
 	 * Add the signal handler for SIGUSR1, SIGUSR2. 
@@ -1261,7 +1265,9 @@ init_start ()
                 reg_to_apphbd = FALSE;
         }
 
-	return_to_orig_privs();
+	if( return_to_orig_privs() ) {
+		cl_perror("%s: failed to raise privileges", __FUNCTION__);
+	}
 	conn_cmd->ops->destroy(conn_cmd);
 	conn_cmd = NULL;
 
@@ -1777,10 +1783,11 @@ on_msg_get_metadata(lrmd_client_t* client, struct ha_msg* msg)
 	provider = ha_msg_value(msg, F_LRM_RPROVIDER);
 
 	lrmd_debug2(LOG_DEBUG
-	,	"%s: the client [pid:%d] want to get rsc metadata of %s::%s."
+	,	"%s: the client [pid:%d] wants to get rsc metadata of %s::%s::%s."
 	,	__FUNCTION__
 	,	client->pid
 	,	lrm_str(rclass)
+	,	lrm_str(provider)
 	,	lrm_str(rtype));
 
 	ret = create_lrm_ret(HA_OK, 5);
@@ -1795,13 +1802,19 @@ on_msg_get_metadata(lrmd_client_t* client, struct ha_msg* msg)
 	}
 	else {
 		char* meta = RAExec->get_resource_meta(rtype,provider);
-		if (NULL != meta) {
+		if (NULL != meta && strlen(meta) > 0) {
 			if (HA_OK != ha_msg_add(ret,F_LRM_METADATA, meta)) {
 				LOG_FAILED_TO_ADD_FIELD("metadata");
 			}
 			g_free(meta);
 		}
 		else {
+			lrmd_log(LOG_WARNING
+			, 	"%s: empty metadata for %s::%s::%s."
+			,	__FUNCTION__
+			,	lrm_str(rclass)
+			,	lrm_str(provider)
+			,	lrm_str(rtype));
 			ha_msg_mod_int(ret, F_LRM_RET, HA_FAIL);
 		}
 	}
@@ -2008,6 +2021,93 @@ on_msg_del_rsc(lrmd_client_t* client, struct ha_msg* msg)
 	return HA_OK;
 }
 
+static int
+prepare_failmsg(struct ha_msg* msg, int fail_rc, const char *fail_reason)
+{
+	call_id++; /* use the next id */
+	if (HA_OK != ha_msg_mod(msg,F_LRM_OP,ASYNC_OP_NAME)
+		|| HA_OK != ha_msg_add(msg,F_LRM_FAIL_REASON,fail_reason)
+		|| HA_OK != ha_msg_mod_int(msg,F_LRM_ASYNCMON_RC,fail_rc)
+		|| HA_OK != ha_msg_mod_int(msg,F_LRM_RC,fail_rc)
+		|| HA_OK != ha_msg_mod_int(msg,F_LRM_OPSTATUS,(int)LRM_OP_DONE)
+		|| HA_OK != ha_msg_mod_int(msg,F_LRM_CALLID,call_id)
+		|| HA_OK != ha_msg_mod_int(msg,F_LRM_TIMEOUT,0)
+		|| HA_OK != ha_msg_mod_int(msg,F_LRM_INTERVAL,0)
+		|| HA_OK != ha_msg_mod_int(msg,F_LRM_TARGETRC,EVERYTIME)
+		|| HA_OK != ha_msg_mod_int(msg,F_LRM_DELAY,0)
+	) {
+		lrmd_log(LOG_ERR,"%s:%d: cannot add field to a message"
+		,	__FUNCTION__, __LINE__);
+		return 1;
+	}
+	return 0;
+}
+
+static void
+async_notify(gpointer key, gpointer val, gpointer data)
+{
+	struct ha_msg* msg = (struct ha_msg*)data;
+	lrmd_client_t* client;
+
+	client = lookup_client_by_name((char *)key);
+	if (!client) {
+		lrmd_log(LOG_INFO,
+			"%s: client %s not found, probably signed out", __FUNCTION__, (char *)key);
+		return;
+	}
+	if (HA_OK != ha_msg_mod(msg,F_LRM_APP,client->app_name)) {
+		lrmd_log(LOG_ERR,"%s:%d: cannot add field to a message"
+		,	__FUNCTION__, __LINE__);
+		return;
+	}
+	if (!client->ch_cbk) {
+		lrmd_log(LOG_WARNING,
+			"%s: callback channel is null", __FUNCTION__);
+	} else if (HA_OK != msg2ipcchan(msg, client->ch_cbk)) {
+		lrmd_log(LOG_WARNING,
+			"%s: can not send the ret msg", __FUNCTION__);
+	}
+}
+
+int
+on_msg_fail_rsc(lrmd_client_t* client, struct ha_msg* msg)
+{
+	lrmd_rsc_t* rsc;
+	const char* id;
+	int fail_rc = -1;
+	const char *fail_reason;
+
+	CHECK_ALLOCATED(client, "client", HA_FAIL);
+	CHECK_ALLOCATED(msg, "message", HA_FAIL);
+
+	id = ha_msg_value(msg, F_LRM_RID);
+	lrmd_debug2(LOG_DEBUG
+	,	"%s: client [%d] wants to fail rsc %s"
+	,	__FUNCTION__, client->pid, lrmd_nullcheck(id));
+
+	rsc = lookup_rsc_by_msg(msg);
+	if (!rsc) {
+		lrmd_log(LOG_ERR, "%s: no resource with id %s."
+		,	__FUNCTION__, lrmd_nullcheck(id));
+		return HA_FAIL;
+	}
+	fail_reason = ha_msg_value(msg,F_LRM_FAIL_REASON);
+	if (!fail_reason || *fail_reason == '\0') {
+		fail_reason = DEFAULT_FAIL_REASON;
+	}
+	if (HA_OK != ha_msg_value_int(msg,F_LRM_ASYNCMON_RC,&fail_rc) || fail_rc <= 0) {
+		fail_rc = DEFAULT_FAIL_RC;
+	}
+	if (prepare_failmsg(msg,fail_rc,fail_reason))
+		return HA_FAIL;
+	lrmd_log(LOG_WARNING
+	,	"received asynchronous failure for rsc %s (rc: %d, reason: %s)"
+	,	lrmd_nullcheck(id), fail_rc, fail_reason);
+	/* notify all clients from last_op table about the failure */
+	g_hash_table_foreach(rsc->last_op_table,async_notify,msg);
+	return HA_OK;
+}
+
 static gboolean
 free_str_hash_pair(gpointer key, gpointer value, gpointer user_data)
 {
@@ -2254,7 +2354,7 @@ on_msg_perform_op(lrmd_client_t* client, struct ha_msg* msg)
 		LOG_FAILED_TO_ADD_FIELD("callid");
 		return -1;
 	}
-	if (HA_OK !=ha_msg_add(msg, F_LRM_APP, client->app_name)) {
+	if (HA_OK !=ha_msg_mod(msg, F_LRM_APP, client->app_name)) {
 		LOG_FAILED_TO_ADD_FIELD("app_name");
 		return -1;
 	}
@@ -2408,6 +2508,7 @@ on_msg_get_state(lrmd_client_t* client, struct ha_msg* msg)
 	if(last_ops != NULL) {
 		g_hash_table_foreach(last_ops, send_last_op, client->ch_cmd);
 	}
+
 	/* send the ops in op list */
 	for(node = g_list_first(rsc->op_list)
 	;	NULL != node; node = g_list_next(node)){
@@ -2446,11 +2547,6 @@ on_msg_get_state(lrmd_client_t* client, struct ha_msg* msg)
 	return HA_OK;
 }
 
-#define mk_op_id(op,id) do { \
-	const char *op_type = ha_msg_value(op->msg, F_LRM_OP); \
-	const char *op_interval = ha_msg_value(op->msg, F_LRM_INTERVAL); \
-	id = lrm_concat(op_type, op_interval, '_'); \
-} while(0)
 #define safe_len(s) (s ? strlen(s) : 0)
 
 static char *
@@ -2474,55 +2570,97 @@ lrm_concat(const char *prefix, const char *suffix, char join)
 	return new_str;
 }
 
-/* /////////////////////op functions//////////////////////////////////////////// */
-static void 
-record_op_completion(lrmd_client_t* client, lrmd_rsc_t* rsc, lrmd_op_t* op)
+/* /////////////////////op functions////////////////////// */
+
+#define mk_op_id(op,id) do { \
+	const char *op_type = ha_msg_value(op->msg, F_LRM_OP); \
+	const char *op_interval = ha_msg_value(op->msg, F_LRM_INTERVAL); \
+	id = lrm_concat(op_type, op_interval, '_'); \
+} while(0)
+
+/* find the last operation for the client
+ * replace it with the new one (if requested)
+ */
+static void
+replace_last_op(lrmd_client_t* client, lrmd_rsc_t* rsc, lrmd_op_t* op)
 {
-	char *op_hash_key = NULL;
-	lrmd_op_t* old_op = NULL;
-	lrmd_op_t* new_op = NULL;
-	GHashTable* client_last_op = NULL;
+	char *op_hash_key;
+	GHashTable *client_last_op;
+	lrmd_op_t *old_op, *new_op;
 
-	LRMAUDIT();
-	/*save the op in the last op finished*/
-	if (rsc->last_op_done != NULL) {
-		lrmd_op_destroy(rsc->last_op_done);
-	}
-	rsc->last_op_done = lrmd_op_copy(op);
-	rsc->last_op_done->repeat_timeout_tag = (guint)0;
-
-	if (!client) {
-		lrmd_debug(LOG_DEBUG, "%s: cannot record %s: client is gone."
-		,	__FUNCTION__, small_op_info(op)); 
-		LRMAUDIT();
+	if (!client || !rsc || !op)
 		return;
-	}
-	/*find the hash table for the client*/
-	client_last_op = g_hash_table_lookup(rsc->last_op_table
-	, 			client->app_name);
-	if (NULL == client_last_op) {
+	client_last_op = g_hash_table_lookup(rsc->last_op_table, client->app_name);
+	if (!client_last_op) {
 		client_last_op = g_hash_table_new_full(	g_str_hash
 		, 	g_str_equal, cl_free, NULL);
 		g_hash_table_insert(rsc->last_op_table
 		,	(gpointer)cl_strdup(client->app_name)
 		,	(gpointer)client_last_op);
 	}
-
 	mk_op_id(op,op_hash_key);
-	old_op = g_hash_table_lookup(client_last_op, op_hash_key);
-	new_op = lrmd_op_copy(op);
-	if (NULL != old_op) {
-		g_hash_table_replace(client_last_op
-		, 	op_hash_key
-		,	(gpointer)new_op);
-		/* Don't let the timers go away */
-		lrmd_op_destroy(old_op);
-	}else{
-		g_hash_table_insert(client_last_op
-		, 	op_hash_key
-		,	(gpointer)new_op);
+	old_op = (lrmd_op_t*)g_hash_table_lookup(client_last_op, op_hash_key);
+
+	/* make a copy of op and insert it into client_last_op */
+	if (!(new_op = lrmd_op_copy(op))) {
+		lrmd_log(LOG_ERR, "%s:%d out of memory" 
+			, __FUNCTION__, __LINE__);
 	}
+	if (old_op) {
+		g_hash_table_replace(client_last_op,op_hash_key,(gpointer)new_op);
+		lrmd_op_destroy(old_op);
+	} else {
+		g_hash_table_insert(client_last_op,op_hash_key,(gpointer)new_op);
+	}
+}
+
+static void
+record_op_completion(lrmd_rsc_t* rsc, lrmd_op_t* op)
+{
+	lrmd_client_t* client;
+
 	LRMAUDIT();
+	/*save the op in the last op finished*/
+	if (rsc->last_op_done != NULL) {
+		lrmd_op_destroy(rsc->last_op_done);
+	}
+	if (!(rsc->last_op_done = lrmd_op_copy(op))) {
+		lrmd_log(LOG_ERR, "%s:%d out of memory" 
+			, __FUNCTION__, __LINE__);
+	}
+	rsc->last_op_done->repeat_timeout_tag = (guint)0;
+
+	client = lookup_client(op->client_id);
+	if (!client) {
+		lrmd_debug(LOG_DEBUG, "%s: cannot record %s: client is gone."
+		,	__FUNCTION__, small_op_info(op)); 
+		LRMAUDIT();
+		return;
+	}
+	/* insert (or replace) the new op in last_op_table for the client */
+	replace_last_op(client,rsc,op);
+	LRMAUDIT();
+}
+
+static void
+to_repeatlist(lrmd_rsc_t* rsc, lrmd_op_t* op)
+{
+	lrmd_op_t *repeat_op;
+
+	if (!(repeat_op = lrmd_op_copy(op))) {
+		lrmd_log(LOG_ERR, "%s:%d out of memory" 
+			, __FUNCTION__, __LINE__);
+	}
+	reset_timestamps(repeat_op);
+	repeat_op->is_copy = FALSE;
+	repeat_op->repeat_timeout_tag = 
+		Gmain_timeout_add(op->interval,	
+				on_repeat_op_readytorun, repeat_op);
+	rsc->repeat_op_list = 
+		g_list_append (rsc->repeat_op_list, repeat_op);
+	lrmd_debug2(LOG_DEBUG
+	, "on_op_done:repeat %s is added to repeat op list to wait" 
+	, op_info(op));
 }
 
 static void 
@@ -2591,102 +2729,65 @@ add_op_to_runlist(lrmd_rsc_t* rsc, lrmd_op_t* op)
 int
 on_op_done(lrmd_rsc_t* rsc, lrmd_op_t* op)
 {
-	int target_rc = -1;
-	int last_rc = -1;
-	int op_rc = -1;
+	int target_rc, last_rc, op_rc;
+	int rc_changed;
 	op_status_t op_status;
-	int op_status_int;
-	int need_notify = 0;
-	lrmd_client_t* client = NULL;
 
 	LRMAUDIT();
 	CHECK_ALLOCATED(op, "op", HA_FAIL );
 	if (op->exec_pid == 0) {
-		lrmd_log(LOG_ERR, "on_op_done: op->exec_pid == 0.");
+		lrmd_log(LOG_ERR, "%s: op->exec_pid == 0",__FUNCTION__);
 		return HA_FAIL;
 	}
 	op->t_done = time_longclock();
 
-	lrmd_debug2(LOG_DEBUG, "on_op_done: %s", op_info(op));
-	lrmd_debug2(LOG_DEBUG
-		 ,"Timestamps: Recv:%ld, Add to List:%ld, Perform:%ld"
-		 ", Done: %ld, Rc change: %ld"
-		 ,longclockto_ms(op->t_recv)
-		 ,longclockto_ms(op->t_addtolist)
-		 ,longclockto_ms(op->t_perform)
-		 ,longclockto_ms(op->t_done)
-		 ,longclockto_ms(op->t_rcchange));
-
-	if (HA_OK != ha_msg_value_int(op->msg,F_LRM_TARGETRC,&target_rc)){
-		lrmd_log(LOG_ERR
-		,	"%s: can not get target status field from a message"
-		,	__FUNCTION__);
-		return HA_FAIL;
-	}
-	if (HA_OK !=
-		ha_msg_value_int(op->msg, F_LRM_OPSTATUS, &op_status_int)) {
-		lrmd_log(LOG_ERR
-		,	"%s: can not get operation status field from a message"
-		,	__FUNCTION__);
-		return HA_FAIL;
-	}
-	op_status = (op_status_t)op_status_int;
-
 	if (debug_level >= 2) {
+		lrmd_debug(LOG_DEBUG, "%s: %s",__FUNCTION__, op_info(op));
 		lrmd_op_dump(op, __FUNCTION__);
 	}
 
+	return_on_no_int_value(op->msg,F_LRM_TARGETRC,&target_rc);
+	return_on_no_int_value(op->msg,F_LRM_OPSTATUS,(int *)&op_status);
+
+	last_rc = op_rc = -1; /* set all rc to -1 */
 	ha_msg_value_int(op->msg,F_LRM_RC,&op_rc);
-	ha_msg_value_int(op->msg,F_LRM_LASTRC, &last_rc);
-	if (op_status != LRM_OP_DONE
-		|| (op_rc == -1)
-		|| (op_rc == target_rc)
-		|| (target_rc == EVERYTIME)
-		|| ((target_rc == CHANGED)
-			&& ((last_rc == -1) || (last_rc != op_rc)))
-		) {
-		need_notify = 1;
-	}
-	if (op_status == LRM_OP_DONE
-		&& CHANGED == target_rc
+	ha_msg_value_int(op->msg,F_LRM_LASTRC,&last_rc);
+	rc_changed = (
+		op_status == LRM_OP_DONE
 		&& op_rc != -1
-		&& HA_OK != ha_msg_mod_int(op->msg, F_LRM_LASTRC, op_rc)) {
-		lrmd_log(LOG_ERR,"on_op_done: can not save status to "
-			"the message op->msg.");
-		return HA_FAIL;
-	}
-	if ((last_rc == -1) || (last_rc != op_rc)) {
+		&& ((last_rc == -1) || (last_rc != op_rc))
+	);
+	if (rc_changed) {
+		if (HA_OK != ha_msg_mod_int(op->msg, F_LRM_LASTRC, op_rc)) {
+			lrmd_log(LOG_ERR,"%s: cannot save status to msg",__FUNCTION__);
+			return HA_FAIL;
+		}
 		op->t_rcchange = op->t_perform;
 	}
+	if (store_timestamps(op))
+		return HA_FAIL;
 
-	/* remove the op from op_list and copy to last_op */
+	/* remove the op from op_list */
 	rsc->op_list = g_list_remove(rsc->op_list,op);
 	lrmd_debug2(LOG_DEBUG
-	, 	"on_op_done:%s is removed from op list" 
-	,	op_info(op));
+	, 	"%s:%s is removed from op list"
+	,	__FUNCTION__, op_info(op));
 
-	client = lookup_client(op->client_id);
-	if ( LRM_OP_CANCELLED != op_status ) {
-		/*record the outcome of the op */
-		record_op_completion(client, rsc, op);
-		/*copy the repeat op to repeat list to wait next perform */
-		if ( client && op->interval ) {
-			lrmd_op_t* repeat_op = lrmd_op_copy(op);
-			repeat_op->is_copy = FALSE;
-			repeat_op->repeat_timeout_tag = 
-				Gmain_timeout_add(op->interval,	
-						on_repeat_op_readytorun, repeat_op);
-			rsc->repeat_op_list = 
-				g_list_append (rsc->repeat_op_list, repeat_op);
-			lrmd_debug2(LOG_DEBUG
-			, "on_op_done:repeat %s is added to repeat op list to wait" 
-			, op_info(op));
+	if (op_status != LRM_OP_CANCELLED) {
+		record_op_completion(rsc,op); /*record the outcome of the op */
+		if (op->interval) { /* copy op to the repeat list */
+			to_repeatlist(rsc,op);
 		}
 	} else {
 		remove_op_history(op);
 	}
 
-	if ( need_notify ) {
+	if (op_status != LRM_OP_DONE
+		|| (op_rc == -1)
+		|| (op_rc == target_rc)
+		|| (target_rc == EVERYTIME)
+		|| ((target_rc == CHANGED) && rc_changed)
+	) {
 		notify_client(op);
 	}
 	lrmd_op_destroy(op);
@@ -2711,11 +2812,11 @@ flush_op(lrmd_op_t* op)
 {
 	CHECK_ALLOCATED(op, "op", HA_FAIL );
 	if (op->exec_pid == 0) {
-		lrmd_debug(LOG_ERR, "%s: op->exec_pid == 0.",__FUNCTION__);
+		lrmd_debug(LOG_ERR, "%s: op->exec_pid == 0",__FUNCTION__);
 		return HA_FAIL;
 	}
 
-	if (HA_OK != ha_msg_add_int(op->msg, F_LRM_RC, HA_FAIL)) {
+	if (HA_OK != ha_msg_mod_int(op->msg, F_LRM_RC, HA_FAIL)) {
 		LOG_FAILED_TO_ADD_FIELD("F_LRM_RC");
 		return HA_FAIL;
 	}
@@ -2837,36 +2938,14 @@ perform_op(lrmd_rsc_t* rsc)
 	return HA_OK;
 }
 
-struct ha_msg*
-op_to_msg(lrmd_op_t* op)
+static int
+store_timestamps(lrmd_op_t* op)
 {
-	struct ha_msg* msg = NULL;
-	longclock_t	now = time_longclock(),
+	struct ha_msg* msg = op->msg;
+	longclock_t	now = time_longclock(), /* tm2unix() needs this */
 		exec_time = zero_longclock,
 		queue_time = zero_longclock;
 
-	CHECK_ALLOCATED(op, "op", NULL);
-	if (op->exec_pid == 0) {
-		lrmd_log(LOG_ERR, "op_to_msg: op->exec_pid is 0.");
-		return NULL;
-	}
-	msg = ha_msg_copy(op->msg);
-	if (NULL == msg) {
-		lrmd_log(LOG_ERR,"op_to_msg: can not copy the msg");
-		return NULL;
-	}
-	if (HA_OK != ha_msg_add_int(msg, F_LRM_CALLID, op->call_id)) {
-		LOG_FAILED_TO_ADD_FIELD("call_id");
-		goto error;
-	}
-	if (HA_OK != ha_msg_add_ul(msg, F_LRM_T_RUN, tm2age(op->t_perform))) {
-		LOG_FAILED_TO_ADD_FIELD("t_run")
-		goto error;
-	}
-	if (HA_OK != ha_msg_add_ul(msg, F_LRM_T_RCCHANGE, tm2age(op->t_rcchange))) {
-		LOG_FAILED_TO_ADD_FIELD("t_rcchange")
-		goto error;
-	}
 	if (op->t_perform) {
 		queue_time =
 			longclockto_ms(sub_longclock(op->t_perform,op->t_addtolist));
@@ -2875,18 +2954,49 @@ op_to_msg(lrmd_op_t* op)
 				longclockto_ms(sub_longclock(op->t_done,op->t_perform));
 		}
 	}
-	if (HA_OK != ha_msg_add_ul(msg, F_LRM_EXEC_TIME, exec_time)) {
-		LOG_FAILED_TO_ADD_FIELD("exec_time")
-		goto error;
+	if ((HA_OK!=ha_msg_mod_ul(msg,F_LRM_T_RUN,tm2unix(op->t_perform)))
+		|| (HA_OK!=ha_msg_mod_ul(msg,F_LRM_T_RCCHANGE,tm2unix(op->t_rcchange)))
+		|| (HA_OK!=ha_msg_mod_ul(msg,F_LRM_EXEC_TIME,exec_time))
+		|| (HA_OK!=ha_msg_mod_ul(msg,F_LRM_QUEUE_TIME,queue_time))
+	) {
+		lrmd_log(LOG_ERR,"%s: can not save timestamps to msg",__FUNCTION__);
+		return 1;
 	}
-	if (HA_OK != ha_msg_add_ul(msg, F_LRM_QUEUE_TIME, queue_time)) {
-		LOG_FAILED_TO_ADD_FIELD("queue_time")
-		goto error;
+	return 0;
+}
+
+static void
+reset_timestamps(lrmd_op_t* op)
+{
+	op->t_perform = zero_longclock;
+	op->t_done = zero_longclock;
+	cl_msg_remove(op->msg, F_LRM_T_RUN);
+	cl_msg_remove(op->msg, F_LRM_T_RCCHANGE);
+	cl_msg_remove(op->msg, F_LRM_EXEC_TIME);
+	cl_msg_remove(op->msg, F_LRM_QUEUE_TIME);
+}
+
+struct ha_msg*
+op_to_msg(lrmd_op_t* op)
+{
+	struct ha_msg* msg = NULL;
+
+	CHECK_ALLOCATED(op, "op", NULL);
+	if (op->exec_pid == 0) {
+		lrmd_log(LOG_ERR, "%s: op->exec_pid is 0",__FUNCTION__);
+		return NULL;
+	}
+	msg = ha_msg_copy(op->msg);
+	if (NULL == msg) {
+		lrmd_log(LOG_ERR,"%s: can not copy the msg",__FUNCTION__);
+		return NULL;
+	}
+	if ((HA_OK!=ha_msg_mod_int(msg,F_LRM_CALLID,op->call_id))) {
+		lrmd_log(LOG_ERR,"%s: can not save F_LRM_CALLID to msg",__FUNCTION__);
+		ha_msg_del(msg);
+		msg = NULL;
 	}
 	return msg;
-
-error:	ha_msg_del(msg);
-	return NULL;
 }
 
 /* //////////////////////////////RA wrap funcs/////////////////////////////////// */
@@ -2918,7 +3028,7 @@ perform_ra_op(lrmd_op_t* op)
 	}
 
 	if (op->exec_pid == 0) {
-		lrmd_log(LOG_ERR, "perform_ra_op: op->exec_pid == 0.");
+		lrmd_log(LOG_ERR, "%s::%d: op->exec_pid == 0.", __FUNCTION__, __LINE__);
 		return HA_FAIL;
 	}
 
@@ -2938,25 +3048,31 @@ perform_ra_op(lrmd_op_t* op)
 
 	if(HA_OK != ha_msg_value_int(op->msg, F_LRM_TIMEOUT, &timeout)){
 		timeout = 0;
-		lrmd_log(LOG_ERR,"perform_ra_op: failed to get timeout from "
-			"the message op->msg.");
+		lrmd_log(LOG_ERR,"%s::%d: failed to get timeout for %s"
+		, __FUNCTION__, __LINE__, small_op_info(op));
 	}
 	
-	return_to_orig_privs();
+	if( return_to_orig_privs() ) {
+		cl_perror("%s::%d: failed to raise privileges"
+		, __FUNCTION__, __LINE__);
+	}
 	switch(pid=fork()) {
 		case -1:
-			cl_perror("perform_ra_op:fork failure");
+			cl_perror("%s::%d: fork", __FUNCTION__, __LINE__);
 			close(stdout_fd[0]);
 			close(stdout_fd[1]);
 			close(stderr_fd[0]);
 			close(stderr_fd[1]);
-			return_to_dropped_privs();
+			if( return_to_dropped_privs() ) {
+				cl_perror("%s::%d: failed to drop privileges"
+				, __FUNCTION__, __LINE__);
+			}
 			return HA_FAIL;
 
 		default:	/* Parent */
 			child_count++;
 			NewTrackedProc(pid, 1
-			,	debug_level ? PT_LOGVERBOSE : PT_LOGNONE
+			,	debug_level ? (op->interval ? PT_LOGNORMAL : PT_LOGVERBOSE) : PT_LOGNONE
 			,	op, &ManagedChildTrackOps);
 
 			close(stdout_fd[1]);
@@ -2980,7 +3096,10 @@ perform_ra_op(lrmd_op_t* op)
 
 				SetTrackedProcTimeouts(pid, op->killseq);
 			}
-			return_to_dropped_privs();
+			if( return_to_dropped_privs() ) {
+				lrmd_log(LOG_WARNING,"%s::%d: failed to drop privileges: %s"
+				, __FUNCTION__, __LINE__, strerror(errno));
+			}
 
 			if ( rapop == NULL) {
 				return HA_FAIL;
@@ -3005,8 +3124,7 @@ perform_ra_op(lrmd_op_t* op)
 			}
 			if (STDERR_FILENO != stderr_fd[1]) {
 				if (dup2(stderr_fd[1], STDERR_FILENO)!=STDERR_FILENO) {
-					cl_perror("%s::%d: dup2"
-						, __FUNCTION__, __LINE__);
+					cl_perror("%s::%d: dup2", __FUNCTION__, __LINE__);
 				}
 				close(stderr_fd[1]);
 			}
@@ -3014,9 +3132,11 @@ perform_ra_op(lrmd_op_t* op)
 			if (NULL == RAExec) {
 				close(stdout_fd[1]);
 				close(stderr_fd[1]);
-				lrmd_log(LOG_ERR,"perform_ra_op: can not find RAExec");
+				lrmd_log(LOG_ERR,"%s::%d: can't find RAExec for class %s"
+				, __FUNCTION__, __LINE__, rsc->class);
 				exit(EXECRA_EXEC_UNKNOWN_ERROR);
 			}
+			
 			/*should we use logging daemon or not in script*/
 			setenv(HALOGD, cl_log_get_uselogd()?"yes":"no",1);
 
@@ -3062,8 +3182,9 @@ on_ra_proc_finished(ProcTrack* p, int status, int signo, int exitcode
 
 	LRMAUDIT();
 	if (--child_count < 0) {
-		lrmd_log(LOG_ERR, "%s:%d: child number is less than zero: %d"
+		lrmd_log(LOG_ERR, "%s:%d: child count is less than zero: %d"
 			, __FUNCTION__, __LINE__, child_count);
+		child_count = 0;
 	}
 
 	CHECK_ALLOCATED(p, "ProcTrack p", );
@@ -3266,6 +3387,19 @@ lookup_client (pid_t pid)
 	return (lrmd_client_t*) g_hash_table_lookup(clients, &pid);
 }
 
+static gboolean
+client_cmp_name(gpointer key, gpointer val, gpointer app_name)
+{
+	return strcmp(((lrmd_client_t*)val)->app_name,(char *)app_name) ?
+		FALSE : TRUE;
+}
+
+static lrmd_client_t*
+lookup_client_by_name(char *app_name)
+{
+	return (lrmd_client_t*)g_hash_table_find(clients,client_cmp_name,app_name);
+}
+
 lrmd_rsc_t*
 lookup_rsc (const char* rid)
 {
