@@ -26,6 +26,7 @@ static int on_msg_get_rsc(lrmd_client_t* client, struct ha_msg* msg);
 static int on_msg_get_last_op(lrmd_client_t* client, struct ha_msg* msg);
 static int on_msg_get_all(lrmd_client_t* client, struct ha_msg* msg);
 static int on_msg_del_rsc(lrmd_client_t* client, struct ha_msg* msg);
+static int on_msg_fail_rsc(lrmd_client_t* client, struct ha_msg* msg);
 static int on_msg_cancel_op(lrmd_client_t* client, struct ha_msg* msg);
 static int on_msg_flush_all(lrmd_client_t* client, struct ha_msg* msg);
 static int on_msg_perform_op(lrmd_client_t* client, struct ha_msg* msg);
@@ -55,6 +56,8 @@ static int read_pipe(int fd, char ** data, gpointer user_data);
 static gboolean handle_pipe_ra_stdout(int fd, gpointer user_data);
 static gboolean handle_pipe_ra_stderr(int fd, gpointer user_data);
 static struct ha_msg* op_to_msg(lrmd_op_t* op);
+static int store_timestamps(lrmd_op_t* op);
+static void reset_timestamps(lrmd_op_t* op);
 static gboolean lrm_shutdown(void);
 static gboolean can_shutdown(void);
 static gboolean free_str_hash_pair(gpointer key
@@ -63,7 +66,9 @@ static gboolean free_str_op_pair(gpointer key
 ,	 gpointer value, gpointer user_data);
 static lrmd_op_t* lrmd_op_copy(const lrmd_op_t* op);
 static void send_last_op(gpointer key, gpointer value, gpointer user_data);
-static void record_op_completion(lrmd_client_t* client, lrmd_rsc_t* rsc, lrmd_op_t* op);
+static void replace_last_op(lrmd_client_t* client, lrmd_rsc_t* rsc, lrmd_op_t* op);
+static void record_op_completion(lrmd_rsc_t* rsc, lrmd_op_t* op);
+static void to_repeatlist(lrmd_rsc_t* rsc, lrmd_op_t* op);
 static void remove_op_history(lrmd_op_t* op);
 static void hash_to_str(GHashTable * , GString *);
 static void hash_to_str_foreach(gpointer key, gpointer value, gpointer userdata);
@@ -71,6 +76,11 @@ static void warning_on_active_rsc(gpointer key, gpointer value, gpointer user_da
 static void check_queue_duration(lrmd_op_t* op);
 static gboolean flush_all(GList** listp);
 static gboolean cancel_op(GList** listp,int cancel_op_id);
+static int prepare_failmsg(struct ha_msg* msg,
+			int fail_rc, const char *fail_reason);
+static void async_notify(gpointer key, gpointer val, gpointer data);
+static gboolean client_cmp_name(gpointer key, gpointer val, gpointer app_name);
+static lrmd_client_t* lookup_client_by_name(char *app_name);
 
 /*
  * following functions are used to monitor the exit of ra proc
