@@ -23,6 +23,7 @@
 
 #include <unistd.h>
 #include <glib.h>
+#include <regex.h>
 
 #include <heartbeat.h>
 #include <clplumbing/cl_malloc.h>
@@ -45,9 +46,10 @@ int in_shutdown = FALSE;
 int init_crm(int cache_cib);
 void final_crm(void);
 
-static void on_cib_diff(const char *event, HA_Message *msg);
+static void on_cib_diff(const char *event, crm_data_t *msg);
 
 static char* on_get_cib_version(char* argv[], int argc);
+static char* on_get_crm_dtd(char* argv[], int argc);
 
 static char* on_get_crm_metadata(char* argv[], int argc);
 static char* on_get_crm_config(char* argv[], int argc);
@@ -100,6 +102,8 @@ static char* on_get_constraint(char* argv[], int argc);
 static char* on_update_constraint(char* argv[], int argc);
 static char* on_delete_constraint(char* argv[], int argc);
 
+static char* on_gen_cluster_report(char* argv[], int argc);
+
 static void get_meta_attributes_id(const char* rsc_id, char* id);
 static void get_instance_attributes_id(const char* rsc_id, char* id);
 static void get_attr_id(const char* rsc_id, const char* attr_type, const char* attr, char* id);
@@ -115,7 +119,8 @@ static const char* uname2id(const char* node);
 static resource_t* get_parent(resource_t* child);
 static int get_fix(const char* rsc_id, char* prefix, char* suffix, char* real_id);
 static const char* get_rsc_tag(resource_t* rsc);
-static int cl_msg_swap_offset(struct ha_msg* msg, int offset1, int offset2);
+static int cl_msg_swap_offset(crm_data_t* msg, int offset1, int offset2);
+int regex_match(const char *regex, const char *str);
 
 pe_working_set_t* cib_cached = NULL;
 int cib_cache_enable = FALSE;
@@ -129,16 +134,9 @@ int cib_cache_enable = FALSE;
 /* internal functions */
 GList* find_xml_node_list(crm_data_t *root, const char *child_name)
 {
-	int i;
 	GList* list = NULL;
-	if (root == NULL) {
-		return NULL;
-	}
-	for (i = 0; i < root->nfields; i++ ) {
-		if (strncmp(root->names[i], child_name, MAX_STRLEN) == 0) {
-			list = g_list_append(list, root->values[i]);
-		}
-	}
+	xml_child_iter_filter(root, child, child_name,
+			      list = g_list_append(list, child));
 	return list;
 }
 
@@ -161,10 +159,7 @@ delete_object(const char* type, const char* entry, const char* id, crm_data_t**
 	rc = cib_conn->cmds->delete(
 			cib_conn, type, cib_object, output, cib_sync_call);
 	free_xml(cib_object);
-	if (rc < 0) {
-		return -1;
-	}
-	return 0;
+	return rc;
 }
 
 pe_working_set_t*
@@ -229,9 +224,9 @@ crm_failed_msg(crm_data_t* output, int rc)
 		return ret;
 	}
 	
-	failed_tag = cl_get_struct(output, XML_FAIL_TAG_CIB);
+	failed_tag = find_entity(output, XML_FAIL_TAG_CIB, NULL);
 	if (failed_tag != NULL) {
-		reason = ha_msg_value(failed_tag, XML_FAILCIB_ATTR_REASON);
+		reason = crm_element_value(failed_tag, XML_FAILCIB_ATTR_REASON);
 		if (reason != NULL) {
 			ret = mgmt_msg_append(ret, reason);
 		}
@@ -344,7 +339,7 @@ get_meta_attributes_id(const char* rsc_id, char* id)
 	resource_t* rsc;
 	const char* cur_id;
 	pe_working_set_t* data_set;
-	struct ha_msg* attrs;
+	crm_data_t* attrs;
 	
 	data_set = get_data_set();
 	rsc = pe_find_resource(data_set->resources, rsc_id);	
@@ -353,13 +348,13 @@ get_meta_attributes_id(const char* rsc_id, char* id)
 		free_data_set(data_set);
 		return;
 	}
-	attrs = cl_get_struct((struct ha_msg*)rsc->xml, "meta_attributes");
+	attrs = find_entity(rsc->xml, "meta_attributes", NULL);
 	if (attrs == NULL) {
 		snprintf(id, MAX_STRLEN, "%s_meta_attrs", rsc_id);
 		free_data_set(data_set);
 		return;
 	}
-	cur_id = ha_msg_value(attrs, "id");
+	cur_id = crm_element_value(attrs, "id");
 	if (cur_id == NULL) {
 		snprintf(id, MAX_STRLEN, "%s_meta_attrs", rsc_id);
 		free_data_set(data_set);
@@ -376,7 +371,7 @@ get_instance_attributes_id(const char* rsc_id, char* id)
 	resource_t* rsc;
 	const char* cur_id;
 	pe_working_set_t* data_set;
-	struct ha_msg* attrs;
+	crm_data_t * attrs;
 	
 	data_set = get_data_set();
 	rsc = pe_find_resource(data_set->resources, rsc_id);	
@@ -385,13 +380,13 @@ get_instance_attributes_id(const char* rsc_id, char* id)
 		free_data_set(data_set);
 		return;
 	}
-	attrs = cl_get_struct((struct ha_msg*)rsc->xml, "instance_attributes");
+	attrs = find_entity(rsc->xml, "instance_attributes", NULL);
 	if (attrs == NULL) {
 		snprintf(id, MAX_STRLEN, "%s_instance_attrs", rsc_id);
 		free_data_set(data_set);
 		return;
 	}
-	cur_id = ha_msg_value(attrs, "id");
+	cur_id = crm_element_value(attrs, "id");
 	if (cur_id == NULL) {
 		snprintf(id, MAX_STRLEN, "%s_instance_attrs", rsc_id);
 		free_data_set(data_set);
@@ -405,12 +400,10 @@ get_instance_attributes_id(const char* rsc_id, char* id)
 static void
 get_attr_id(const char* rsc_id, const char* attr_type, const char* attr, char* id)
 {
-	int i;
 	resource_t* rsc;
 	const char * name_nvpair;
 	const char * id_nvpair;
-	struct ha_msg* attrs;
-	struct ha_msg* nvpair;
+	crm_data_t * attrs;
 	pe_working_set_t* data_set;
 	const char * mid = "";
 
@@ -427,32 +420,31 @@ get_attr_id(const char* rsc_id, const char* attr_type, const char* attr, char* i
 		return;
 	}
 
-	attrs = cl_get_struct((struct ha_msg*)rsc->xml, attr_type);
+	attrs = find_entity(rsc->xml, attr_type, NULL);
 	if(attrs == NULL) {
 		snprintf(id, MAX_STRLEN,  "%s_%s%s", rsc_id, mid, attr);
 		free_data_set(data_set);
 		return;
 	}
-	attrs = cl_get_struct(attrs, "attributes");
+	attrs = find_entity(attrs, "attributes", NULL);
 	if(attrs == NULL) {
 		snprintf(id, MAX_STRLEN,  "%s_%s%s", rsc_id, mid, attr);
 		free_data_set(data_set);
 		return;
 	}
-	for (i = 0; i < attrs->nfields; i++) {
-		if (STRNCMP_CONST(attrs->names[i], "nvpair") == 0) {
-			nvpair = (struct ha_msg*)attrs->values[i];
-			name_nvpair = ha_msg_value(nvpair, "name");
+
+	xml_child_iter_filter(attrs, nvpair, "nvpair",
+			name_nvpair = crm_element_value(nvpair, "name");
 			if ( strncmp(name_nvpair,attr,MAX_STRLEN) == 0 ) {
-				id_nvpair = ha_msg_value(nvpair,"id");
+				id_nvpair = crm_element_value(nvpair,"id");
 				if (id_nvpair != NULL) {
 					STRNCPY(id,id_nvpair,MAX_STRLEN);
 					free_data_set(data_set);
 					return;					
 				}
 			}
-		}
-	}
+	    );
+	
 	snprintf(id, MAX_STRLEN,  "%s_%s%s", rsc_id, mid, attr);
 	free_data_set(data_set);
 	return;
@@ -493,6 +485,7 @@ init_crm(int cache_cib)
 			, on_cib_connection_destroy);
 
 	reg_msg(MSG_CIB_VERSION, on_get_cib_version);
+	reg_msg(MSG_CRM_DTD, on_get_crm_dtd);
 	reg_msg(MSG_CRM_METADATA, on_get_crm_metadata);
 	reg_msg(MSG_CRM_CONFIG, on_get_crm_config);
 	reg_msg(MSG_UP_CRM_CONFIG, on_update_crm_config);
@@ -542,6 +535,8 @@ init_crm(int cache_cib)
 	reg_msg(MSG_GET_CONSTRAINT, on_get_constraint);
 	reg_msg(MSG_DEL_CONSTRAINT, on_delete_constraint);
 	reg_msg(MSG_UP_CONSTRAINT, on_update_constraint);
+
+	reg_msg(MSG_GEN_CLUSTER_REPORT, on_gen_cluster_report);
 	
 	return 0;
 }	
@@ -557,7 +552,7 @@ final_crm(void)
 
 /* event handler */
 void
-on_cib_diff(const char *event, HA_Message *msg)
+on_cib_diff(const char *event, crm_data_t *msg)
 {
 	if (debug_level) {
 		mgmt_debug(LOG_DEBUG,"update cib finished");
@@ -591,7 +586,7 @@ on_get_cib_version(char* argv[], int argc)
 	char* ret;
 	
 	data_set = get_data_set();
-	version = ha_msg_value(data_set->input, "num_updates");
+	version = crm_element_value(data_set->input, "num_updates");
 	if (version != NULL) {
 		ret = cl_strdup(MSG_OK);
 		ret = mgmt_msg_append(ret, version);
@@ -603,6 +598,39 @@ on_get_cib_version(char* argv[], int argc)
 	return ret;
 }
 
+static char*
+on_get_crm_dtd(char* argv[], int argc)
+{
+	const char *dtd_file = HA_NOARCHDATAHBDIR"/crm.dtd";
+	char buf[MAX_STRLEN];	
+	char* ret = cl_strdup(MSG_OK);
+	FILE *fstream = NULL;
+
+	ARGC_CHECK(1);
+
+	if ((fstream = fopen(dtd_file, "r")) == NULL){
+		mgmt_log(LOG_ERR, "error on fopen %s: %s",
+			 dtd_file, strerror(errno));
+		return cl_strdup(MSG_FAIL);
+	}
+
+	while (!feof(fstream)){
+		memset(buf, 0, sizeof(buf));
+		if (fgets(buf, sizeof(buf), fstream) != NULL){
+			ret = mgmt_msg_append(ret, buf);
+			ret[strlen(ret)-1] = '\0';
+		}
+		else{
+			sleep(1);
+		}
+	}
+
+	if (fclose(fstream) == -1)
+		mgmt_log(LOG_WARNING, "failed to fclose stream");
+
+	return ret;
+}
+
 static char*
 on_get_crm_metadata(char* argv[], int argc)
 {
@@ -613,6 +641,11 @@ on_get_crm_metadata(char* argv[], int argc)
 
 	ARGC_CHECK(2);
 
+	if (STRNCMP_CONST(argv[1], "pengine") != 0 &&
+			STRNCMP_CONST(argv[1], "crmd") != 0) {
+		return cl_strdup(MSG_FAIL);
+	}
+
 	snprintf(cmd, sizeof(cmd), BIN_DIR"/%s metadata", argv[1]);
 	if ((fstream = popen(cmd, "r")) == NULL){
 		mgmt_log(LOG_ERR, "error on popen %s: %s",
@@ -624,6 +657,7 @@ on_get_crm_metadata(char* argv[], int argc)
 		memset(buf, 0, sizeof(buf));
 		if (fgets(buf, sizeof(buf), fstream) != NULL){
 			ret = mgmt_msg_append(ret, buf);
+			ret[strlen(ret)-1] = '\0';
 		}
 		else{
 			sleep(1);
@@ -691,8 +725,8 @@ on_update_crm_config(char* argv[], int argc)
 		cur = find_xml_node_list(attrs, "nvpair");
 		while (cur != NULL) {
 			attr = (crm_data_t*)cur->data;
-			if(strncmp(ha_msg_value(attr,"name"),argv[1], MAX_STRLEN)==0) {
-				id = ha_msg_value(attr,"id");
+			if(strncmp(crm_element_value(attr,"name"),argv[1], MAX_STRLEN)==0) {
+				id = crm_element_value(attr,"id");
 				break;
 			}
 			cur = g_list_next(cur);
@@ -946,7 +980,7 @@ on_del_rsc(char* argv[], int argc)
 static int
 delete_lrm_rsc(IPC_Channel *crmd_channel, const char *host_uname, const char *rsc_id)
 {
-	HA_Message *cmd = NULL;
+	crm_data_t *cmd = NULL;
 	crm_data_t *msg_data = NULL;
 	crm_data_t *rsc = NULL;
 	crm_data_t *params = NULL;
@@ -973,17 +1007,17 @@ delete_lrm_rsc(IPC_Channel *crmd_channel, const char *host_uname, const char *rs
 	crm_free(key);
 
 	if(send_ipc_message(crmd_channel, cmd)) {
-		crm_msg_del(cmd);
+		free_xml(cmd);
 		return 0;
 	}
-	crm_msg_del(cmd);
+	free_xml(cmd);
 	return -1;
 }
 
 static int
 refresh_lrm(IPC_Channel *crmd_channel, const char *host_uname)
 {
-	HA_Message *cmd = NULL;
+	crm_data_t *cmd = NULL;
 	char our_pid[11];
 	
 	snprintf(our_pid, 10, "%d", getpid());
@@ -993,10 +1027,10 @@ refresh_lrm(IPC_Channel *crmd_channel, const char *host_uname)
 			     CRM_SYSTEM_CRMD, client_name, our_pid);
 	
 	if(send_ipc_message(crmd_channel, cmd)) {
-		crm_msg_del(cmd);
+		free_xml(cmd);
 		return 0;
 	}
-	crm_msg_del(cmd);
+	free_xml(cmd);
 	return -1;
 }
 
@@ -1007,8 +1041,10 @@ on_cleanup_rsc(char* argv[], int argc)
 	char our_pid[11];
 	char *now_s = NULL;
 	time_t now = time(NULL);
+	char *dest_node = NULL;
+	int rc;
 	
-	ARGC_CHECK(2);
+	ARGC_CHECK(3);
 	snprintf(our_pid, 10, "%d", getpid());
 	our_pid[10] = '\0';
 	
@@ -1016,11 +1052,20 @@ on_cleanup_rsc(char* argv[], int argc)
 				    NULL, &crmd_channel);
 
 	send_hello_message(crmd_channel, our_pid, client_name, "0", "1");
-	delete_lrm_rsc(crmd_channel, NULL, argv[1]);
+	delete_lrm_rsc(crmd_channel, argv[1], argv[2]);
 	refresh_lrm(crmd_channel, NULL); 
-
+	
+	rc = query_node_uuid(cib_conn, argv[1], &dest_node);
+	if (rc != cib_ok) {
+		mgmt_log(LOG_WARNING, "Could not map uname=%s to a UUID: %s\n",
+				argv[1], cib_error2string(rc));
+	} else {
+		delete_attr(cib_conn, cib_sync_call, XML_CIB_TAG_STATUS, dest_node, NULL,
+				NULL, crm_concat("fail-count", argv[2], '-'), NULL, FALSE);
+		mgmt_log(LOG_INFO, "Delete fail-count for %s from %s", argv[2], argv[1]);
+	}
 	/* force the TE to start a transition */
-	sleep(5); /* wait for the refresh */
+	sleep(2); /* wait for the refresh */
 	now_s = crm_itoa(now);
 	update_attr(cib_conn, cib_sync_call,
 		    XML_CIB_TAG_CRMCONFIG, NULL, NULL, NULL, "last-lrm-refresh", now_s, FALSE);
@@ -1194,7 +1239,7 @@ on_add_rsc(char* argv[], int argc)
 }
 
 int
-cl_msg_swap_offset(struct ha_msg* msg, int offset1, int offset2)
+cl_msg_swap_offset(crm_data_t* msg, int offset1, int offset2)
 {
 	char* name;
 	int nlen;
@@ -1226,11 +1271,10 @@ cl_msg_swap_offset(struct ha_msg* msg, int offset1, int offset2)
 char*
 on_move_rsc(char* argv[], int argc)
 {
-	int i, rc, pos = -1;
+	int i = 0, rc, pos = -1;
 	int first_child = -1;
 	int last_child = -1;
 	const char* child_id;
-	struct ha_msg* child;
 	resource_t* rsc;
 	resource_t* parent;
 	pe_working_set_t* data_set;
@@ -1243,21 +1287,23 @@ on_move_rsc(char* argv[], int argc)
 		free_data_set(data_set);
 		return cl_strdup(MSG_FAIL);
 	}
-	for (i=0; i < parent->xml->nfields ; i++){
-		if (STRNCMP_CONST(parent->xml->names[i], "primitive")!=0) {
-			continue;
-		}
-		child = (struct ha_msg*)parent->xml->values[i];
-		if (first_child == -1) {
+	xml_child_iter(
+		parent->xml, child,
+		const char *name = crm_element_name(child);
+		if (STRNCMP_CONST(name, "primitive") == 0) {
+		    if (first_child == -1) {
 			first_child = i;
-		}
-		last_child = i;
-		child_id = ha_msg_value(child,"id");
-		if (strcmp(child_id, argv[1]) == 0) {
-			mgmt_log(LOG_INFO,"find %s !",child_id);
+		    }
+		    last_child = i;
+		    child_id = crm_element_value(child, "id");
+		    if (strcmp(child_id, argv[1]) == 0) {
+			mgmt_log(LOG_INFO,"find %s !", child_id);
 			pos = i;
+		    }
 		}
-	}	
+		i++;
+	    );
+	
 	if (STRNCMP_CONST(argv[2],"up")==0) {
 		if (pos-1<first_child) {
 			free_data_set(data_set);
@@ -1276,12 +1322,13 @@ on_move_rsc(char* argv[], int argc)
 		free_data_set(data_set);
 		return cl_strdup(MSG_FAIL);
 	}
-	mgmt_log(LOG_INFO, "on_move_rsc:%s",dump_xml_formatted(parent->xml));
-	free_data_set(data_set);
+	mgmt_log(LOG_INFO, "on_move_rsc:%s", dump_xml_formatted(parent->xml)); /* Memory leak! */
 	
 	rc = cib_conn->cmds->variant_op(
 			cib_conn, CIB_OP_REPLACE, NULL,"resources",
 			parent->xml, &output, cib_sync_call);
+
+	free_data_set(data_set);
 	
 	if (rc < 0) {
 		return crm_failed_msg(output, rc);
@@ -1357,48 +1404,48 @@ on_get_rsc_attrs(char* argv[], int argc)
 	resource_t* rsc;
 	char* ret;
 	const char* value;
-	struct ha_msg* attrs;
+	crm_data_t * attrs;
 	pe_working_set_t* data_set;
 	
 	data_set = get_data_set();
 	GET_RESOURCE()
 
 	ret = cl_strdup(MSG_OK);
-	attrs = (struct ha_msg*)rsc->xml;
-	ret = mgmt_msg_append(ret, ha_msg_value(attrs, "id"));
-	ret = mgmt_msg_append(ret, ha_msg_value(attrs, "description"));
+	attrs = rsc->xml;
+	ret = mgmt_msg_append(ret, crm_element_value(attrs, "id"));
+	ret = mgmt_msg_append(ret, crm_element_value(attrs, "description"));
 	if (rsc->variant == pe_native) {
-		ret = mgmt_msg_append(ret, ha_msg_value(attrs, "class"));
-		ret = mgmt_msg_append(ret, ha_msg_value(attrs, "provider"));
-		ret = mgmt_msg_append(ret, ha_msg_value(attrs, "type"));
+		ret = mgmt_msg_append(ret, crm_element_value(attrs, "class"));
+		ret = mgmt_msg_append(ret, crm_element_value(attrs, "provider"));
+		ret = mgmt_msg_append(ret, crm_element_value(attrs, "type"));
 	}
-	value = ha_msg_value(attrs, "is_managed");
+	value = crm_element_value(attrs, "is_managed");
 	ret = mgmt_msg_append(ret, value?value:"#default");
-	value = ha_msg_value(attrs, "restart_type");
+	value = crm_element_value(attrs, "restart_type");
 	ret = mgmt_msg_append(ret, value?value:"#default");
-	value = ha_msg_value(attrs, "multiple_active");
+	value = crm_element_value(attrs, "multiple_active");
 	ret = mgmt_msg_append(ret, value?value:"#default");
-	value = ha_msg_value(attrs, "resource_stickiness");
+	value = crm_element_value(attrs, "resource_stickiness");
 	ret = mgmt_msg_append(ret, value?value:"#default");
-	value = ha_msg_value(attrs, "resource_failure_stickiness");
+	value = crm_element_value(attrs, "resource_failure_stickiness");
 	ret = mgmt_msg_append(ret, value?value:"#default");
 	
 	switch (rsc->variant) {
 		case pe_group:
-			value = ha_msg_value(attrs, "ordered");
+			value = crm_element_value(attrs, "ordered");
 			ret = mgmt_msg_append(ret, value?value:"#default");
-			value = ha_msg_value(attrs, "collocated");
+			value = crm_element_value(attrs, "collocated");
 			ret = mgmt_msg_append(ret, value?value:"#default");
 			break;
 		case pe_clone:
 		case pe_master:
-			value = ha_msg_value(attrs, "notify");
+			value = crm_element_value(attrs, "notify");
 			ret = mgmt_msg_append(ret, value?value:"#default");
-			value = ha_msg_value(attrs, "globally_unique");
+			value = crm_element_value(attrs, "globally_unique");
 			ret = mgmt_msg_append(ret, value?value:"#default");
-			value = ha_msg_value(attrs, "ordered");
+			value = crm_element_value(attrs, "ordered");
 			ret = mgmt_msg_append(ret, value?value:"#default");
-			value = ha_msg_value(attrs, "interleave");
+			value = crm_element_value(attrs, "interleave");
 			ret = mgmt_msg_append(ret, value?value:"#default");
 			break;
 		default:
@@ -1543,35 +1590,31 @@ on_get_sub_rsc(char* argv[], int argc)
 char*
 on_get_rsc_metaattrs(char* argv[], int argc)
 {
-	int i;
 	resource_t* rsc;
 	char* ret;
-	struct ha_msg* attrs;
-	struct ha_msg* nvpair;
+	crm_data_t * attrs;
 	pe_working_set_t* data_set;
 	
 	data_set = get_data_set();
 	GET_RESOURCE()
 
 	ret = cl_strdup(MSG_OK);
-	attrs = cl_get_struct((struct ha_msg*)rsc->xml, "meta_attributes");
+	attrs = find_entity(rsc->xml, "meta_attributes", NULL);
 	if(attrs == NULL) {
 		free_data_set(data_set);
 		return ret;
 	}
-	attrs = cl_get_struct(attrs, "attributes");
+	attrs = find_entity(attrs, "attributes", NULL);
 	if(attrs == NULL) {
 		free_data_set(data_set);
 		return ret;
 	}
-	for (i = 0; i < attrs->nfields; i++) {
-		if (STRNCMP_CONST(attrs->names[i], "nvpair") == 0) {
-			nvpair = (struct ha_msg*)attrs->values[i];
-			ret = mgmt_msg_append(ret, ha_msg_value(nvpair, "id"));
-			ret = mgmt_msg_append(ret, ha_msg_value(nvpair, "name"));
-			ret = mgmt_msg_append(ret, ha_msg_value(nvpair, "value"));
-		}
-	}
+	xml_child_iter_filter(attrs, nvpair, "nvpair",
+			ret = mgmt_msg_append(ret, crm_element_value(nvpair, "id"));
+			ret = mgmt_msg_append(ret, crm_element_value(nvpair, "name"));
+			ret = mgmt_msg_append(ret, crm_element_value(nvpair, "value"));
+	    );
+	
 	free_data_set(data_set);
 	return ret;
 }
@@ -1580,35 +1623,30 @@ on_get_rsc_metaattrs(char* argv[], int argc)
 char*
 on_get_rsc_params(char* argv[], int argc)
 {
-	int i;
 	resource_t* rsc;
 	char* ret;
-	struct ha_msg* attrs;
-	struct ha_msg* nvpair;
+	crm_data_t * attrs;
 	pe_working_set_t* data_set;
 	
 	data_set = get_data_set();
 	GET_RESOURCE()
 
 	ret = cl_strdup(MSG_OK);
-	attrs = cl_get_struct((struct ha_msg*)rsc->xml, "instance_attributes");
+	attrs = find_entity(rsc->xml, "instance_attributes", NULL);
 	if(attrs == NULL) {
 		free_data_set(data_set);
 		return ret;
 	}
-	attrs = cl_get_struct(attrs, "attributes");
+	attrs = find_entity(attrs, "attributes", NULL);
 	if(attrs == NULL) {
 		free_data_set(data_set);
 		return ret;
 	}
-	for (i = 0; i < attrs->nfields; i++) {
-		if (STRNCMP_CONST(attrs->names[i], "nvpair") == 0) {
-			nvpair = (struct ha_msg*)attrs->values[i];
-			ret = mgmt_msg_append(ret, ha_msg_value(nvpair, "id"));
-			ret = mgmt_msg_append(ret, ha_msg_value(nvpair, "name"));
-			ret = mgmt_msg_append(ret, ha_msg_value(nvpair, "value"));
-		}
-	}
+	xml_child_iter_filter(attrs, nvpair, "nvpair",
+			ret = mgmt_msg_append(ret, crm_element_value(nvpair, "id"));
+			ret = mgmt_msg_append(ret, crm_element_value(nvpair, "name"));
+			ret = mgmt_msg_append(ret, crm_element_value(nvpair, "value"));
+	    );
 	free_data_set(data_set);
 	return ret;
 }
@@ -1875,34 +1913,27 @@ on_set_target_role(char* argv[], int argc)
 char*
 on_get_rsc_ops(char* argv[], int argc)
 {
-	int i;
 	resource_t* rsc;
 	char* ret;
-	struct ha_msg* ops;
-	struct ha_msg* op;
+	crm_data_t * ops;
 	pe_working_set_t* data_set;
 	
 	data_set = get_data_set();
 	GET_RESOURCE()
 
 	ret = cl_strdup(MSG_OK);
-	ops = cl_get_struct((struct ha_msg*)rsc->xml, "operations");
+	ops = find_entity(rsc->xml, "operations", NULL);
 	if (ops == NULL) {
 		free_data_set(data_set);
 		return ret;
 	}
-	for (i = 0; i < ops->nfields; i++) {
-		if (STRNCMP_CONST(ops->names[i], "op") == 0) {
-			if (ops->types[i] != FT_STRUCT) {
-				continue;
-			}
-			op = (struct ha_msg*)ops->values[i];
-			ret = mgmt_msg_append(ret, ha_msg_value(op, "id"));
-			ret = mgmt_msg_append(ret, ha_msg_value(op, "name"));
-			ret = mgmt_msg_append(ret, ha_msg_value(op, "interval"));
-			ret = mgmt_msg_append(ret, ha_msg_value(op, "timeout"));
-		}
-	}
+	xml_child_iter_filter(ops, op, "op",
+			ret = mgmt_msg_append(ret, crm_element_value(op, "id"));
+			ret = mgmt_msg_append(ret, crm_element_value(op, "name"));
+			ret = mgmt_msg_append(ret, crm_element_value(op, "interval"));
+			ret = mgmt_msg_append(ret, crm_element_value(op, "timeout"));
+	    );
+	
 	free_data_set(data_set);
 	return ret;
 }
@@ -1922,11 +1953,10 @@ on_get_rsc_ops(char* argv[], int argc)
 char*
 on_get_rsc_full_ops(char* argv[], int argc)
 {
-	int i;
+	const char *value;
 	resource_t* rsc;
 	char* ret;
-	struct ha_msg* ops;
-	struct ha_msg* op;
+	crm_data_t * ops;
 	pe_working_set_t* data_set;
 	
 	data_set = get_data_set();
@@ -1934,33 +1964,28 @@ on_get_rsc_full_ops(char* argv[], int argc)
 
 	ret = cl_strdup(MSG_OK);
 	ret = mgmt_msg_append(ret, "10");
-	ops = cl_get_struct((struct ha_msg*)rsc->xml, "operations");
+	ops = find_entity(rsc->xml, "operations", NULL);
 	if (ops == NULL) {
 		free_data_set(data_set);
 		return ret;
 	}
-	for (i = 0; i < ops->nfields; i++) {
-		if (STRNCMP_CONST(ops->names[i], "op") == 0) {
-			const char* value = NULL;
-			if (ops->types[i] != FT_STRUCT) {
-				continue;
-			}
-			op = (struct ha_msg*)ops->values[i];
-			ret = mgmt_msg_append(ret, ha_msg_value(op, "id"));
-			ret = mgmt_msg_append(ret, ha_msg_value(op, "name"));
-			ret = mgmt_msg_append(ret, ha_msg_value(op, "description"));
-			ret = mgmt_msg_append(ret, ha_msg_value(op, "interval"));
-			ret = mgmt_msg_append(ret, ha_msg_value(op, "timeout"));
-			value = ha_msg_value(op, "start_delay");
+
+	xml_child_iter_filter(ops, op, "op",
+			ret = mgmt_msg_append(ret, crm_element_value(op, "id"));
+			ret = mgmt_msg_append(ret, crm_element_value(op, "name"));
+			ret = mgmt_msg_append(ret, crm_element_value(op, "description"));
+			ret = mgmt_msg_append(ret, crm_element_value(op, "interval"));
+			ret = mgmt_msg_append(ret, crm_element_value(op, "timeout"));
+			value = crm_element_value(op, "start_delay");
 			ret = mgmt_msg_append(ret, value==NULL?"0":value);
-			value = ha_msg_value(op, "disabled");
+			value = crm_element_value(op, "disabled");
 			ret = mgmt_msg_append(ret, value==NULL?"false":value);
-			value = ha_msg_value(op, "role");
+			value = crm_element_value(op, "role");
 			ret = mgmt_msg_append(ret, value==NULL?"Started":value);
-			ret = mgmt_msg_append(ret, ha_msg_value(op, "prereq"));
-			ret = mgmt_msg_append(ret, ha_msg_value(op, "on_fail"));
-		}
-	}
+			ret = mgmt_msg_append(ret, crm_element_value(op, "prereq"));
+			ret = mgmt_msg_append(ret, crm_element_value(op, "on_fail"));
+	    );
+
 	free_data_set(data_set);
 	return ret;
 }
@@ -2258,7 +2283,7 @@ on_get_constraints(char* argv[], int argc)
 	cur = list;
 	while (cur != NULL) {
 		crm_data_t* location = (crm_data_t*)cur->data;
-		ret = mgmt_msg_append(ret, ha_msg_value(location, "id"));
+		ret = mgmt_msg_append(ret, crm_element_value(location, "id"));
 		
 		cur = g_list_next(cur);
 	}
@@ -2279,7 +2304,7 @@ on_get_constraint(char* argv[], int argc)
 	crm_data_t* cos = NULL;
 	pe_working_set_t* data_set;
 	const char* path[] = {"configuration","constraints"};
-	ARGC_CHECK(3); 
+	int i;
 	
 	data_set = get_data_set();
 	cos = find_xml_node_nested(data_set->input, path, 2);
@@ -2292,36 +2317,30 @@ on_get_constraint(char* argv[], int argc)
 	cur = list;
 	while (cur != NULL) {
 		crm_data_t* constraint = (crm_data_t*)cur->data;
-		if (strncmp(argv[2],ha_msg_value(constraint, "id"), MAX_STRLEN)==0) {
+		if (strncmp(argv[2],crm_element_value(constraint, "id"), MAX_STRLEN)==0) {
 			if (STRNCMP_CONST(argv[1],"rsc_location")==0) {
-				ret = mgmt_msg_append(ret, ha_msg_value(constraint, "id"));
-				ret = mgmt_msg_append(ret, ha_msg_value(constraint, "rsc"));
+				ret = mgmt_msg_append(ret, crm_element_value(constraint, "id"));
+				ret = mgmt_msg_append(ret, crm_element_value(constraint, "rsc"));
 				rule = find_xml_node(constraint,"rule",TRUE);
-				ret = mgmt_msg_append(ret, ha_msg_value(rule, "score"));
-				ret = mgmt_msg_append(ret, ha_msg_value(rule, "boolean_op"));
+				ret = mgmt_msg_append(ret, crm_element_value(rule, "score"));
+				ret = mgmt_msg_append(ret, crm_element_value(rule, "boolean_op"));
 				expr_list = find_xml_node_list(rule, "expression");
 				expr_cur = expr_list;
 				while(expr_cur) {
 					crm_data_t* expr = (crm_data_t*)expr_cur->data;
-					ret = mgmt_msg_append(ret, ha_msg_value(expr, "id"));
-					ret = mgmt_msg_append(ret, ha_msg_value(expr, "attribute"));
-					ret = mgmt_msg_append(ret, ha_msg_value(expr, "operation"));
-					ret = mgmt_msg_append(ret, ha_msg_value(expr, "value"));
+					ret = mgmt_msg_append(ret, crm_element_value(expr, "id"));
+					ret = mgmt_msg_append(ret, crm_element_value(expr, "attribute"));
+					ret = mgmt_msg_append(ret, crm_element_value(expr, "operation"));
+					ret = mgmt_msg_append(ret, crm_element_value(expr, "value"));
 					expr_cur = g_list_next(expr_cur);
 				}
 				g_list_free(expr_list);
 			}
-			else if (STRNCMP_CONST(argv[1],"rsc_order")==0) {
-				ret = mgmt_msg_append(ret, ha_msg_value(constraint, "id"));
-				ret = mgmt_msg_append(ret, ha_msg_value(constraint, "from"));
-				ret = mgmt_msg_append(ret, ha_msg_value(constraint, "type"));
-				ret = mgmt_msg_append(ret, ha_msg_value(constraint, "to"));
-			}
-			else if (STRNCMP_CONST(argv[1],"rsc_colocation")==0) {
-				ret = mgmt_msg_append(ret, ha_msg_value(constraint, "id"));
-				ret = mgmt_msg_append(ret, ha_msg_value(constraint, "from"));
-				ret = mgmt_msg_append(ret, ha_msg_value(constraint, "to"));
-				ret = mgmt_msg_append(ret, ha_msg_value(constraint, "score"));
+			else if (STRNCMP_CONST(argv[1],"rsc_order")==0 ||
+					STRNCMP_CONST(argv[1],"rsc_colocation")==0) {
+				for(i = 3; i < argc; i++){
+					ret = mgmt_msg_append(ret, crm_element_value(constraint, argv[i]));
+				}
 			}
 			break;
 		}
@@ -2353,6 +2372,7 @@ on_update_constraint(char* argv[], int argc)
 	crm_data_t* output;
 	int i;
 	char xml[MAX_STRLEN];
+	char buf[MAX_STRLEN];
 
 	if (STRNCMP_CONST(argv[1],"rsc_location")==0) {
 		snprintf(xml, MAX_STRLEN,
@@ -2369,15 +2389,14 @@ on_update_constraint(char* argv[], int argc)
 		strncat(xml, "</rule></rsc_location>",
 				sizeof(xml)-strlen(xml)-1);
 	}
-	else if (STRNCMP_CONST(argv[1],"rsc_order")==0) {
-		snprintf(xml, MAX_STRLEN,
-			 "<rsc_order id=\"%s\" from=\"%s\" type=\"%s\" to=\"%s\"/>",
-			 argv[2], argv[3], argv[4], argv[5]);
-	}
-	else if (STRNCMP_CONST(argv[1],"rsc_colocation")==0) {
-		snprintf(xml, MAX_STRLEN,
-			 "<rsc_colocation id=\"%s\" from=\"%s\" to=\"%s\" score=\"%s\"/>",
-			 argv[2], argv[3], argv[4], argv[5]);
+	else if (STRNCMP_CONST(argv[1],"rsc_order")==0 ||
+			STRNCMP_CONST(argv[1],"rsc_colocation")==0) {
+		snprintf(xml, MAX_STRLEN, "<%s", argv[1]);
+		for (i = 2; i < argc-1; i += 2){
+			snprintf(buf, MAX_STRLEN, " %s=\"%s\"", argv[i], argv[i+1]);
+			strncat(xml, buf, sizeof(xml)-strlen(xml)-1);
+		}
+		strncat(xml, " />", sizeof(xml)-strlen(xml)-1);
 	}
 	cib_object = string2xml(xml);
 	if(cib_object == NULL) {
@@ -2398,3 +2417,96 @@ on_update_constraint(char* argv[], int argc)
 	return cl_strdup(MSG_OK);
 }
 
+static char*
+on_gen_cluster_report(char* argv[], int argc)
+{
+	char cmd[MAX_STRLEN];
+	char buf[MAX_STRLEN];
+	char filename[MAX_STRLEN];
+	char *dest = tempnam("/tmp", "clrp.");
+	char* ret = cl_strdup(MSG_OK);
+	FILE *fstream = NULL;
+	const char* date_regex = \
+		"[0-9]{4}-[0-1][0-9]-[0-3][0-9] [0-2][0-9]:[0-5][0-9]:[0-5][0-9]";
+
+	ARGC_CHECK(3);
+
+	if (regex_match(date_regex, argv[1])) {
+		snprintf(buf, sizeof(buf), "-f \"%s\"", argv[1]);
+	}
+	else {
+		mgmt_log(LOG_ERR, "invalid \"from\" date expression: \"%s\"", argv[1]);
+		free(dest);
+		return cl_strdup(MSG_FAIL);
+	}
+
+	if (strnlen(argv[2], MAX_STRLEN) != 0) {
+		if (regex_match(date_regex, argv[2])) {
+			strncat(buf, " -t \"", sizeof(buf)-strlen(buf)-1);
+			strncat(buf, argv[2], sizeof(buf)-strlen(buf)-1);
+			strncat(buf, "\"", sizeof(buf)-strlen(buf)-1);
+		}
+		else {
+			mgmt_log(LOG_ERR, "invalid \"to\" date expression: \"%s\"", argv[2]);
+			free(dest);
+			return cl_strdup(MSG_FAIL);
+		}
+	}
+
+	snprintf(cmd, sizeof(cmd), "hb_report -DC %s %s", buf, dest);
+	mgmt_log(LOG_INFO, "cluster_report: %s", cmd);
+	if (system(cmd) < 0) {
+		mgmt_log(LOG_ERR, "error on system %s: %s",
+			 cmd, strerror(errno));
+		free(dest);
+		return cl_strdup(MSG_FAIL);
+	}
+
+	snprintf(filename, sizeof(filename), "%s.tar.gz", dest);
+	snprintf(cmd, sizeof(cmd), "base64 %s", filename);
+	if ((fstream = popen(cmd, "r")) == NULL) {
+		mgmt_log(LOG_ERR, "error on popen %s: %s",
+			 cmd, strerror(errno));
+		unlink(filename);
+		free(dest);
+		return cl_strdup(MSG_FAIL);
+	 }
+
+	ret = mgmt_msg_append(ret, filename);
+	while (!feof(fstream)) {
+		memset(buf, 0, sizeof(buf));
+		if (fgets(buf, sizeof(buf), fstream) != NULL) {
+			ret = mgmt_msg_append(ret, buf);
+			ret[strlen(ret)-1] = '\0';
+		}
+		else {
+			sleep(1);
+		}
+	}
+	if (pclose(fstream) == -1)
+		mgmt_log(LOG_WARNING, "failed to close pipe");
+
+	unlink(filename);
+	free(dest);
+	return ret;
+}
+
+int
+regex_match(const char *regex, const char *str)
+{
+	regex_t preg;
+	int ret;
+
+	if (regcomp(&preg, regex, REG_EXTENDED|REG_NOSUB) != 0){
+		mgmt_log(LOG_ERR, "error regcomp regular expression: \"%s\"", regex);
+		return 0;
+	}
+
+	ret = regexec(&preg, str, 0, NULL, 0);
+	if (ret != 0) {
+		mgmt_log(LOG_WARNING, "no match or error regexec: \"%s\" \"%s\"", regex, str);
+	}
+
+	regfree(&preg);
+	return (ret == 0);
+}
