@@ -429,6 +429,7 @@ NodeStatus(const char * node, const char * status, void * private)
         cl_log(LOG_NOTICE, "Status update: Node %s now has status %s"
         ,       node, status);
 	walk_nodetable();
+	walk_iftable();
 
 	nodestatus_trap(node, status);
 }
@@ -1455,6 +1456,7 @@ main(int argc, char ** argv)
 
 	/* you're main loop here... */
 	while(keep_running) {
+		int	save_errno;
 	/* if you use select(), see snmp_select_info() in snmp_api(3) */
 	/*     --- OR ---  */
 		/* 0 == don't block */
@@ -1489,9 +1491,10 @@ main(int argc, char ** argv)
 		}
 
 		ret = select(numfds, &fdset, 0, 0, tvp);
+		save_errno=errno;
 
 		if (ret < 0) {
-			if (errno == EBADF) {
+			if (errno == EBADF && keep_running) {
 				struct stat	foo;
 				/* Probably the membership layer shut down for some reason. */
 				if (mem_fd >= 0 && fstat(mem_fd, &foo) < 0 && errno == EBADF) {
@@ -1506,7 +1509,11 @@ main(int argc, char ** argv)
 			}
 				
 			/* error */
-			cl_perror("select() returned an error. shutting down");
+			if (keep_running) {
+				errno = save_errno;
+				cl_perror("%s: select() returned an error, shutting down."
+				,       __FUNCTION__);
+			}
 			break;
 		} else if (ret == 0) {
 			/* timeout */
