@@ -24,6 +24,7 @@
 #ifndef __hasubagent_h__
 #define __hasubagent_h__
 
+#include <stdio.h>
 #ifdef HAVE_STDINT_H
 #include <stdint.h>
 #endif
@@ -31,9 +32,11 @@
 #include <glib.h>
 #include "saf/ais.h"
 #include "snmp-config-resolve.h"
-#include <uuid/uuid.h>
+#include <clplumbing/cl_uuid.h>
 
 #define CACHE_TIME_OUT 5
+#define LHAAGENTID "lha-snmpagent"
+#define DEFAULT_TIME_OUT 5 /* default timeout value for snmp in sec. */
 
 typedef enum lha_group {
 	LHA_CLUSTERINFO,
@@ -64,7 +67,25 @@ int clusterinfo_get_int_value(lha_attribute_t attr, size_t index, uint32_t * val
 
 int rsinfo_get_int_value(lha_attribute_t attr, size_t index, uint32_t * value);
 
-int hbconfig_get_str_value(const char * attr, char * * value);
+
+/* LHAHeartbeatConfigInfo partial-mode */
+typedef enum lha_hbconfig {
+	LHA_CONF_HBVERSION,
+	LHA_CONF_KEEPALIVE,
+	LHA_CONF_DEADTIME,
+	LHA_CONF_DEADPING,
+	LHA_CONF_WARNTIME,
+	LHA_CONF_INITDEAD,
+	LHA_CONF_BAUDRATE,
+	LHA_CONF_AUTOFAIL,
+	LHA_CONF_STONITH,
+	LHA_CONF_STONITHHOST,
+	LHA_CONF_RESPAWN,
+	LHA_CONF_END
+} lha_hbconfig_t;
+
+int hbconfig_get_str(lha_hbconfig_t attr_no, char * * value);
+
 
 GPtrArray * get_hb_info(lha_group_t group);
 
@@ -74,7 +95,7 @@ struct hb_nodeinfo {
 	uint32_t type;
 	uint32_t status;
 	size_t ifcount;
-	uuid_t uuid;
+	cl_uuid_t uuid;
 };
 
 struct hb_ifinfo {
